#!/usr/bin/perl -wT -I../../lib

=head1 NAME

callbacks/acknowledge.cgi -- Acknowledge (respond) a callback message

=head1 SYNOPSIS

    callbacks/acknowledge.cgi?table=customer_log&log_id=12345

=head1 DESCRIPTION

The callbacks/acknowledge.cgi program updates the log message and set the
respond_date to today and the respond_by to the current user.

This intended to be a quick way for users to quickly acknowledge a callback
so it won't be printed or emailed.

At this time, the following database field names which are updated are
hardwired into this script:

	respond_date	-- date
	respond_by	-- user id
	log_id		-- ID key field name

=head1 CHANGE LOG

 8/12/00  Initial version.

=head1 AUTHOR

Mack Pexton <mack@acmebase.org>

=head1 COPYRIGHT

Copyright 2000, Mackley F. Pexton.  All rights reserved.

This is open software and is licensed under the terms of the GNU General Public License.

=cut

use Acme::HTML		qw(display_message);
use Acme::Db;
use Acme::Date		qw(today);
use Local::Acme::User	qw(user);
use CGI;

use Acme::Security;
Acme::Security::check('Access');

$|=1;		# flush output (synchronize errors with output)

$cgi = new CGI;
die "No arguments\n" unless $cgi->param();

#
# Program Arguments
#

$table = $cgi->url_param('table')		|| $cgi->param('table');

die "No table specified." unless $table;

Acme::Security::check('Access,Update',$table);

#
# Find ID field from schema
#
## use Acme::Base::Schema;
## $S = new Acme::Base::Schema;
## $id_field = $S->{$table}->id;
$id_field = 'log_id';				## hardwire for efficiency

$id = $cgi->url_param($id_field)		|| $cgi->param($id_field);

$user = user();
$today = today();

$post_script = <<End_Of_SQL;
	update $table set
		respond_date = '$today',
		respond_by = '$user'
	where $id_field = $id;
End_Of_SQL

print $cgi->header(-type=>'text/html') unless @ARGV;

if (Acme::Db->execute($post_script)) {
	display_message("The log record has been acknowledged.",
			"Message",
			"");
}
else {
	# It is assumed that an error message has already been output.
	display_message("The log record has <font color='red'>NOT</font> been acknowledged.",
			"Message",'',0);
}

exit 0;
