#!/usr/bin/perl -T -I../../../lib

# Change Customer Log Information
# This is a program front end to the /templates/customer_log_response.htm.

# The HTML input fields in the pattern file must have their name=""
# parameter on the same line as the value="" parameter in order for the
# substitution of the customer information to work. The quotes are important.
# Also, only one input field code per source line.
#
# The field names just happen to correspond to database field names.

use Acme::HTML;
use Acme::HTML::ListMenus;
use Acme::Db;
use Acme::Parse		qw(parse_args);
use Acme::Date		qw(today	timestamp);
use Local::Acme::User	qw(user);
use Acme::Config;

$|=1;
print "Content-type: text/html\n\n";

my ($field_name, $field_value);

local %args;
unless (parse_args(\%args) and ($log_id = $args{log_id})) {
	print "<html><body onLoad='self.focus()'>(empty)</body></html>";
	exit 0;
}

$Templates = Acme::Config->Templates;

my $user = user();
local $today = today();
local $timestamp = timestamp();

if (open HTML, "$Templates/customer_log_response.htm") {

	# Get customer log
	my (
		$customer_id,
		$customer_name,
		$name,
		$company,
		$first_name,
		$address1,
		$address2,
		$city,
		$state,
		$zip,
		$country,
		$phone,
		$company_phone,
		$fax,
		$email,
		$customer_type,
		$ad_source,
		$original_ad_source,
		$referral,
		$comments,
		$log_id,
		$log_type,
		$log,
		$callback_date,
		$user_id,
		$message,
		$entry_date,
		$last_update,
		$last_update_by,
		$respond_log_id,
		$respond_date,
		$respond_by,
		$response_to_log_id,
	) = Acme::Db->get(<<End_of_Sql);
		select
			customers.customer_id,
			customers.customer_name,
			customers.name,
			customers.company,
			customers.first_name,
			customers.address1,
			customers.address2,
			customers.city,
			customers.state,
			customers.zip,
			customers.country,
			customers.phone,
			customers.company_phone,
			customers.fax,
			customers.email,
			customers.customer_type,
			customers.ad_source,
			customers.original_ad_source,
			customers.referral,
			customers.comments,
			customer_log.log_id,
			customer_log.log_type,
			customer_log.log,
			customer_log.callback_date,
			customer_log.user_id,
			customer_log.message,
			customer_log.entry_date,
			customer_log.last_update,
			customer_log.last_update_by,
			customer_log.respond_log_id,
			customer_log.respond_date,
			customer_log.respond_by,
			customer_log.response_to_log_id
		from customer_log, customers
		where customer_log.log_id = $log_id
		  and customer_log.customer_id = customers.customer_id
End_of_Sql

	# Get response log if there is one
	my (
		$R_log_id,
		$R_customer_id,
		$R_log_type,
		$R_log,
		$R_callback_date,
		$R_user_id,
		$R_message,
		$R_entry_date,
		$R_last_update,
		$R_last_update_by,
		$R_respond_date,
		$R_respond_by,
	) = Acme::Db->get(<<End_of_Sql) if ($respond_log_id);
		select
			customer_log.log_id,
			customer_log.customer_id,
			customer_log.log_type,
			customer_log.log,
			customer_log.callback_date,
			customer_log.user_id,
			customer_log.message,
			customer_log.entry_date,
			customer_log.last_update,
			customer_log.last_update_by,
			customer_log.respond_date,
			customer_log.respond_by	
		from customer_log
		where customer_log.log_id = $respond_log_id
End_of_Sql

	# Format original log message that is being responded to.
	my $original_log = <<End_of_HTML;
<div class="log"> $log_type
<p>$log</p>
<p>Callback was scheduled for $user_id on $callback_date.<br>$message</p>
<div class="audit">
  <table class="audit">
  <tr>
    <td class="audit-label">Log entered</td>
    <td class="audit-field">$entry_date</td>
    <td class="audit-label">changed</td>
    <td class="audit-field">$last_update</td>
    <td class="audit-label">by</td>
    <td class="audit-field">$last_update_by</td>
  </tr>
  <tr><td align=left>
  </td><td align=right>
  </td></tr>
  </table>
</div>
</div>
End_of_HTML

	# Substitute customer information into input field value strings.
	while (<HTML>) {
		# personalize window title
		s/<title>Log: Customer Log Response</<title>cc $customer_name</o
		  if ($customer_name);

		# uncheck window open option
		s/(<input type=checkbox name="keep_open".*) checked/$1/;

		# Display customer ID and name on form
		s;\(not assigned\);<b>$customer_id</b>;o;
		 
		s;\(not available\);<a href="/cgi-bin/customers/view.cgi?customer_id=$customer_id"  onClick="new_target(this,'view')" title="View customer information."><font size=+2><b>$customer_name</b></font></a>;o;

		s;<!-- ORIGINAL LOG -->;$original_log;o;

		s;<!-- RESPONSE BY -->;<b>Responded</b> $response_date by $respond_by;o
		  if ($response_date);

		# Enter information into textarea fields
		if ( ($field_name) = m/<textarea.*name\s*=\s*\"(\w+)\"/i ) {
			$field_value = eval "\$$field_name";
			$field_value = (defined $field_value) ? $field_value : "";
			s|(<textarea.*?>).*(</textarea>)|$1$field_value$2|;
		}

		# Enter information into form input field default values
		if ( ($field_name) = m/input.*name\s*=\s*\"(\w+)\".*value\s*=/i ) {
			$field_value = eval "\$$field_name";
			$field_value = defined $field_value ? $field_value : "";

			SWITCH: {
				if ($field_name eq "log_type") {
					s{<input.*name="log_type".*?>}
					 {Acme::HTML::ListMenus->log_types("log_type",$field_value)}e;
					last SWITCH;
				}
				
				if ($field_name eq "user_id") {
					$field_value = $user_id
					  unless $field_value;
					s{<input.*name="user_id".*?>}
					 {Acme::HTML::ListMenus->users($field_name,$field_value)}e;
					last SWITCH;
				}
				
				if ($field_name eq "current_user") {
					s/(value\s*=\s*")/$1$user/;
					last SWITCH;
				}

				if ($field_name eq "respond_by") {
					s/(value\s*=\s*")/$1$user/;
					last SWITCH;
				}

				# prime respond_by check box with today's date
				if ($field_name eq "respond_date") {
					if ($respond_date) {
						s/(value\s*=\s*")/$1$field_value/;
						s{(<input.*name="respond_date".*)>}
						 {$1 checked>};
					}
					else {
						s/(value\s*=\s*")/$1$today/;
					}
					last SWITCH;
				}

				if ($field_name eq "entry_date") {
					s/<input.*name="entry_date".*?>/$field_value/;
					last SWITCH;
				}

				if ($field_name eq "last_update") {
					s/<input.*name="last_update".*?>/$field_value/;
					last SWITCH;
				}

				if ($field_name eq "last_update_by") {
					s/<input.*name="last_update_by".*?>/$field_value/;
					last SWITCH;
				}
				
				#
				# Response form fields
				# Prime response field linking to log record.
				#

				if ($field_name eq "R_response_to_log_id") {
					s/(value\s*=\s*")/$1$log_id/;
					last SWITCH;
				}

				if ($field_name eq "R_customer_id") {
					$field_value = $customer_id 
					  unless ($R_customer_id);
					s/(value\s*=\s*")/$1$field_value/;
					last SWITCH;
				}

				if ($field_name eq "R_log_type") {
					$field_value = $log_type 
					  unless ($R_log_type);
					s/(value\s*=\s*")/$1$field_value/;
					last SWITCH;
				}

				if ($field_name eq "R_user_id") {
					$field_value = $user
					  unless ($field_value);
					s{(<input.*name="R_user_id".*?>)}
					 {Acme::HTML::ListMenus->users($field_name,$field_value)}e;
					last SWITCH;
				}
				
				if ($field_name eq "R_current_user") {
					s/(value\s*=\s*")/$1$user/;
					last SWITCH;
				}

				# Enter response record audit info
				if ($field_name eq "R_entry_date") {
					$field_value = $today
					  unless ($field_value);
					s/<input.*name="R_entry_date".*?>/$field_value/;
					last SWITCH;
				}
				if ($field_name eq "R_last_update") {
					$field_value = $timestamp
					  unless ($field_value);
					s/<input.*name="R_last_update".*?>/$field_value/;
					last SWITCH;
				}
				if ($field_name eq "R_last_update_by") {
					$field_value = $user
					  unless ($field_value);
					s/<input.*name="R_last_update_by".*?>/$user/;
					last SWITCH;
				}
				
				s/(value\s*=\s*")/$1$field_value/;
			}
		}

		print;
	}
	close HTML;
}
exit 0;
