#!/usr/bin/perl -wT -I../lib
require 5.6.0;
use strict;

=head1 NAME

form.cgi -- Enter and display database information in a form.

=head1 SYNOPSIS

    form.cgi?form=customers

    form.cgi?table=customers&search=Folly

    form.cgi?template=orders.html&request=new

=head1 DESCRIPTION

The form.cgi program displays a form for entering and changing database
information. A form template can be specified or a form can be built from 
the database schema definition or a form specification (which is a copy of
the schema definition).

The search parameter is used to determine which record to edit.

The C<form> parameter first looks to see if a form specification file exists,
and if not, looks for a template file with the same name. If a template file
does not exist, then a form for a database table with the form's name is built.

=head1 CHANGE LOG

 5/8/00  Initial version.
 5/2/01  Use new versions of Acme::Base::Schema and Form modules.
 7/28/01 Added form specification files.
 3/17/02 Generalized C<form> parameter to look for templates and schema tables.

=head1 AUTHOR

Mack Pexton <mack@acmebase.org>

=head1 COPYRIGHT

Copyright 2000,2001, Mackley F. Pexton.  All rights reserved.

This is open software and is licensed under the terms of the GNU General Public License.

=cut

use Acme::HTML;
use Acme::Form;
use CGI;

use Acme::Security;
Acme::Security::check('Access');

$|=1;		# flush output (synchronize errors with output)

our $cgi = new CGI;
$cgi->cache('no');
print $cgi->header(-type=>'text/html', -expires=>'-1d') unless @ARGV;

our $request  = $cgi->param('request');

our $title    = $cgi->param('title');

our $template = $cgi->url_param('template')	|| $cgi->param('template');

our $form     = $cgi->url_param('form')		|| $cgi->param('form');

our $table    = $cgi->url_param('table')	|| $cgi->param('table');

our $search   = $cgi->param('search');

our $save_history = $cgi->param('save_history');

if ($save_history && $save_history =~ /yes/i) {
	my $hist = $form || $template || $table;
	if ($search && $hist) {

		#
		# Save search in user's history list
		#

		require Acme::Options;
		Acme::Options->new->enter("${hist}Search",$search);
	}
	$cgi->delete('save_history');
}

my $f = new Acme::Form (
		request  => $request,
		args     => $cgi,
		title    => $title,
		table    => $table,
		form     => $form,
		template => $template,
		);

print $f->as_html;

exit;
