#!/usr/bin/perl -wT -I../lib

=head1 NAME

help.cgi -- Display help screens

=head1 SYNOPSIS

    help.cgi?file=customer_help.html
      or
    help.cgi?template=customer_help.html

=head1 DESCRIPTION

The help.cgi program is a front end for help messages. Currently, it is 
primarily a stub, it only fills in the page headers on help files.

=head1 CHANGE LOG

 8/6/00   Initial version.

=head1 AUTHOR

Mack Pexton <mack@acmebase.org>

=head1 COPYRIGHT

Copyright 2000, Mackley F. Pexton.  All rights reserved.

This is open software and is licensed under the terms of the GNU General Public License.

=cut

$|=1;

use Acme::HTML;
use Acme::HTML::NavButtons;
use Acme::Config;
use Local::Acme::Banners	qw(common_header common_footer);
use CGI;

use Acme::Security;
Acme::Security::check('Access');

#
# Begin processing form
#

$|=1;		# flush output (synchronize errors with output)

$cgi = new CGI;
$cgi->cache('no');
print $cgi->header(-type=>'text/html') unless @ARGV;

#
# Parse cgi arguments
#

$Template = $cgi->url_param('template') || $cgi->param('template');
$Template = $cgi->url_param('form') || $cgi->param('form')
  unless $Template; 		# alternative template keyword
$Template = $cgi->url_param('file') || $cgi->param('file')
  unless $Template; 		# alternative template keyword

# Make full path out of template name
$Template = Acme::Config->DocumentRoot
  . ($Template =~ m|^/| ? '' : '/')
  . $Template
  if defined $Template;

if ($Template) {

	#
	# Read form template
	#

	open(TEMPLATE, "<$Template")
	  or die "Cannot open form template '$Template'. $!.\n";
	local $/;
	$Template = <TEMPLATE>;
	close(TEMPLATE);
}
else {
	die "No help file was specified";
}

#
# Attach local header and footer
#

$Template =~ s;<!--.*?\bHeader\b.*?-->;common_header();e;
$Template =~ s;<!--.*?\bFooter\b.*?-->;common_footer();e;

#
# Attach navigation buttons.
#

$b = new Acme::HTML::NavButtons;
$Template =~ s;(<body\b.*?>);$1 . "\n" . $b->buttons();e;	# first buttons
$Template =~ s;<!--.*\bNav.*Buttons\b.*-->;$b->buttons();e;
$Template =~ s;(</body);$b->last_buttons() . "\n" . $1;e;	# last buttons

print $Template;

exit;
