#!/usr/bin/perl -wT -I../lib

=head1 NAME

mkschema.cgi -- make schema

=head1 SYNOPSIS

    mkschema.cgi [ table ... ]

=head1 DESCRIPTION

The mkschema.cgi program searches the schema and prints SQL DML statements
which can be used to create database tables.

This version is intended to be viewed from the Web and is for information only.
Refer to C<mkschema> command, which is intended to be run from the command line.
Often the output from C<mkschema> is piped directly into the database program.
As an example, for Postgres:

	mkschema.cgi customers | psql acme

=head1 CHANGE LOG

 3/24/01  Initial version.

=head1 AUTHOR

Mack Pexton <mack@acmebase.org>

=head1 COPYRIGHT

Copyright 2001, Mackley F. Pexton.  All rights reserved.

This is open software and is licensed under the terms of the GNU General Public License.

=cut

$|=1;

# Print the SQL code to create tables and indexes.

use Acme::HTML;
use Acme::Db::Schema	qw(print_ddl);

use Acme::Security;
Acme::Security::check('Access');

# Check for existance of QUERY_STRING to determine if this program was
# executed from the Web server or from the command line.

if (defined $ENV{QUERY_STRING}) {
	# program called from web server
	use CGI;
	my $cgi = new CGI;

	print $cgi->header(-type=>'text/html');
	print <<EOF;
<html>
<title>DDL</title>
<h1>Define Database Tables</h1>
<pre>
EOF
	if ($cgi->param('table')) {
		print_ddl($cgi->param('table'));
	}
	elsif (my $k = $cgi->param('keywords')) {
		# cgi arguments like '?customers&customer_info'
		print_ddl((split(/\&/,$k)));
	}
	else {
		# cgi arguments like '?customers=&customer_info='
		print_ddl($cgi->param);
	}

	print <<EOF;
</pre>
</body>
</html>
EOF
}
else {
	# program called from shell
	print_ddl(@ARGV);
}

exit;
