#!/usr/bin/perl -wT -I../lib

=head1 NAME

options.cgi --  Display and save user options

=head1 SYNOPSIS

    options.cgi

    options.cgi?request=save

=head1 DESCRIPTION

This program display's a form of user options and saves them to a file. Options
can be stored in a user's home directory or in a common space reserved for
database users, depending upon the settings in the site configuration file
(.../etc/site.conf).

The CGI module is used to format and save the options in a text file.
The name is set in the site configuration file.

=head1 CHANGE LOG

 4/98    Initial version
 11/99   Change to use CGI.
 1/22/02 Process Search_Tables structure defined in site configuration file.

=head1 AUTHOR

Mack Pexton <mack@acmebase.org>

=head1 COPYRIGHT

Copyright(c) 1998-2002  Mackley F. Pexton.  All rights reserved.

This is open software and is licensed under the terms of the GNU General Public License.

=cut

use Acme::HTML				qw(display_message);
use Acme::HTML::Tag;
use Acme::HTML::ListMenus;
use Acme::Options;
use Acme::Base::Schema;
use Acme::DailyPics			qw(next_pic	prev_pic);
use Local::Acme::User			qw(user		user_name);
use Acme::Config;
use Local::Acme::Form::Extensions	qw(tag_command);
use CGI;

use Acme::Security;
Acme::Security::check('Access');

$|=1;
$cgi = new CGI;
$cgi->cache('no');
print $cgi->header(-type=>'text/html', -expires=>'-1d') unless @ARGV;

local $Templates = Acme::Config->Templates;
local $Images    = Acme::Config->Images;

local $opt = new Acme::Options;

$request   = $cgi->param('request') || '';
$cgi->delete('request');

if ($request =~ /save/i) {

	# Set "off" values for on/off option settings.
	# Note: browsers do not pass back field settings for checkbox
	# fields if they are unchecked.

	$cgi->param('keep_open','no')
	  if ! $cgi->param('keep_open');

	$cgi->param('DisplayDailyPicture','no')
	  if ! $cgi->param('DisplayDailyPicture');
	$cgi->param('RandomDailyPicture','no')
	  if ! $cgi->param('RandomDailyPicture');
	$cgi->param('RandomDailyFortune','no')
	  if ! $cgi->param('RandomDailyFortune');

	$cgi->param('CallbacksTodayEmail','no')
	  if ! $cgi->param('CallbacksTodayEmail');
	$cgi->param('CallbacksYesterdayEmail','no')
	  if ! $cgi->param('CallbacksYesterdayEmail');

	$cgi->param('listUseHiliter','no')
	  if ! $cgi->param('listUseHiliter');

	$cgi->param('AutoRefresh','no')
	  if ! $cgi->param('AutoRefresh');

	# save arguments to options
	for ($cgi->param()) {
		# overwrite options with form arguments
		$opt->set($_ => $cgi->param($_));
	}
	$opt->save;

	if ( $cgi->param('keep_open') !~ /yes/
	&& ! $cgi->param('Next Picture.x')
	&& ! $cgi->param('Next Picture.y')
	&& ! $cgi->param('Previous Picture.x')
	&& ! $cgi->param('Previous Picture.y')) {
		# Close options window
		display_message("Options saved.");
		exit;
	}
}

if ($request =~ /picture/i) {
	# save arguments to options
	for ($cgi->param()) {
		# overwrite options with form arguments
		$opt->set($_ => $cgi->param($_));
	}
	$opt->save;
}

if ($cgi->param('Next Picture.x') or $cgi->param('Next Picture.y')) {
	# select next picture in daily pictures directory

	# remove non-option arguments
	$cgi->delete('Next Picture.x');
	$cgi->delete('Next Picture.y');

	# save arguments to options
	for ($cgi->param()) {
		# overwrite options with form arguments
		$opt->set($_ => $cgi->param($_));
	}
	$opt->set('DailyPicture' => next_pic($cgi->param('DailyPicture')));
	$opt->save;
}

if ($cgi->param('Previous Picture.x') or $cgi->param('Previous Picture.y')) {
	# select previous picture in daily pictures directory

	# remove non-option arguments
	$cgi->delete('Previous Picture.x');
	$cgi->delete('Previous Picture.y');

	# save arguments to options
	for ($cgi->param()) {
		# overwrite options with form arguments
		$opt->set($_ => $cgi->param($_));
	}
	$opt->set('DailyPicture' => prev_pic($cgi->param('DailyPicture')));
	$opt->save;
}

# display Options form

my $file = "$Templates/options.htm";
open TEMPLATE, '<', $file or die "Cannot open options template '$file' $!";

local $/;
my $Template = <TEMPLATE>;     # read entire file
close(TEMPLATE);

# Get default values for search form control settings.
local $SearchEnter        = $opt->get('SearchEnter')        || '';
local $SearchControlEnter = $opt->get('SearchControlEnter') || '';
local $SearchMax          = $opt->get('SearchMax')          || '';

#
# Substitute option settings into input field value strings
#

$Template =~ s|<!--+ Daily Picture --+>|daily_pic()|e;

$Template =~ s|<!--+ Browse Picture --+>|browse_pic()|e;

$Template =~ s|<!--+ Search Control Options --+>|search_control_options()|e;

$Template =~ s|<!--+ SQL Control Options --+>|sql_control_options()|e
  if Acme::Security::allow('Access','SQL');

$Template =~ s|<!--+ User Name --+>|user_name() . " (" . user() . ")"|e;

#
# Substitute commands
#

$Template =~ s/(<!--+\$+\s*.*?\s*--+>)/tag_command($1)/egs;

#
# Substitute <input> tags.
#

$Template =~ s|(<[iI][nN][pP][uU][tT]\s.*?>)|tag_input($1,$opt)|egs;

print $Template;

exit;

#-------------------------------------------------------------------------------

sub tag_input {
	my ($tag,$opt) = @_;

	#
	# Substitue values into <input> tags.
	#

	my $tg = new Acme::HTML::Tag $tag;
	my $tag_type = $tg->type;
	my $tag_name = $tg->name;

	return $tag unless $tag_name;

	if ($tag_name eq "current_user") {
		$tag_value = user();
	}
	else {
		$tag_value = $opt->get($tag_name);
	}

	if (! $tag_value) {
		# Use default values for search control settings.

		$tag_value = $SearchEnter
		  if $tag_name =~ /SearchEnter$/;
		$tag_value = $SearchControlEnter
		  if $tag_name =~ /SearchControlEnter$/;
		$tag_value = $SearchMax
		  if $tag_name =~ /SearchMax$/;
	}

	$tag_value ||= '';

	#
	# Fill tag value
	#

	if ($tag_name eq 'MessageWindowAutoCloseTime') {

		#
		# Make a dropdown list
		#

		%close_options = (
			''    => '(disabled)',
			'3'   => '3',
			'2'   => '2',
			'1'   => '1',
			'1.5' => '1.5',
			'.7'  => '.7',
			'.5'  => '.5',
			 );

		return Acme::HTML::ListMenus->List(
				    \%close_options,	# hash of options
  				    $tag_name,		# name of control
				    $tag_value,		# default value
				    undef,
				    undef,
				    undef,
				    'noundef',
				    );

	}
	elsif ($tag_type eq 'checkbox' || $tag_type eq 'radio') {

		#
		# Set 'checked' property for checkbox and radio buttons.
		#

		if ($tag_value && $tag_value eq $tg->value) {
			$tg->set(checked => 'yes');
		}
		else {
			$tg->set(checked => 'no');
		}
	}
	else {

		#
		# Redefine tag with new tag value.
		#

		$tg->set(value => $tag_value);
	}

	return $tg->as_html;
}

#------------------------------------------------------------------------------

sub search_control_options {
	# Display an options panel for every search control listed in
	# configuration file.

	my $s = new Acme::Base::Schema;

	my $t;
	my $title = '';
	my $script = '';

	for my $table (('customers',@{Acme::Config->Search_Tables},'ad_sources')) {

		next if ! $table;
		next if $table eq '-';

		next unless Acme::Security::allow('Access',$table);

		$t       = $s->t($table);
		$title   = $t->title;
		$script .= search_control_script($table);
		$html   .= search_control_html($table,$title);
	}

	return <<End_Of_HTML;
<!-- Search Control Options -->
<script language="JavaScript" type="text/javascript">
function search_control_options_save(f,w) {
	// Window w is assumed open.
	var i;
	$script
}
</script>
$html
End_Of_HTML
}

sub search_control_script {
	my ($table) = @_;

	return <<End_Of_Script;

	if (w.${table}SearchEnter) {
		for (i=0; i < f.${table}SearchEnter.length; i++) {
			if (f.${table}SearchEnter[i].checked)
				w.${table}SearchEnter =
				  f.${table}SearchEnter[i].value;
		}
	}

	if (w.${table}SearchControlEnter) {
		for (i=0; i < f.${table}SearchControlEnter.length; i++) {
			if (f.${table}SearchControlEnter[i].checked)
				w.${table}SearchControlEnter =
				  f.${table}SearchControlEnter[i].value;
		}
	}
End_Of_Script
}

sub search_control_html {
	my ($table, $title) = @_;

	return <<End_Of_HTML;

<!-- ${title} Search Options -->
<div align="left"><a name="${table}_search_options"></a>

<!--\$ Navigation Buttons -->

<table border="0" cellpadding="5" cellspacing="0">
<tr>
  <th align="right" valign="top" width="75">${title} Search<br>
  <input type="submit" value=" Save " onClick='this.form.request.value="Save";return true' class="small-button">
  </th>
  <td valign="top">

  <table cellspacing="0" cellpadding="4" border="1">
  <tr>
    <td nowrap>Enter key in search text:<br>
      <input type="radio" name="${table}SearchEnter" value="enter" checked>
      goes to next line in search text
      <br>
      <input type="radio" name="${table}SearchEnter" value="tab">
      acts like Tab key
      <br>
      <input type="radio" name="${table}SearchEnter" value="ctrl-enter">
      acts like CTRL-Enter key
    </td>
  </tr>
  </table>

  <td valign="top">

  <table cellspacing="0" cellpadding="4" border="1">
  <tr>
    <td nowrap>CTRL-Enter key in search text:<br>
      <input type="radio" name="${table}SearchControlEnter" value="View" checked>
      presses View button
      <br>
      <input type="radio" name="${table}SearchControlEnter" value="Edit">
      presses Edit button
      <br>
      <input type="radio" name="${table}SearchControlEnter" value="List">
      presses List button
      <br>
      <input type="radio" name="${table}SearchControlEnter" value="Count">
      presses Count button
      <br>
      <input type="radio" name="${table}SearchControlEnter" value="New">
      presses New button
    </td>
  </tr>
  </table>

  </td>
</tr>
<tr>
  <td></td>
  <td valign="top" colspan="2">
    <table cellspacing="0" cellpadding="2" border="0">
    <tr>
    <tr>
      <td align="right" valign="top">
      <input type="text" name="${table}SearchMax" value="" size="4" maxlength="4" class="number">
      </td>
      <td>
      Maximum number of search history entries retained.
      </td>
    </tr>
    </table>
  </td>
</tr>
</table></div>
End_Of_HTML

# Optional line separator.
#<!-- line --><table width="100%" cellpadding="5" cellspacing="0" border="0"><tr><td width="75">&nbsp;</td><td><hr align="left" width="100%"></td></tr></table>

}

#------------------------------------------------------------------------------

sub sql_control_options {

	return <<End_Of_HTML;
<!-- SQL Queries -->
<div><a name="sql_options"></a>

<!--\$ Navigation Buttons -->

<table width=100% border=0 cellpadding="5">
<tr>
  <th align=right valign=top width=75>SQL Queries<br>
  <input type=submit value=" Save " onClick='this.form.request.value="Save";return true' class=small-button>
  </th>
  <td nowrap style="vertical-align:bottom">
  </td>
</tr>
<tr>
  <td></td>
  <td valign="top">
    <table cellspacing=0 cellpadding=2 border=0>
    <tr>
      <td align=right valign=top>
      <input type=text name="sqlQueryMax" value="" size=4 maxlength=4 class=number>
      </td>
      <td>
      Maximum number of past SQL queries retained.
      </td>
    </tr>
    </table>

  </td>
</tr>
</table></div>

End_Of_HTML

}

#------------------------------------------------------------------------------

sub daily_pic {
	# return code for daily picture, and buttons to change it

	#
	# Assure picture setting is "white" if it is not set.
	#

	my $default_pic = Acme::Config->Default_Pic;
	$opt->set('DailyPicture' => "$default_pic") unless $opt->DailyPicture;

	# Note: width of picture is set to be 3 times larger than displayed
	# on the banner (55 pixels).

	return <<EOS;
<center>
<img src="@{[$opt->DailyPicture]}" id="daily_pic" border="1" width="165">
<br>
<input type="image" name="Previous Picture" src="$Images/prev_pic.gif" alt="Previous" width="21" height="14" border="0">
&nbsp;
<input type="image" name="Next Picture" src="$Images/next_pic.gif" alt="Next" width="21" height="14" border="0">
</center>
EOS
}

#------------------------------------------------------------------------------

sub browse_pic {
	# return code button to browse picture directory

	my $url = '/cgi-bin/browse.cgi?filter=\\\\.(gif|jpg|jpeg|png)&directory='.Acme::Config->User_Pic_Dir;

	return <<EOS;
<!-- Begin Browse button DailyPicture -->
<input type="button" value="Browse" onClick="_browse_DailyPicture()" class="browse-button" tabindex="-1">
<script language="JavaScript" type="text/javascript">

/*
 * Browse button support for DailyPicture
 */

var _browse_win_DailyPicture;

function _browse_DailyPicture(options) {
	// Open browse window.
	var url = '$url';

	_browse_win_DailyPicture = new_browse(
					url,
					'DailyPicture',
					options);
	return true;
}
function _browse_return_DailyPicture() {
	// Return from browse window opened in _browse_DailyPicture().
	var f = self.document.forms['options_form'] || self.document.forms[0];
	if (_browse_win_DailyPicture && _browse_win_DailyPicture.result) {
		var r = _browse_win_DailyPicture.result;

		// Assign selection values to fields.
		if (f.DailyPicture && r.value) {
			f.DailyPicture.value = r.value;
		}

		// Display picture
		document.getElementById('daily_pic').src = f.DailyPicture.value;

		// Go to next input field.
		f.DailyPicture.focus();
	} else {
		// No records were found.
		f.DailyPicture.focus();
	}
}

</script>
<!-- End Search button DailyPicture -->
EOS
}
__END__
