#!/usr/bin/perl -wT -I../lib

################################
# TODO, copied form mkform.cgi
################################

=head1 NAME

acknowledge.cgi -- Make a form template

=head1 SYNOPSIS

    acknowledge.cgi?table=customers&template=newform.htm

=head1 DESCRIPTION

The acknowledge.cgi marks an order as having been acknowledged and sends an
email to the company placing the order.

=head1 CHANGE LOG

 9/5/02   Initial version.

=head1 AUTHOR

Mack Pexton <mack@acmebase.org>

=head1 COPYRIGHT

Copyright 2002, Mackley F. Pexton.  All rights reserved.

This is open software and is licensed under the terms of the GNU General Public License.

=cut

$|=1;

use Acme::HTML			qw(escape_html);
use Acme::HTML::Parameters;
use Acme::Form::Builder;
use Local::Acme::User		qw(user);
use Acme::Config;
use CGI;

use Acme::Security;
Acme::Security::check('Access');

$user = user();

#
# Begin processing form
#

$|=1;		# flush output (synchronize errors with output)

$cgi = new CGI;
$cgi->cache('no');

$Base		= Acme::Config->Base;
$DocumentRoot	= Acme::Config->DocumentRoot;
$New_Forms	= Acme::Config->New_Forms;

#
# Parse cgi arguments
#

$Primary_Table	= $cgi->url_param('table') || $cgi->param('table') || '';
$Title		= $cgi->param('title')     || '';
$Type		= $cgi->param('type')      || 'form';
$File		= $cgi->param('file')      || '';

#
# Get parameters if not in cgi arguments
#

unless ($File) {
	$cgi->cache('no');
	print $cgi->header(-type=>'text/html', -expires=>'-1d') unless @ARGV;
	get_parameters();
	exit;
}

my $filepath = "${DocumentRoot}${New_Forms}/$File";

if (-e $filepath) {
	$overwrite = $cgi->param('overwrite') || 'no';
	unless ($overwrite =~ /yes/i) {
		$cgi->cache('no');
		print $cgi->header(-type=>'text/html', -expires=>'-1d') unless @ARGV;
		print Acme::HTML::Parameters->confirm(
			cgi	=> $cgi,
			title   => 'Make a Form',
			message => "The file <code>$File</code> already exists.",
			prompt  => "Do you want to overwrite it?",
			answer  => 'overwrite',
			);

		exit;
	}
}

#
# Create form template
#

$Template = Acme::Form::Builder::make_form(
		table => $Primary_Table,
		title => $Title,
		type  => $Type,
		file  => $File)
  if $Primary_Table;

die "Form template not specified.\n" unless ($Template);

#
# Save template in the templates directory so that the links in the user 
# instructions work.
#

($filepath) = $filepath =~ /([^\$\@`'"|;]+)/;   # untaint
$filepath =~ s/\.\.\///g;			# remove ../ from file path
open(TEMPLATE,">$filepath") or 
  die "New form cannot be saved to the file: $filepath: $!\n";
print TEMPLATE $Template;
close(TEMPLATE); 

#
# Adjust privileges of new file.
#

chmod 0666, $filepath;

#
# Display newly created form.
#

print $cgi->redirect("${Base}${New_Forms}/${File}");

exit;

#------------------------------------------------------------------------------

sub get_parameters {

	$File ||= "${user}_form.htm";		# set default value

	my $parameters = <<EOF;
<div style="margin:5ex 10ex">
<table cellspacing=0 cellpadding=2 border=0>
<tr>
<th align=right>File Name</th>
<td><input type="text" name="file" value="$File" size="25" class="text" onChange="check_input(this,'text')"></td>
</tr>
<tr><th></th><td>&nbsp;</td></tr> <!-- blank line -->
<tr>
<th align=right>Form Type</th>
<td><input type="radio" name="type" value="form" checked> Form
&nbsp; <input type="radio" name="type" value="query"> Query
&nbsp; <input type="radio" name="type" value="editlist"> Edit List
</tr>
</table>
</div>
EOF

	my $param_form = Acme::HTML::Parameters->get(
				cgi	     => $cgi,
				title        => 'Acknowledge an Order',
				html         => $parameters,
				instructions => instructions(),
				submit       => 'Acknowledge',
				);

	print $param_form;
}

#------------------------------------------------------------------------------

sub instructions {

	return <<EOF;
<!-- Start Template Instructions -->
<p>The new form is stored in a directory (folder) named <code>$New_Forms</code>
on the server.</p>

<p>You can customize the appearance of the new form to suit your desires.</p>

<p>To enter new information, use a link like:

<blockquote>
<a href="javascript:open_form('${Base}/cgi-bin/form.cgi?template=${New_Forms}/${File}&request=New')">${Base}/cgi-bin/form.cgi?template=${New_Forms}/${File}&request=new</a>
</blockquote>

or to edit information, use a link like:

<blockquote>
<a href="javascript:open_form('${Base}/cgi-bin/form.cgi?template=${New_Forms}/${File}&request=edit&search=111')">${Base}/cgi-bin/form.cgi?template=${New_Forms}/${File}&request=edit&search=111</a>
</blockquote>

Change the search value 111 to the real ID number of the record you want to edit.
</p>
<!-- End Template Instructions -->
EOF
}
