#!/usr/bin/perl -wT -I../lib

=head1 NAME

schema.cgi -- Display a database schema information

=head1 SYNOPSIS

    schema.cgi?table=customers&table=customer_info

    schema.cgi?sort=yes

=head1 DESCRIPTION

The schema.cgi program displays a list of database tables with their
type and description.

=head1 CHANGE LOG

 4/30/00   Initial version.

=head1 AUTHOR

Mack Pexton <mack@acmebase.org>

=head1 COPYRIGHT

Copyright 2000, Mackley F. Pexton.  All rights reserved.

This is open software and is licensed under the terms of the GNU General Public License.

=cut

use Acme::HTML;
use Acme::Base::Schema;
use Acme::HTML::NavButtons;
use Local::Acme::Banners	qw(common_header common_footer);
use Acme::Config;
use CGI;

use Acme::Security;
Acme::Security::check('Access');

$\="\n";	# append newlines to all print statements
$|=1;		# flush output (synchronize errors with output)

$cgi = new CGI;
print $cgi->header(-type=>'text/html') unless @ARGV;

$Styles = Acme::Config->Styles;		# location of css stylesheets

@tables = $cgi->param('table');
$sort = $cgi->param('sort');

$schema = new Acme::Base::Schema @tables;

@tables = $schema->tables unless @tables;  # list all if not specified
@tables = sort @tables if defined $sort && 'no' !~ /^\Q$sort\E/i;

$b = new Acme::HTML::NavButtons;

display_header();

for my $tab (@tables) {
	$t = $schema->t($tab)
	  or die "No information for the table '$tab' is available.\n";

	display_item($t);
}

display_footer();

exit 0;

#------------------------------------------------------------------------------

sub display_header {

	print <<EOF;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
"http://www.w3.org/TR/html4/loose.dtd">

<html>
<head>
<title>doc Database Schema Information</title>
<link rel="stylesheet" href="$Styles/common.css" type="text/css">
<link rel="stylesheet" href="$Styles/schema.css" type="text/css">
</head>
<body>
EOF
	print $b->first_buttons;
	print common_header('Database Schema Information');

	print "<br>";
}

sub display_footer {

	print common_footer('Database Schema Information');
	print $b->last_buttons;
	print "</body>\n</html>";
}

sub display_item {
	my $t = shift;

	my ($type, $ref, $private, $fld, $f);
	my $name = $t->name;
	my $desc = $t->description;
	##$desc =~ s/\n/<br>/sg;		# change linefeeds to <br>
	my $note = $t->note;
	$note = qq(<p class="note">$note</p>) if $note;

	print <<EOF;
<table width=100% cellpadding="4" cellspacing="4">
<tr>
  <td width="20%" valign="top" nowrap><a name="$name"></a><span class="tablename">$name</span></td>
  <td><span class="tabledesc">$desc</span>$note</td>
</tr>
</table>

<div class="indent">
<table width="90%">
EOF
	if ($fld = $t->id) {
		# ID fields are special
		$type = 'serial';
		$desc = 'A system assigned ID number.';
		print <<EOF;
<tr>
  <td width="20%" valign="top" nowrap><span class="fieldname">$fld </span></td>
  <td width="15%" valign="top" nowrap><span class="property">$type </span></td>
  <td ><span class="desc">$desc</span></td>
</tr>
EOF
	}

	my $include_private_fields = 1;		# set flag
	for $fld ($t->fields_secure('Access',$include_private_fields)) {
		$f = $t->f($fld);
		$type = $f->type || '';
		$desc = $f->description || '';
		##$desc =~ s/\n/<br>/sg;	# change linefeeds to <br>
		$private = $f->private || '';
		$private = " Private. $private" if $private;
		$ref = $f->choose_table || $f->search_table || '';
		$note = $f->note || '';
		$note = qq(<p class="field-note">$note</p>) if $note;
		if ($ref) {
			my $r = $t->schema->t($ref);
			if (defined $r) {
				my $name = $r->name;
				my $refname = $r->title;
				
				# Check global @tables to see if reference
				# link should be a link to an anchor or to
				# another instance of this program.
				if ($cgi->param('table')) {
					# Recall this program
					$ref =
qq( Refer to <a href="/cgi-bin/schema.cgi?table=$ref" title="Show table information for $refname.">$name<a>.);
				}
				else {
					# Refer to anchor.
					$ref =
qq( Refer to <a href="#$ref" title="Show table information for $refname.">$name<a>.);
				}

				if (! $desc) {
					my $rf = $r->f($r->key);
					if (not defined $rf) {
						# Happens if key is an ID key
						$desc = 
qq(Link to $name table key ) . $r->key . '.';
					}
					else {
						$desc = $rf->description || '';
					}
				}
			}
			else {
				$ref = '';	# Referred table is not defined.
			}
		}

		print <<EOF;
<tr>
  <td width="20%" valign="top" nowrap><span class="fieldname">$fld </span></td>
  <td width="15%" valign="top" nowrap><span class="property">$type </span></td>
  <td ><span class="desc">$desc$ref$private$note</span></td>
</tr>
EOF
	}
	print <<EOF;
</table>
</div>
EOF
}
__END__
