#!/usr/bin/perl -wT -I../../../lib

# Change Supplier Log Information
# This is a program front end to the /templates/supplier_log_response.htm.

# The HTML input fields in the pattern file must have their name=""
# parameter on the same line as the value="" parameter in order for the
# substitution of the supplier information to work. The quotes are important.
# Also, only one input field code per source line.
#
# The field names just happen to correspond to database field names.

use Acme::HTML;
use Acme::Db;
use Acme::Parse		qw(parse_args);
use Acme::Date		qw(today);
use Local::Acme::User	qw(user);
use Local::Acme::ListMenus;
use Acme::Config;

$|=1;
print "Content-type: text/html\n\n";

my ($field_name, $field_value);

local (%args);
unless (parse_args(\%args) and ($log_id = $args{log_id})) {
	print "<html><body onLoad='self.focus()'>(empty)</body></html>";
	exit 0;
}

$Templates = Acme::Config->Templates;

my $user = user();
local $today = today();

if (open HTML, "$Templates/supplier_log_response.htm") {

	# Get supplier log
	my (
		$supplier_id,
		$supplier_name,
		$log_id,
		$log,
		$callback_date,
		$user_id,
		$message,
		$entry_date,
		$last_update,
		$last_update_by,
		$respond_date,
		$respond_by,
		$response_to_log_id,
	) = Acme::Db->get(<<End_of_Sql);
		select
			suppliers.supplier_id,
			suppliers.supplier_name,
			supplier_log.log_id,
			supplier_log.log,
			supplier_log.callback_date,
			supplier_log.user_id,
			supplier_log.message,
			supplier_log.entry_date,
			supplier_log.last_update,
			supplier_log.last_update_by,
			supplier_log.respond_date,
			supplier_log.respond_by,
			supplier_log.response_to_log_id
		from supplier_log, suppliers
		where supplier_log.log_id = $log_id
		  and supplier_log.supplier_id = suppliers.supplier_id
End_of_Sql

	# Substitute supplier information into input field value strings
	while (<HTML>) {
		# personalize window title
		s/<title>Log: Supplier Log Response</<title>sc $supplier_name</o
		  if ($supplier_name);

		# uncheck window open option
		s/(<input type=checkbox name="keep_open".*) checked/$1/;

		# Display supplier ID and name on form
		s;\(not assigned\);<b>$supplier_id</b>;o;
		 
		s;\(not available\);<a href='javascript:open_view("/cgi-bin/sv.pl?supplier_id=$supplier_id")' title="View supplier information."><font size=+2><b>$supplier_name</b></font></a>;o;

		s;<!-- RESPONSE BY -->;<b>Responded</b> $response_date by $respond_by;o
		  if ($response_date);

		# Enter information into textarea fields
		if ( ($field_name) = m/<textarea.*name\s*=\s*\"(\w+)\"/i ) {
			$field_value = eval "\$$field_name";
			$field_value = defined $field_value ? $field_value : "";
			s|(<textarea.*>).*(</textarea>)|$1$field_value$2|;
		}

		# Enter information into form input field default values
		if ( ($field_name) = m/input.*name\s*=\s*\"(\w+)\".*value\s*=/i ) {
			$field_value = eval "\$$field_name";
			$field_value = defined $field_value ? $field_value : "";

			SWITCH: {
				if ($field_name eq "log_type") {
					s{<input\s.*?name="log_type".*?>}
					 {Local::Acme::ListMenus->log_types("log_type",$field_value)}e;
					last SWITCH;
				}
				
				if ($field_name eq "user_id") {
					s{<input.*name="user_id".*>}
					 {Local::Acme::ListMenus->user_options($field_value,"user_id")}e;
					last SWITCH;
				}
				
				if ($field_name eq "current_user") {
					s/(value\s*=\s*")/$1$user/;
					last SWITCH;
				}

				if ($field_name eq "respond_by") {
					s/(value\s*=\s*")/$1$user/;
					last SWITCH;
				}

				# prime respond_by check box with today's date
				if ($field_name eq "respond_date") {
					if ($respond_date) {
						s/(value\s*=\s*")/$1$field_value/;
						s{(<input.*name="respond_date".*)>}
						 {$1 checked>};
					}
					else {
						s/(value\s*=\s*")/$1$today/;
					}
					last SWITCH;
				}

				if ($field_name eq "entry_date") {
					s/<input.*name="entry_date".*>/$field_value/;
					last SWITCH;
				}

				if ($field_name eq "last_update") {
					s/<input.*name="last_update".*>/$field_value/;
					last SWITCH;
				}

				if ($field_name eq "last_update_by") {
					s/<input.*name="last_update_by".*>/$field_value/;
					last SWITCH;
				}
				
				# Response form fields
				# prime response field linking to log record
				if ($field_name eq "R_response_to_log_id") {
					s/(value\s*=\s*")/$1$log_id/;
					last SWITCH;
				}

				if ($field_name eq "R_supplier_id") {
					$field_value = $supplier_id 
					  unless ($R_supplier_id);
					s/(value\s*=\s*")/$1$field_value/;
					last SWITCH;
				}

				if ($field_name eq "R_log_type") {
					$field_value = $log_type 
					  unless ($R_log_type);
					s/(value\s*=\s*")/$1$field_value/;
					last SWITCH;
				}

				if ($field_name eq "R_user_id") {
					$field_value = $user
					  unless ($field_value);
					s{<input.*name="R_user_id".*>}
					 {Local::Acme::ListMenus->user_options($field_value,"R_user_id")}e;
					last SWITCH;
				}
				
				if ($field_name eq "R_current_user") {
					$field_value = $user
					  unless ($field_value);
					s/(value\s*=\s*")/$1$field_value/;
					last SWITCH;
				}

				# Enter response record audit info
				if ($field_name eq "R_entry_date") {
					$field_value = $today
					  unless ($field_value);
					s/<input.*name="R_entry_date".*>/$field_value/;
					last SWITCH;
				}
				if ($field_name eq "R_last_update") {
					$field_value = $today
					  unless ($field_value);
					s/<input.*name="R_last_update".*>/$field_value/;
					last SWITCH;
				}
				if ($field_name eq "R_last_update_by") {
					$field_value = $user
					  unless ($field_value);
					s/<input.*name="R_last_update_by".*>/$user/;
					last SWITCH;
				}
				
				s/(value\s*=\s*")/$1$field_value/;
			}
		}

		print;
	}
	close HTML;
}
exit 0;
