#!/usr/bin/perl -wT -I../../../lib

# Supplier Log Post routine

# Post information from a form to the database. A message is displayed and
# and OK button is displayed which when pushed closes the window the message
# is displayed in.

use Acme::Db;
use Acme::HTML		qw(display_message);
use Acme::Parse		qw(parse_args);
use Acme::Date		qw(today timestamp);
use Local::Acme::User	qw(user);

use Acme::Security;
Acme::Security::check('Access');

local $user = user();
local $today = today();
local $timestamp = timestamp();

local %args;
if (parse_args(\%args)) {
	local $log_id = $args{log_id};
	delete $args{log_id};	# key field, do not update

	if (defined $args{R_log_id}) {
		# Response record is being posted
		local $R_log_id = $args{R_log_id};
		delete $args{R_log_id};	# key field, do not update

		# remove non-database column names from %args
		delete $args{keep_open};	# flag field, do not update
		delete $args{R_current_user};	# phantom field, do not update

		# Construct record audit info
		$args{R_last_update} = $timestamp;	# system assigned values
		$args{R_last_update_by} = $user;	# system assigned values

		$args{R_log_type} = $args{log_type} unless $args{R_log_type};

		if ($R_log_id and $R_log_id > 0) {
			post_change($R_log_id);
		}
		else {
			$args{R_entry_date} = $today;	# system assigned values
			post_new();
		}
	}
	else {
		# Original log record being posted
		# remove non-database column names from %args
		delete $args{keep_open};	# flag field, do not update
		delete $args{current_user};	# phantom field, do not update

		# Construct record audit info
		$args{last_update} = $timestamp;	# system assigned values
		$args{last_update_by} = $user;		# system assigned values

		if ($log_id and $log_id > 0) {
			post_change($log_id);
		}
		else {
			$args{entry_date} = $today;	# system assigned values
			post_new();
		}
	}
}
exit 0;

#------------------------------------------------------------------------------

sub post_change {
	my ($key, $value, @kv);
	while (($key,$value) = each %args) {
		$value = Acme::Db->escape_sql($value);
		push @kv, join " = '", $key, $value;
	}
	my $post_script = join '',
	  "update supplier_log set \n",
	  (join "',\n", @kv), 
	  "'\nwhere log_id = $log_id;";

	$|=1;
	print "Content-type: text/html","\n\n";

	if (Acme::Db->execute($post_script)) {
		display_message("The supplier log record has been changed.",
				"Message",
				"post_cleanup()");
	}
	else {
		# It is assumed that an error message has already been output.
		display_message("The supplier log record has <font color='red'>NOT</font> been changed.",
				"Message",'',0);
	}
}

#------------------------------------------------------------------------------

sub post_new {
	foreach $a (keys %args) {  # change CR/LF to pass Informix syntax
		$args{$a} = Acme::Db->escape_sql($args{$a});
	}

	# Construct SQL script
	# the only arguments in %args should be database columns
	my $c = join ",\n", (keys %args);	# column names
	my $v = join "',\n'", (values %args);	# surround values with quotes

	# response log record field names have R_ prepended to them to 
	# differentiate them from the original log record fields
	$c =~ s/\bR_//gm;		# remove field name R_ prefixes

	my $post_script = "insert into supplier_log (\n$c\n) values (\n'$v'\n);";

	$|=1;
	print "Content-type: text/html","\n\n";

	my $rc = Acme::Db->execute($post_script);

	unless ($rc) {
		display_message("The supplier log record has <font color='red'>NOT</font> been added.",
				"Message",'',0);
		return 0;
	}

	if (defined $args{R_response_to_log_id}) {
		# This posting is for a response log entry
		# Post to the original log entry the response date.

		$rc = Acme::Db->execute(<<EOS);
update supplier_log set 
  respond_by = '$remote_user',
  respond_date = current_date
where log_id = $args{R_response_to_log_id}
EOS
	}

	# Display results screen
	display_message("The supplier log record has been added.",
			"Posted Supplier Log",
			"post_cleanup()");
}
