/*
 * Common JavaScript functions for Acme Base
 *
 * The following are common routines that should be included in every page
 * requiring interaction from the user. It defines a common event handler
 * capturing user's mouse clicks and key codes, and it includes the functions
 * used to open the multitude of different types of windows for the system.
 */

self.focus();	// Netscape gets confused which window should be on top

/*
 * Event handlers
 */

// Assign keypress event handler only if not assigned before
if (!window.document.onkeypress) window.document.onkeypress = common_keypress;

function common_keypress(evt) {
	evt = (evt) ? evt : ((window.event) ? window.event : null);

	switch (evt.keyCode) {
	case 27:	// escape key
		window.close();
		return true;

	case 32:		// space key
		/*
		 * Ctrl-Space bar displays the console screen.
		 */
		if (evt.ctrlKey) {
			if (typeof window.console == "function") {
				console();
				return false;
			}
		}
		return true;
	}
}

/*
 * Button functions
 */

consoleProg = "/cgi-bin/console.cgi";
consoleName = "console";

function console() {

	// Open console window.

	if (window.name == consoleName) {
		// Current window is console window.
		location.href = consoleProg;		// refresh
	}
	else {
		var settings = "menubar,status,resizable,scrollbars" + 
		  ",location,toolbar,width=640,height=640";

		var w = window.open("", consoleName, settings);
		w.focus();		// bring to forefront

		if (!w.isConsoleLoaded) {	// see console.cgi
			// Load new copy.
			w.location.href = consoleProg;
		}
	}
}

/*
 * Image highlight functions
 *
 * The following functions highlight images by swapping the source of the
 * image tag graphic with a file name that has the string '_hilite' appended
 * to its name.
 *
 * A four state button can be simulated simply by altering the global variables
 * hilite_image_obj and hilite_image_orig during an onClick event.
 */

var hilite_image_obj;		// image object highlight
var hilite_image_orig = "";	// image changed to highlight image (original)

function hilite(img) {

	if (hilite_image_orig != "") unhilite(img);	// safety

	hilite_image_obj = img;
	hilite_image_orig = img.src;

	img.src = hilite_image_orig.substring(0, hilite_image_orig.indexOf(".gif")) +
	  "_hilite.gif";
}

function unhilite(img) {
	
	if (hilite_image_orig != "") {
		hilite_image_obj.src = hilite_image_orig;
		hilite_image_orig = "";
	}
}

/*
 * New target function
 *
 * Javascript routines are used to open windows so that different type of 
 * windows have a similar size and appearance. However, using the href
 * attribute in a <a> tag like:
 *	<a href="javascript:open_form(url,window_name)">Open Form</a>
 * causes the link to be inoperable whenever Javascript is turned off,
 * which usually happens in email.
 *
 * The new_target() function is designed to be used as an onClick event
 * in an <a> tag which opens a new window and then assigns the newly
 * opened window name to the target attribute of the <a> tag. This allows
 * the <a> links to operate when JavaScript is not available, but still
 * opens consistant type windows when it is.
 *
 * Example:
 *	<a href="/cgi-bin/form.cgi?form=test" onClick="new_target(this,'form')">
 *
 * Parameter:
 *	alink	-- the <a> link object (this)
 *	wintype	-- the type of window, 'list', 'view', 'form', 'message', ...
 *	message	-- message text displayed in the new window while getting url
 *		   Only used in a 'message' type window.
 *	options -- window options passed to window.open() Javascript function
 */

function new_target(alink,wintype,message,options) {

	var url;	// empty, used as placeholder in open_*() calls
	var name;

	// Use time counter as a number to make a unique window name
	var unique = (new Date()).getTime().toString() + Math.floor(Math.random() * 100);

	if (wintype) {
		switch (wintype.toLowerCase()) {
		case "list":
			name = "List" + unique;
			open_list(url,name,options,message);
			break;

		case "view":
			name = "View" + unique;
			open_view(url,name,options,message);
			break;

		case "form":
			name = "Form" + unique;
			open_form(url,name,options,message);
			break;

		case "log":
			name = "Log" + unique;
			open_log(url,name,options,message);
			break;

		case "response":
			name = "Response" + unique;
			open_log_response(url,name,options,message);
			break;

		case "audit":
			name = "Audit" + unique;
			open_audit(url,name,options,message);
			break;

		case "options":
			name = "Options" + unique;
			if (!url) url = alink.href;
			open_options(url,name,options,message);
			break;

		case "help":
			name = "Help" + unique;
			open_help(url,name,options,message);
			break;

		case "doc":
			name = "Doc" + unique;
			open_doc(url,name,options,message);
			break;

		case "browse":
			name = "Browse" + unique;
			open_browse(url,name,options,message);
			break;

		case "print":
			name = "Print" + unique;
			open_print(url,name,options,message);
			break;

		case "calendar":
			name = "Calendar" + unique;
			open_calendar(n_months,url,name,options,message);
			break;

		case "message":
			name = "Message" + unique;
			var title;	// placeholder
			open_message(message, title, url, name, options);
			break;

		case "report":
			name = "Report" + unique;
			open_window(url,name,options,message);
			break;

		default:
			name = "Window" + unique;
			open_window(url,name,options,message);
		}
	}
	else {
		name = "Window" + unique;
		open_window(url,name,options,message);
	}

	// re-assign new target window name
	if (alink) alink.target = name;

	return true;
}

/*
 * Window functions
 *
 * The following functions are used to open different types of windows used
 * by the system. They enforce a common size and look for each type of window.
 * The open_*() functions do not return anything so that they can be used in
 * links like <a href="javascript:open_*()"> without having to use the void
 * operator. The new_*() functions return the window variable to the
 * calling programs.
 */

function new_window(url,name,options,message,title,noprogress) {

	if (!name) name = "";	// empty name makes a new window each time

	var settings = options ? options :
	  "copyhistory,directories,location,menubar" +
	  ",resizable,scrollbars,status,toolbar" +
	  ",width=800,height=600";

	// open window without url so other scripts can close it
	var w = window.open("", name, settings);
	/** w.focus(); ** cannot use if window to be opened from Outlook **/
	display_message(w, message, title, url, noprogress);
	return w;
}

function new_list(url,name,options,message,title,noprogress) {

	if (!name) name = "";

	// Set screen width and height.
	var width  = 690;
	var height = 700;

	// Position new window up and to the right.
	var top  = window.screenTop  - 10;
	var left = window.screenLeft + 10;

	var settings = options ? options :
	  "copyhistory,directories,location,menubar" +
	  ",resizable,scrollbars,status,toolbar" +
	  ",width="+width+",height="+height +
	  ",top="+top+",left="+left;

	var w = window.open("", name, settings);
	w.focus();
	display_message(w, message, title, url, noprogress);
	return w;
}

function new_view(url,name,options,message,title,noprogress) {

	if (!name) name = "";

	// Set screen width and height.
	var width  = 690;
	var height = 600;

	// Position new window up and to the right.
	var top  = window.screenTop  - 10;
	var left = window.screenLeft + 10;

	var settings = options ? options :
	  "copyhistory,directories,location,menubar" +
	  ",resizable,scrollbars,status,toolbar" +
	  ",width="+width+",height="+height +
	  ",top="+top+",left="+left;

	var w = window.open("", name, settings);
	/** w.focus(); ** cannot use if window to be opened from Outlook **/
	display_message(w, message, title, url, noprogress);
	return w;
}

function new_form(url,name,options,message,title,noprogress) {

	if (!name) name = "";

	// Set screen width and height.
	var width  = 760;
	var height = 472;

	// Position new window up and to the right.
	var top  = window.screenTop  - 10;
	var left = window.screenLeft + 10;

	var settings = options ? options :
	  "menubar,resizable,scrollbars,status" +
	  ",width="+width+",height="+height +
	  ",top="+top+",left="+left;
	var w = window.open("", name, settings);
	w.focus();				// bring to forefront

	if (url && w.location.href.indexOf("about") != 0) {

		// Just bring form to forefront if it is the same url.
		// Do not refresh screen and loose any edits user made.

		if (w.location.href == url) return w;

		var fullurl = w.location.protocol + '//' + w.location.hostname + url;
		if (w.location.href == fullurl) return w;
	}

	display_message(w, message, title, url, noprogress);
	return w;
}

function new_log(url,name,options,message,title,noprogress) {

	if (!name) name = "";

	var settings = options ? options :
	  "menubar,resizable,scrollbars,status" +
	  ",width=680,height=560";

	var w = window.open("", name, settings);
	display_message(w, message, title, url, noprogress);
	return w;
}

function new_log_response(url,name,options,message,title,noprogress) {

	if (!name) name = "";

	var settings = options ? options :
	  "menubar,resizable,scrollbars,status" +
	  ",width=680,height=600";

	var w = window.open("", name, settings);
	display_message(w, message, title, url, noprogress);
	return w;
}

function new_audit(url,name,options,message,title,noprogress) {

	if (!name) name = "";

	// Set screen width and height.
	var width  = 550;
	var height = 400;

	// Position new window up and to the right.
	var top  = window.screenTop  - 10;
	var left = window.screenLeft + 10;

	var settings = options ? options :
	  "menubar,resizable,scrollbars,status" +
	  ",width="+width+",height="+height +
	  ",top="+top+",left="+left;

	var w = window.open("", name, settings);
	w.focus();
	display_message(w, message, title, url, noprogress);
	return w;
}

function new_options(url,name,options,message,title,noprogress) {

	if (!name) name = "options";

	var settings = options ? options :
	  "menubar,location,status,resizable,scrollbars,toolbar";

	var opt;
	if (url.match(/#(.*)/)) opt = RegExp.$1;

	// Size window according to desired option
	if (! opt) {
		settings += ",width=700,height=500"
	}
	else if (opt == "banner_picture") {
		settings += ",width=600,height=400"
	}
	else {
		settings += ",width=700,height=240"
	}

	var w = window.open("",name, settings);
	w.focus();
	if (!w.isOptionsLoaded || opt) {	// see options.htm
		// load new copy
		display_message(w, message, title, url, noprogress);
	}
	return w;
}

function new_help(url,name,options,message,title,noprogress) {

	if (!name) name = "";

	var settings = options ? options :
	  "copyhistory,directories,location,menubar" +
	  ",resizable,scrollbars,status,toolbar" +
	  ",width=600,height=700";

	var w = window.open("", name, settings);
	w.focus();
	display_message(w, message, title, url, noprogress);
	return w;
}

function new_doc(url,name,options,message,title,noprogress) {

	if (!name) name = "";

	var settings = options ? options :
	  "copyhistory,directories,location,menubar" +
	  ",resizable,scrollbars,status,toolbar" +
	  ",width=780,height=550";

	var w = window.open("", name, settings);
	w.focus();
	display_message(w, message, title, url, noprogress);
	return w;
}

function new_browse(url,name,options,message,title,noprogress) {

	if (!name) name = "";

	var settings = options ? options :
	  "menubar,resizable,scrollbars,status" +
	  ",width=450,height=600";

	var w = window.open("", name, settings);
	w.focus();
	display_message(w, message, title, url, noprogress);
	return w;
}

function new_print(url,name,options,message,title,noprogress) {

	if (!name) name = "";

	// Set window to small size
	var settings = options ? options :
	  "width=80,height=80,top=100,left=100,resizable";

	var w = window.open("", name, settings);

	if (!title)   title   = "Printing...";
	if (!message) message = "Printing...";

	var d = w.document;
	d.open();
	d.writeln('<html>');
	d.writeln('<head>');
	d.writeln('<title>' + title + '</title>');
	d.writeln('<style>.message{font-family:"Comic Sans MS" serif; font-size:8pt; padding:12pt 3pt</style>');
	d.writeln('<script language="JavaScript" type="text/javascript">');
	d.writeln('window.document.onkeypress = do_keypress;');
	d.writeln('function do_keypress(evt) {');
	d.writeln(' evt = (evt) ? evt : ((window.event) ? window.event : null);');
	d.writeln(' switch (evt.keyCode) {');
	d.writeln(' case 27: self.close(); return true;');
	d.writeln('}}');
	d.writeln('</script>');
	d.writeln('</head>');
	d.writeln('<body>');
	d.writeln('<div class="message">');
	d.writeln('<p>' + message + '</p>');
	d.writeln('</div>');
	d.writeln('</body>');
	d.writeln('</html>');
	d.close();

	if (url) w.location.href = url;
	return w;
}

function new_calendar(n_months,url,name,options,message,title,noprogress) {

	if (!name) name = "";

	var settings = options;

	if (!settings) {
		var w,h;
		settings = "resizable,scrollbars,status";

		if (n_months > 1) {
			settings = "copyhistory,directories,location,menubar" +
			  ",resizable,scrollbars,status,toolbar";
		}

		if (n_months > 6)       { w=880; h=1020;}
		else if (n_months > 3)  { w=880, h=780; }
		else if (n_months > 1)  { w=880, h=300; }
		else if (n_months == 1) { w=305, h=260; }

		settings += ",width="+w+",height="+h;
	}

        var cal = window.open("", "", settings);
	display_message(cal, message, title, url, noprogress);
	return cal;
}

function new_message(message, title, url, name, options, noprogress) {

	// Print the given message with a "Please wait..." appended.
	// Open a message window to be filled in by cgi routine 
	// (usually a form action routine).

	if (!name) name = "Message";

	var width  = 475;
	var height = 270;

	var settings = options ? options :
	  "resizable,scrollbars,width="+width+",height="+height;

	// Center new window
	var w_width  = window.document.body.offsetWidth;
	var w_height = window.document.body.offsetHeight;
	var top  = window.screenTop  + (w_height/2 - height);
	var left = window.screenLeft + (w_width/2  - width/2);
	settings += ',top='+top+',left='+left;

	var w = window.open("", name, settings);

	// Resize in case an old window opened instead of opening a new one.
	// Arbitrary factors are computed for IE5.
	if (!options) w.resizeTo(width+12, height+44);

	w.focus();

	display_message(w, message, title, url, noprogress);
	return w;
}

function display_message(w, message, title, url, noprogress) {
	// Display message in specified window.

	if (! w) w = self;
	if (! message) message = "";
	if (! title)   title   = "Message";

	var d = w.document;
	d.open();
	d.writeln('<html>');
	d.writeln('<head>');
	d.writeln('<title>' + title + '</title>');
	// The following does not work in IE5.5. Hard-wire the CSS styles.
	//d.writeln('<link rel="stylesheet" href="/styles/message.css" type="text/css">');
	d.writeln('<style>.message{font-family:"Comic Sans MS" serif; font-size:10pt; padding:12pt 2em}</style>');
	d.writeln('<script language="JavaScript" type="text/javascript">');
	d.writeln('window.document.onkeypress = do_keypress;');
	d.writeln('function do_keypress(evt) {');
	d.writeln(' evt = (evt) ? evt : ((window.event) ? window.event : null);');
	d.writeln(' switch (evt.keyCode) {');
	d.writeln(' case 27: self.close(); return true;');
	d.writeln('}}');
	if (url) {

		// Put protocol and hostname in url because Netscape sometimes
		// opens a window with a href of "about:blank" and uses that
		// as a default for protocol and hostname, and then it issues
		// an error for a url like: about:blank/cgi-bin/form.cgi....

		if (! url.match(/:/)) {
			if (! url.match(/^\w+\.\w+/)) {
				url = self.location.hostname +
				  (url.indexOf('/') == 0 ? '' : '/') + url;
			}
			url = self.location.protocol + '//' + url;
		}
		d.writeln('function goto_url() {');
		d.writeln(' self.location.replace("'+url+'");');
		d.writeln('}');
		d.writeln('window.onload=goto_url;');
	}
	d.writeln('</script>');
	d.writeln('</head>');
	d.writeln('<body>');
	d.writeln('<table width=100% border=1 cellspacing=0 cellpadding=10><tr><td>');
	d.writeln('<div class="message">');
	d.writeln('<p>' + message + '</p>');
	d.writeln('<p>Please wait...</p>');
	if (! noprogress) d.writeln(progress_bar('center'));
	d.writeln('</div>');
	d.writeln('</td></tr></table>');
	d.writeln('</body>');
	d.writeln('</html>');
	d.close();
}

function open_window(url,name,options,message,title,noprogress)  {
	  new_window(url,name,options,message,title,noprogress)  }
function open_list(url,name,options,message,title,noprogress)    {
          new_list(url,name,options,message,title,noprogress)    }
function open_view(url,name,options,message,title,noprogress)    {
          new_view(url,name,options,message,title,noprogress)    }
function open_form(url,name,options,message,title,noprogress)    {
          new_form(url,name,options,message,title,noprogress)    }
function open_log(url,name,options,message,title,noprogress)     {
          new_log(url,name,options,message,title,noprogress)     }
function open_log_response(url,name,options,message,title,noprogress) {
          new_log_response(url,name,options,message,title,noprogress) }
function open_audit(url,name,options,message,title,noprogress)   {
          new_audit(url,name,options,message,title,noprogress)   }
function open_options(url,name,options,message,title,noprogress) {
          new_options(url,name,options,message,title,noprogress) }
function open_help(url,name,options,message,title,noprogress)    {
          new_help(url,name,options,message,title,noprogress)    }
function open_doc(url,name,options,message,title,noprogress)     {
          new_doc(url,name,options,message,title,noprogress)     }
function open_browse(url,name,options,message,title,noprogress)  {
          new_browse(url,name,options,message,title,noprogress)  }
function open_print(url,name,options,message,title,noprogress)   {
          new_print(url,name,options,message,title,noprogress)   }
function open_calendar(n_months,url,name,options,message,title,noprogress)  {
	  new_calendar(n_months,url,name,options,message,title,noprogress)  }
function open_message(message, title, url, name, options, noprogress)       {
	  new_message(message, title, url, name, options, noprogress) }

/*
 * Progress Bar
 */

function progress_bar(alignment,color) {
	if (!alignment) alignment = 'center';
	if (!color)     color     = '#CCCCCC';

	return ''
  + '<!-- Begin Progress Bar -->\n'
  + '<table align="'+alignment+'"><tr><td>\n'
  + '<div style="text-align:center; font-size: 8pt;padding:2px">\n'
  + '<span id="progress1">&nbsp; &nbsp; </span>\n'
  + '<span id="progress2">&nbsp; &nbsp; </span>\n'
  + '<span id="progress3">&nbsp; &nbsp; </span>\n'
  + '<span id="progress4">&nbsp; &nbsp; </span>\n'
  + '<span id="progress5">&nbsp; &nbsp; </span>\n'
  + '<span id="progress6">&nbsp; &nbsp; </span>\n'
  + '<span id="progress7">&nbsp; &nbsp; </span>\n'
  + '<span id="progress8">&nbsp; &nbsp; </span>\n'
  + '<span id="progress9">&nbsp; &nbsp; </span>\n'
  + '</div>\n'
  + '</td></tr></table>\n'
  + '<script language="JavaScript" type="text/javascript">\n'
  + 'var progressEnd = 9;\n'
  + 'var progressColor = "'+color+'";\n'
  + 'var progressInterval = 500;\n'
  + '\n'
  + 'var progressAt = 0;\n'
  + 'var progressDir = 1;\n'
  + 'var progressTimer;\n'
  + 'function progress_clear() {\n'
  + '	for (var i = 1; i <= progressEnd; i++) document.getElementById("progress"+i).style.backgroundColor = "transparent";\n'
  + '	progressAt = 0;\n'
  + '	progressDir = 1;\n'
  + '}\n'
  + 'function progress_update() {\n'
  + '	progressAt += progressDir;\n'
  + '	if (progressAt < 1)           { progressAt = 1;           progressDir = 1;  }\n'
  + '	if (progressAt > progressEnd) { progressAt = progressEnd; progressDir = -1; }\n'
  + '	if (progressDir == 1)  document.getElementById("progress"+progressAt).style.backgroundColor = progressColor;\n'
  + '	if (progressDir == -1) document.getElementById("progress"+progressAt).style.backgroundColor = "transparent";\n'
  + '	progressTimer = setTimeout("progress_update()",progressInterval);\n'
  + '}\n'
  + 'function progress_stop() {\n'
  + '	clearTimeout(progressTimer);\n'
  + '	progress_clear();\n'
  + '}\n'
  + 'progress_update()\n'
  + '</script>\n'
  + '<!-- End Progress Bar -->\n';
}
