/*
 * Colorwheel
 */

// Open a popup window with a color wheel so a color can be selected.
// A form field is filled with the picked color. An element can be specified
// so that it's color, background-color, or border-color is set upon return.
// Only the form field argument fld is required. The optional attributes can
// be 'color', 'backgroundColor', or 'borderColor';

var ColorwheelImage = '/images/colorwheel.jpg';

var Colorwheels = new Object;		// saved arguments for colorwheel()

function colorwheel(fld,e, attr1,attr2,attr3) {

	if (typeof fld == 'function') return colorwheel_call(fld);

	// Define popup window name to be same as the form field name.
	wname = (fld && fld.name) || 'color';

	// Save arguments for colorwheel_return()
	if (!Colorwheels[wname]) Colorwheels[wname] = new Object;
	var args = Colorwheels[wname];

	args['field'] = fld;
	args['element'] = e;

	// Scan remaining arguments for style attributes.
	if (arguments.length < 3) args['backgroundColor'] = true; // default
	for (var i = 2; i < arguments.length; i++) {
		if (typeof arguments[i] == 'string') args[arguments[i]] = true;
	}

	var w = args['window'] = new_colorwheel(wname);

	_colorwheel_display(w,fld.value);

	return true;
}

function colorwheel_call(callback) {
	// A version of colorwheel() accepting a callback function.

	var wname = 'color_' + Math.floor(Math.random() * 100);

	// Save arguments for colorwheel_return()
	if (!Colorwheels[wname]) Colorwheels[wname] = new Object;
	var args = Colorwheels[wname];

	args['callback'] = callback;

	var w = args['window'] = new_colorwheel(wname);

	_colorwheel_display(w);

	return true;
}
		
function colorwheel_return(window_name) {
	// This routine gets called from the window that colorwheel()
	// opens. The name of the window that colorwheel() opened is used
	// to reference the arguments that were saved for this routine to use.

	var args = Colorwheels[window_name];
	var w = args['window'];

	var c;		// returned color
	if (w.result) c = w.result['color'];

	var callback = args['callback'];
	if (callback && eval(callback)) {
		// Execute callback function.
		return eval(callback+'(c)');
	}

	if (c) {
		var fld = args['field'];
		var e = args['element'];

		// Fill in form field.
		if (fld) {
			var oldc = fld.value;
			fld.value = c;

			if (fld.onchange && oldc != c) {
				// execute the onChange event function.

				// Hack: substitute 'fld' for any reference
				// to 'this'.

				var func = fld.onchange.toString();
				func = func.replace(/\bthis\b/g,"fld");
				if ( eval(func + ";anonymous()") == false) {
					Colorwheels[window_name] = null;// clear
					fld.value = oldc;
					return false;
				}
			}
		}

		// Set element's color attributes.
		if (e) {
			if (args['color']) e.style.color = c;
			if (args['backgroundColor']) e.style.backgroundColor = c;
			if (args['borderColor']) e.style.borderColor = c;
		}
	}
		
	Colorwheels[window_name] = null;	// clear
}

/*---------------------------------------------------------------------------*/

function new_colorwheel(win_name) {
	
	var settings = "resizable";

	var w = 315;
	var h = 375;

	settings += ",width="+w+",height="+h;

	if (window.event) {
		// Center new window to right of event coord
		var top  = window.screenTop  + (event.clientY - h);
		var left = window.screenLeft + (event.clientX + 25);
		settings += ',top='+top+',left='+left;
	}

	var win = window.open("", win_name, settings);
	win.focus();
	return win;
}

function open_colorwheel(win_name)	{ new_colorwheel(win_name) }

/*---------------------------------------------------------------------------*/

function _colorwheel_display(w,initial_color) {
	if (initial_color == null || ! initial_color.match(/^#?[0-9abcdef]{6}/i))
		initial_color = '#ffffff';

	w.document.open();
	w.document.writeln(
    '<html>\n'
  + '<head>\n'
  + '<title>Colorwheel</title>\n'
  + '<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">\n'
  + '\n'
  + '<style type="text/css">\n'
  + 'BODY TD {font-size: 9pt}\n'
  + '</style>\n'
  + '\n'
  + '<script language="JavaScript" type="text/javascript">\n'
  + '\n'
  + '        result = new Object;	// returned result\n'
  + '\n'
  + '        window.document.onkeypress = keypress;\n'
  + '\n'
  + '        function keypress(evt) {\n'
  + '                var unicode = window.event ? window.event.keyCode : evt.charCode\n'
  + '                switch (unicode) {\n'
  + '                case 27:        // escape key\n'
  + '                        window.close();\n'
  + '                        return true;\n'
  + '                case 10:        // ctrl enter\n'
  + '                        do_exit();\n'
  + '                        return true;\n'
  + '                }\n'
  + '        }\n'
  + '\n'
  + '        function do_exit() {\n'
  + '                // terminate and clean up\n'
  + '                var w = window.opener;\n'
  + '                if (w && ! w.closed) {\n'
  + '                        result.color = self.document.forms[0].color_code.value;\n'
  + '                        w.colorwheel_return(self.name);\n'
  + '                }\n'
  + '                self.close();\n'
  + '        }\n'
  + '\n'
  + '        function change_color(fld,e) {\n'
  + '                var c = fld.value.replace(/#/,"");\n'
  + '                if (c.length != 6 || c.match(/[^0-9aAbBcCdDeEfF]/)) {\n'
  + '                        alert("Not a valid color code.");\n'
  + '                        fld.select();\n'
  + '                        fld.focus();\n'
  + '                        return false;\n'
  + '                }\n'
  + '                fld.value = "#" + c;\n'
  + '\n'
  + '                if (e) e.style.backgroundColor = fld.value;\n'
  + '                return true;\n'
  + '        }\n'
  + '</script>\n'
  + '<script language="JavaScript" type="text/javascript">\n'
  + _colorwheel_javascript()
  + '</script>\n'
  + '</head>\n'
  + '\n'
  + '<body bgcolor="' + initial_color + '" onLoad="capture()">\n'
  + '\n'
  + '<table border=0 cellpadding=0 cellspacing=0 align="center">\n'
  + '<tr>\n'
  + '  <td align="center">Click on a Color</td>\n'
  + '</tr>\n'
  + '<tr>\n'
  + '  <td>\n'
  + '  <div id=wheel style="position:relative; visibility:visible;">\n'
  + '  <img src="' + ColorwheelImage + '" width=256 height=256 border=0>\n'
  + '  </div>\n'
  + '  </td>\n'
  + '</tr>\n'
  + '<tr>\n'
  + '  <td align="center">\n'
  + '  <form name="f"\n'
  + '   action="javascript:do_exit()"\n'
  + '   onSubmit="return change_color(this.color_code,document.body)"\n'
  + '   style="margin:0px 0px 1ex 0px"\n'
  + '  >\n'
  + '  <br><nobr>'
  + '  <a href="javascript:void change_color(document.f.color_code,document.body)"'
  + ' title="Set color">Set</a>'
  + '  <input type="text" name="color_code" value="' + initial_color + '"'
  + ' size="7" onChange="change_color(this,document.body)"'
  + ' onDblClick="if(window.event)window.event.cancelBubble=true">\n'
  + '  <input type="image" style="vertical-align:top"'
  + ' src="/images/undo.gif" title="Undo change."'
  + ' onMouseOver="this.src=\'/images/undo_hilite.gif\'"'
  + ' onMouseOut="this.src=\'/images/undo.gif\'"'
  + ' onClick="document.f.color_code.value=\''+initial_color+'\';'
  + 'change_color(document.f.color_code,document.body);'
  + 'if(window.event)window.event.cancelBubble=true;return false" tabindex="-1">\n'
  + '  </nobr>\n'
  + '  </form>\n'
  + '  </td>\n'
  + '</tr>\n'
  + '<tr>\n'
  + '  <td align="center">\n'
  + '  <input type="button" value="Save " style="font-size:8pt" onClick="do_exit()">'
  + '  &nbsp;'
  + '  <input type="button" value="Close" style="font-size:8pt" onClick="self.close()">'
  + '  </td>\n'
  + '</tr>\n'
  + '</table>\n'
  + '\n'
  + '</body>\n'
  + '</html>\n'
  );
        w.document.close();
}

function _colorwheel_javascript() {
        return '\n'
  + '/********************************************************\n'
  + ' * Original:  D10n (biab@iinet.net.au)\n'
  + ' * Web Site:  http://www.iinet.net.au/~biab\n'
  + ' * The JavaScript Source!! http://javascript.internet.com\n'
  + ' ********************************************************/\n'
  + '\n'
  + 'addary = new Array();           //red\n'
  + 'addary[0] = new Array(0,1,0);   //red green\n'
  + 'addary[1] = new Array(-1,0,0);  //green\n'
  + 'addary[2] = new Array(0,0,1);   //green blue\n'
  + 'addary[3] = new Array(0,-1,0);  //blue\n'
  + 'addary[4] = new Array(1,0,0);   //red blue\n'
  + 'addary[5] = new Array(0,0,-1);  //red\n'
  + 'addary[6] = new Array(255,1,1);\n'
  + 'clrary = new Array(360);\n'
  + 'for(i = 0; i < 6; i++) {\n'
  + '	for(j = 0; j < 60; j++) {\n'
  + '		clrary[60 * i + j] = new Array(3);\n'
  + '		for(k = 0; k < 3; k++) {\n'
  + '			clrary[60 * i + j][k] = addary[6][k];\n'
  + '			addary[6][k] += (addary[i][k] * 4);\n'
  + '		}\n'
  + '	}\n'
  + '}\n'
  + '\n'
  + 'function capture() {\n'
  + '	if(document.layers) {\n'
  + '		layobj = document.layers["wheel"];\n'
  + '		layobj.document.captureEvents(Event.MOUSEMOVE);\n'
  + '		layobj.document.onmousemove = moved;\n'
  + '		layobj.document.onclick = do_exit;\n'
  + '	}\n'
  + '	else {\n'
  + '		layobj = document.getElementById("wheel");\n'
  + '		layobj.onmousemove = moved;\n'
  + '		layobj.onclick = do_exit;\n'
  + '           document.body.ondblclick = do_exit;\n'
  + '	}\n'
  + '}\n'
  + '\n'
  + 'function moved(e) {\n'
  + '	y = 4 * ((document.layers)?e.layerX:(window.event?event.offsetX:e.layerX));\n'
  + '	x = 4 * ((document.layers)?e.layerY:(window.event?event.offsetY:e.layerY));\n'
  + '	sx = x - 512;\n'
  + '	sy = y - 512;\n'
  + '	qx = (sx < 0)?0:1;\n'
  + '	qy = (sy < 0)?0:1;\n'
  + '	q = 2 * qy + qx;\n'
  + '	quad = new Array(-180,360,180,0);\n'
  + '	xa = Math.abs(sx);\n'
  + '	ya = Math.abs(sy);\n'
  + '	d = ya * 45 / xa;\n'
  + '	if(ya > xa) d = 90 - (xa * 45 / ya);\n'
  + '	deg = Math.floor(Math.abs(quad[q] - d));\n'
  + '	n = 0;\n'
  + '	sx = Math.abs(x - 512);\n'
  + '	sy = Math.abs(y - 512);\n'
  + '	r = Math.sqrt((sx * sx) + (sy * sy));\n'
  + '	if(x == 512 & y == 512) {\n'
  + '		c = "000000";\n'
  + '	}\n'
  + '	else {\n'
  + '		for(i = 0; i < 3; i++) {\n'
  + '			r2 = clrary[deg][i] * r / 256;\n'
  + '			if(r > 256) r2 += Math.floor(r - 256);\n'
  + '			if(r2 > 255) r2 = 255;\n'
  + '			n = 256 * n + Math.floor(r2);\n'
  + '		}\n'
  + '		c = n.toString(16);\n'
  + '		while(c.length < 6) c = "0" + c;\n'
  + '	}\n'
  + '	if(document.layers) {\n'
  + '		document.layers["wheel"].document.f.color_code.value = "#" + c;\n'
  + '		document.layers["wheel"].bgColor = "#" + c;\n'
  + '	}\n'
  + '	else {\n'
  + '		document.f.color_code.value = "#" + c;\n'
  + '		//document.getElementById("wheel").style.backgroundColor = "#" + c;\n'
  + '		document.body.style.backgroundColor = "#" + c;\n'
  + '	}\n'
  + '	return false;\n'
  + '}\n';
}
