/*
 * Process form events
 */

/*
 * Note: It is desired that the fields' text is selected when an input field
 * gets the focus. The field gets the focus 2 ways: tabbing to the field and
 * clicking on the field. Selecting the text when tabbing to the field needs
 * to be set for each field element as the document object does not recognize
 * the onfocus property and you cannot set the window.onfocus property. (Setting
 * individual fields onfocus properties cannot be done here if this code is
 * included in the <head> section as the form fields haven't been defined yet.)
 * Selecting the text when the user clicks on it needs to be set here after
 * other field events in the input field deselect the text selected in the
 * onFocus event. But that is still funky in IE4 as onFocus is fired after
 * onClick when the onClick event selects the text.
 */

/* 8/3/2000  Update to IE5.5 */
   
/*
 * Note: Shift-Tab looses the index for Enter processing. To correct, I have
 * to capture Tab events and loose all special processing. All I wanted to do
 * is capture Enter keys and change them to a Tab key. I can, but I loose all
 * field migration characteristics of the Tab key. (!!)
 */

/*
 * In order to provide a "function key" access to the console screen, I use
 * Ctrl-space as the function key as most other logical "control" characters
 * are not available for capture. (!!)
 */

/*
 * Image buttons are not considered to be Submit buttons. To submit a form,
 * image buttons must have an explicit onClick event handler that calls
 * submit(). This "feature" should be removed.
 */

/*
 * Scripts can install handlers for the enter key and the control enter key
 * keyCode events simply by defining functions with the names:
 *
 *   enterKey_form_name()
 *    -or-
 *   ctrlEnterKey_form_name()
 *
 * where the "form_name" is the name of the form receiving the keyboard events.
 */

/* 1/2/2002
 * Added dblclick() event handler to open up an audit window when the user
 * double clicks on a field label. Audit window is only opened if there is
 * a function open_audit() defined elsewhere, such as in /scripts/common.js.
 * The program path name is defined below. The parameters "field" and "keyval"
 * are automatically appended to the Audit_Prog path.
 */


var Audit_Prog = "/cgi-bin/audit.cgi";

/*
 * Note: Pressing Enter to go from field to field will end up on the next field
 * even though the onChange event returns false. There is no way I know yet
 * to execute the onChange event in the Enter key event without having the
 * onChange event from being re-executed by IE5 if the onChange event did NOT
 * return false. Now, if the onChange event returns false, the field focus
 * incorrectly ends up on the next field in order because the keypress event
 * handler is called before the onChange handler.
 */

/* 2/20/2003 Update to NS6 */

/* -------------------------------------------------------------------------- */

function mouseclick(evt) {
	evt = (evt) ? evt : ((window.event) ? window.event : null);
	var me = evt.target ? evt.target : evt.srcElement ? evt.srcElement : null;

	if (typeof me != "object") return true;// events not supported by browser
	if (!me.type) return true;	// user clicked outside of fields

	if (me.readOnly) return true;	// do not select read only fields
	if (me.disabled) return true;	// do not select disabled fields

	var rc = true;		// return code

// IE5 already fires element's onClick events before getting here, do not repeat
//	if (me && me.onclick && me.type != 'button') {
//		rc = false;
//		var func = me.onclick.toString();
//		func = func.replace(/\bthis\b/gi,"me");
//		if ( eval(func + ";anonymous()")) {
//			rc = true;
//		}
//	}

	switch (me.type) {
	  case "text":
	  case "textarea":
	  case "password":
	  case "file":
//IE4		me.select();		// select text in input field
					// (triggers onfocus)
		find_me(me, me.form);	// maintain current index for tabbing
		return rc;

	  case "checkbox":
	  case "radio":
	  case "select-one":
	  case "select-multiple":
		me.focus();		// conflicts with HideMoreLess buttons
		find_me(me, me.form);	// maintain current index for tabbing
		return rc;

	  case "button":
	  case "reset":
//		me.focus();		// conflicts with HideMoreLess buttons
		find_me(me, me.form);	// maintain current index for tabbing
		return rc;

	  case "image":
//		return false;		// disable default submit action
		return true;		// default action

	  case "submit":
		/*
		 * Note: Convention for Internet form processing is for the
		 * form to be submitted when the user presses Enter.
		 * IE4 implements that feature by changing the Enter key to a
		 * mouse click event on the first submit button in the form if
		 * the element type is undefined. In other words, control
		 * passes here if the user presses the Enter key after clicking
		 * on the form outside of any fields. (!!)
		 * (to be delt with later -- hopefully by Microsoft!)
		 */
		return rc;
	}

	return rc;
}

/* -------------------------------------------------------------------------- */

function dblclick(evt) {
	// Open audit window if double clicked on a field label.

	evt = (evt) ? evt : ((window.event) ? window.event : null);
	var me = evt.target ? evt.target : evt.srcElement ? evt.srcElement : null;

	// Look for first container element of event with an ID field
	// and an Id label beginning with 'l_'.
	while (me && !(me.id && me.id.match(/^l_/))) {
		me = me.parentNode;
	}

	if (!me) return;

	var myid = me.id.replace(/^l_/,'');

	// Ignore labels of fields that are not table and fields.
	if (!myid.match(/__/)) return;

	// Find form name
	while (me) {
		if (me.nodeName == 'FORM') break;
		me = me.parentNode;
	}

	if (!me) return;

	// Find name of key field and get the current value of the key.
	var key = me['_key'];
	var keyval;
	if (me['_table'] && me['_key']) {
		var keyfield = me['_table'].value + '__' +  me['_key'].value;
		if (me[keyfield] && me[keyfield].value) {
			keyval = me[keyfield].value;
		}
	}

	if (!keyval) return;	// Record is probably new and not saved

	// The open_audit() function is defined in /scripts/common.js
	if (self.open_audit) {
		var url = Audit_Prog + "?field=" + myid + "&keyval=" + keyval;
		open_audit(url,"audit_"+myid);
	}
}

/* -------------------------------------------------------------------------- */

function keypress(evt) {
	evt = (evt) ? evt : ((window.event) ? window.event : null);
	var me = evt.target ? evt.target : evt.srcElement ? evt.srcElement : null;

	// Note: Netscape 7 submits a form (e.g. processes the onsubmit event)
	// before passing the event to this handler. (dumb!) That subverts
	// processing the enter key to go to the next field.

	switch (evt.keyCode) {
	case 32:		// space key
		/*
		 * Ctrl-Space bar displays the console screen.
		 */
		if (evt.ctrlKey) {
			if (typeof window.console == "function") {
				console();
				return false;
			}
		}
		return true;

	case 27:	// escape key
		if (me) {
			var f = me.form;	// current form
			if (typeof window.form_cancel == 'function') {
					return form_cancel(f);
			}
		}

		window.close();
		return true;

	case 13:	// enter key
		if (!me) return true;		 // nothing loaded

		/* In IE5, clicking in the white space of a form and then
		 * pressing Enter automatically posts the form. Disable and
		 * force the user to use CTRL-Enter to submit the form.
		 */
		var f = me.form;		// current form
		if (!f) return false;		// not a form, cancel
						// otherwise IE5 submits it (!!)

		/*
		 * The convention is that Ctrl-Enter posts a form. However, we
		 * don't check for the evt.ctrlKey property here as on a
		 * PC the Ctrl-Enter transmits a LF, not a CR which is trapped
		 * here. See case 10: below.
		 */

		/*
		 * Shift-Enter in text or textarea field transmits a CR.
		 * No special processing done.
		 */

		if (evt.shiftKey) {
			if (me.type == "textarea") {
				// default processing enters CR into field
				return true;		// default processing
			}

			/*
			 * Cannot enter a CR into a text field. Returning
			 * false cancels form submit, but also cancels text
			 * entry into field.
			 */

			if (me.type == "text") {
				// Text fields need to be tricked into entering
				// a CR into the field.
				return false;		// cancel submit
			}
		}

		/*
		 * Special processing of enter key if there exists a handler
		 * with the special name of 'enterKey_' plus the form name.
		 */

		var func = 'enterKey_'+f.name;
		if (eval('self.'+func)) {
			var rc = eval(func+'()');

			if (rc == true) return true;
			if (rc != null) return false;

			// undefined return codes fall through for
			// normal tab key processing
		}

		/* 
		 * Default processing of buttons means activating them.
		 * Intercept event and space through them like input fields.
		 *
		 * if (me.type == "button") return true;// default processing
		 * if (me.type == "reset")  return true;// default processing
		 */

		if (me.type == "submit") return true;	// default processing

		// Go to next element in tabbing order. (fake it...)
		if (next_me(me,f)) return false;

		return true;	// Should never get here, but can if fields
				// are readOnly or disabled.

	case 10:	// ctrl-enter key
		/*
		 * Note: a CTRL-Enter key sequence on PC keyboard is
		 * transmitted as a LineFeed (10).
		 */

		/* Submit form.
		 * Note: in IE5, if the user clicks in the whitespace outside
		 * the form fields, the form id is no longer available. (!!)
		 */

		if (me.onchange) {
			// execute the onChange event function first

			/*
			 * Note: the onsubmit property is an anonymous function
			 * definition enclosing the real onSubmit text defined
			 * in the form. To execute it, we define the function
			 * anonymous in the eval() and immediately execute it
			 * (before function definition goes out of scope).
			 * Also, unless the onchange function specifically
			 * returns false, the result is assumed to be true.
			 * So onchange event handlers that do not return a
			 * value are considered to be successful and the
			 * form is posted.
			 */

			/*
			 * Note: this should be called like Javascript calls
			 * an event handler. But I haven't figured that out yet
			 * so I just substitute 'me' for any reference
			 * to 'this'. (use me.onchange.call(me) ???)
			 */

			var func = me.onchange.toString();
			func = func.replace(/\bthis\b/g,"me");
			if ( eval(func + ";anonymous()") == false) {
				return false;	// cancel submit
			}
		}

		var f = me.form;
		// Note: f is undefined for non-form elements (e.g. <a> tags)
		if (f) {
			/*
			 * Special processing of control enter key if there exists
			 * a handler with the special name of 'ctrlEnterKey_' plus
			 * the form name.
			 */

			var func = 'ctrlEnterKey_'+f.name;
			if (eval('self.'+func)) {
				eval(func+'()')
				return false;
			}

			if (f.onsubmit) {
				// execute the onSubmit event function

				/*
				 * Note: the onsubmit property is an anonymous
				 * function definition enclosing the real
				 * onSubmit text defined in the form. To
				 * execute it, we define the function
				 * anonymous in the eval() and immediately
				 * execute it (before function definition goes
				 * out of scope).
				 */

				// As above, substitute 'f' for any reference
				// to 'this'.

				var func = f.onsubmit.toString();
				func = func.replace(/\bthis\b/gi,"f");
				if ( eval(func + ";anonymous()") ) {
					f.submit(); 	// post form
				}
			}
			else {
				f.submit();		// post form
			}
		}
		return true;		// keep default action for event
	}
	return true;
}

/* -------------------------------------------------------------------------- */

var curr_i = 0;	// current field index
		// kept as global to speed up searches

function find_me(e, f) {
	// find current element index in form f.
	// Note: This maintains the curr_i global index.

	var i;
	var found_me = false;	//flag
	if (f) {		// make sure element is within a form
		for (i=curr_i; i < f.elements.length; i++) {
			if (f.elements[i] == e) {
				curr_i = i;
				found_me = true;
				break;
			}
		}
		if (! found_me) {
			for (i=0; i < curr_i; i++) {
				if (f.elements[i] == e) {
					curr_i = i;
					found_me = true;
					break;
				}
			}
		}
	}
	if (! found_me)
		curr_i = -1;	// initialize for next loop
}

function next_me(e, f) {
	// Note: This maintains the curr_i global index.

	var next_tabIndex = 999999;	// initialize to large
	var next_i;
	var i;
	var check_e;

	if (e.tabIndex >= 0) {
		// find next lowest tabIndex value
		if (curr_i > 0) curr_i--;	// take a running start
		for (i=curr_i; i < f.elements.length; i++) {
			check_e = f.elements[i];
			if (check_e.tabIndex != null
			&& ! check_e.readOnly
			&& ! check_e.disabled
			&& ! (check_e.type == 'hidden')
			&& ! (check_e.type == 'radio' && e.type == 'radio'
			   && check_e.name == e.name)
			&& (check_e.tabIndex < next_tabIndex && check_e.tabIndex > e.tabIndex)
			&& is_visible(check_e)){
				next_i = i;
				next_tabIndex = check_e.tabIndex;
			}

			// save current element index for below
			if (check_e == e) curr_i = i;
		}

		if (next_i != null) {
			// next tabIndex found
			if (f.elements[next_i].type == "text") {
				f.elements[next_i].select();
				f.elements[next_i].focus();
			}
			else {
				unselect_me(e);
				f.elements[next_i].focus();
			}
			curr_i = next_i;
			return true;
		}

	}
	else {
		find_me(e,f);
	}

	// next tabIndex not found
	// get next element in source code order
	for (i=curr_i+1; i < f.elements.length; i++) {
		check_e = f.elements[i];
		if (check_e.readOnly) continue;
		if (check_e.disabled) continue;
		if (check_e.tabIndex < 0) continue;
		if (!is_visible(check_e)) continue;

		switch (check_e.type) {
		  case "text":
		  case "textarea":
		  case "password":
		  case "file":
			unselect_me(e);
			check_e.focus();	// scroll window
// The following causes the next (wrong) field to be selected if an onChange
// event returns false, because the onChange gets executed after this routine.
//			check_e.select();
			curr_i = i;
			return true;

		  case "radio":
			if (e.type == 'radio' && check_e.name == e.name) continue;
			unselect_me(e);
			check_e.focus();
			curr_i = i;
			return true;

		  case "button":
		  case "checkbox":
		  case "select-one":
		  case "select-multiple":
		  case "reset":
		  case "submit":
			unselect_me(e);
			check_e.focus();
			curr_i = i;
			return true;
		}
	}

	// if still here, we need to wrap around to beginning
	for (i=0; i < curr_i; i++) {
		check_e = f.elements[i];
		if (check_e.readOnly) continue;
		if (check_e.disabled) continue;
		if (check_e.tabIndex < 0) continue;
		if (!is_visible(check_e)) continue;

		switch (check_e.type) {
		  case "text":
		  case "textarea":
		  case "password":
		  case "file":
			unselect_me(e);
			check_e.focus();	// scroll window
// The following causes the next (wrong) field to be selected if an onChange
// event returns false, because the onChange gets executed after this routine.
//			check_e.select();
			curr_i = i;
			return true;

		  case "radio":
			if (e.type == 'radio' && check_e.name == e.name) continue;
			unselect_me(e);
			check_e.focus();
			curr_i = i;
			return true;

		  case "button":
		  case "checkbox":
		  case "select-one":
		  case "select-multiple":
		  case "reset":
		  case "submit":
			check_e.focus();
			curr_i = i;
			return true;
		}
	}
	return false;		// next element not found
}

function unselect_me(e) {
	// Remove selection (hilited text) from element

	// This is a IE5 work-around for moving hilited field from a text
	// field to a selection field, which does not remove the hilite from
	// the text field.

	if (e.isTextEdit) {
		var te = e.createTextRange();
		te.collapse(true);
		te.select();
	}
}

function is_visible(e) {
	// Check visibility of element's containers
	while (e && (e.style == null
	   || (e.style.visibility == null || e.style.visibility == 'visible'))) {
		e = e.parentNode;
	}
	if (!e) return false;

	// Check display setting of container nodes.
	while (e && (e.style == null
	   || (e.style.display == null || e.style.display != 'none'))) {
		e = e.parentNode;
	}

	if (!e) return true;

	return false;
}

function form_events_setup() {

	// Initialize form events.
	// This is used when forms are re-initialized for the next entry.

	curr_i = 0;	// reset global field index
}

/* -------------------------------------------------------------------------- */

/*
 * Capture events
 */

window.document.ondblclick = dblclick;		// capture mouse double clicks

if (document.all
&& navigator.userAgent.indexOf("Opera") == -1
&& navigator.userAgent.indexOf('Mac') == -1) {
	// Use only with IE5+ for PC at this time ....
	window.document.onclick = mouseclick;		// capture mouse clicks
	window.document.onkeypress = keypress;		// capture keystrokes
}

