/*
 * Form Validate Input
 *
 * Field input is checked and reformatted if neccessary
 */

var Default_Form = 0;	// form to use by default if not specified below
var DS = "-";		// date separator string between date components
			// the - is the default of the Postgres database

function translate_input(input_field, func_meth) {

	// Note: in IE5, changing the value of input_field causes the
	// onChange event to act like it returns true, no matter what we
	// (or the user's function) returns.

	// Check if func_meth is a defined function.
	// Strip () before checking existance.
	var func = func_meth.replace(/\(.*/,"");
	if (eval("typeof "+func) == "function") {
		// Substitute "this" in function declaration to "input_field".
		func = func_meth.replace(/\bthis\b/g,"input_field");
		return eval(func);
	}

	// Assume func_meth is a string method
	// Append () before executing if not already there.
	if (! func_meth.match(/\(/)) func_meth += "()";
	var s = new String(input_field.value);
	input_field.value = eval("s." + func_meth);
	return true;
}

function check_input(input_field, type_class, maxlength, decimals) {

	switch(type_class) {
		case 'text':
			return checkText(input_field, maxlength);
		case 'date':
			return checkDate(input_field);
		case 'time':
			return checkTime(input_field);
		case 'money':
			return checkMoney(input_field, decimals);
		case 'number':
			return checkNumber(input_field, 0);
		case 'id':
			return checkNumber(input_field, 0);
		case 'decimal':
			return checkNumber(input_field, decimals);
		case 'phone':
			return checkPhone(input_field);
		case 'url':
			return checkUrl(input_field);
		case 'email':
			return true;		// TODO
		case 'color':
			return checkColor(input_field);
		case 'date_range':
			return checkDateRange(input_field);
		default:
			return true;
	}
}

function validate_input(input_field, condition, error_message) {
	if (condition == true) return true;
	if (error_message) {
		alert(error_message);
	}
	else {
		// Default error message
		alert(input_field.value + " is not a valid entry.\nPlease re-enter.");
	}
	input_field.select();
	input_field.focus();
	return false;
}

/* ------------------------------------------------------------------------- */

function checkText(input_field, maxlength) {
	if (maxlength == null) maxlength = 255;

	if (input_field.value.length > maxlength) {
		alert ("Field entry is too long.\n\n" +
		  "It will be truncated to " + maxlength + " characters.");
		input_field.value = input_field.value.substr(0,maxlength);
		return false;
	}
	return true;
}

function checkNumber(input_field, decimals) {
	// Validate a number entered into a form field and redisplay it.

	// The decimals argument determines the number of digits displayed
	// after the decimal point. If missing or less than 0, the number
	// is treated as a floating point number and the number of digits
	// after the decimal point is not adjusted.

	if (decimals == null || decimals < 0) {
		if (input_field.value == "") {
			input_field.value = "0";
			return true;
		}

		// Redisplay number in field
		input_field.value = parseFloat(input_field.value);
		return true;
	}
	else {
		var v;
		if (decimals == 0) {
			v = parseInt(input_field.value,10);
		}
		else {
			v = parseFloat(input_field.value);
		}

		if (isNaN(v)) v = 0;

		// Redisplay number in field
		input_field.value = roundValue(v,decimals);
		return true;
	}
}

function checkMoney(input_field, decimals) {
	// Validate a monitary value and redisplay it.

	// This routine simple strips off any $ or commas the user may have
	// entered and then checks it as a number.

	input_field.value = input_field.value.replace(/[$,]/g,"");

	return checkNumber(input_field, decimals);
}

function checkDate(input_field) {
	// Validate a date entered into a form input field and redisplay it.
	// Two digit years less than 30 have 2000 added, otherwise 1900 is added.

	if (input_field.value == "") return true; 	// ignore null values

	var s = dateOf(input_field.value);

	if (s) {
		// Redisplay date 
		input_field.value = s;
		return true;
	}
	else {
		alert("'" + input_field.value + "' is not recognized as a date.");
		input_field.select();
		input_field.focus();
		return false;
	}
}

function checkTime(input_field) {
	// Validate a time entered into a form input field and redisplay it.

	// Alter time strings hhmm to hh:mm before parsing
	// Accept strings like 1a or 12pm.

	if (input_field.value == "") return true; 	// ignore null values

	var s = timeOf(input_field.value);

	if (s) {
		// Redisplay time
		input_field.value = s;
		return true;
	}
	else {
		alert("'" + input_field.value + "' is not recognized as a time value.");
		input_field.select();
		input_field.focus();
		return false;
	}
}

function checkPhone(input_field) {
	// Validate a phone number entered into a form field and redisplay it.

	if (input_field.value == "") return true; 	// ignore null values

	var s = new String(input_field.value);
	var ns = s;
	if (s.match(/^\d+$/)) {
		// put hyphens in phone number
		if (s.match(/^011/)) {
			// international phone number
			input_field.value = s.replace(
				/(011)(\d\d)(\d\d)(\d\d\d)(\d*)/,
				"$1-$2-$3-$4-$5"
				);
		}
		else {
			// domestic phone number
			// work from end of string to beginning
			ns = s.replace(/(.*)(\d\d\d\d)$/,"$1-$2");

			s = ns;
			while ((ns = s.replace(/(.*[^-])(\d\d\d-.*)$/, "$1-$2")) != s) {
				s = ns;
			}

			input_field.value = s;	// redisplay reformatted number
		}
	} 

	return true;
}

function checkUrl(input_field) {
	// Validate a url entered into a form field and redisplay it.

	if (input_field.value == "") return true; 	// ignore null values

	var s = new String(input_field.value);

	if (! s.match(/http/i)) { input_field.value = s.replace(/^\s*/,"http://") }

	return true;
}

function checkColor(input_field) {
	// Validate a color code entered into a form field and redisplay it.

	if (! input_field.value) return true;

	if (! input_field.value.match(/^#?([0-9aAbBcCdDeEfF]{6})/)) {
		alert("'" + input_field.value + "' is not a recognized color code.");
		input_field.select();
		input_field.focus();
		return false;
	}

	input_field.value = "#" + RegExp.$1;	// redisplay

	return true;
}

function checkDateRange(input_field) {
	// Validate a date range.
	// Date ranges are used on Query screens.

	if (! input_field.value) return true;

	if (input_field.value.match(/(.*):(.*)/)) {
		var d2 = RegExp.$2;
		input_field.value = dateOf(RegExp.$1) + ":" + dateOf(d2);
	}
	else if (input_field.value.match(/^(\s*[<>!=][=]?\s*)(\d\d\d\d(\d\d)?)/)) {
		input_field.value = RegExp.$1 + dateOf(RegExp.$2);
	}
	else if (input_field.value.match(/^\s*(\d\d\d\d(\d\d)?)/)) {
		input_field.value = dateOf(RegExp.$1);
	}
	return true;
}

/* -------------------------------------------------------------------------- */

/*
 * Change strings into valid date or time strings.
 */

// The following Date_Order can have one of three values. mdy, dmy, and ymd.
// The mdy setting is assumed by the server software and Date_Order is
// reassigned later if different. Note that it is the browser that parses the
// date and validates it, and it usually uses the client machine's date settings,
// so this all gets complicated....

var Date_Order = 'mdy';		// default order of dates in strings.

function dateOf(s) {
	// Validate and return a date string.
	// Two digit years less than 30 have 2000 added, otherwise 1900 is added.

	// Alter date strings mmddyy to mm/dd/yy before parsing
	var cd, y;

	var ds = s;
	if (s.match(/^\s*[+-]\d+$/)) {			// relative number
		ds = dayadd(parseInt(s,10));
	}
	else if (s.match(/^0+$/)) {			// only a 0
		var now = new Date();
		var m = now.getMonth() + 1;
		ds = m + "/" + now.getDate() + "/" + now.getFullYear();
	}
	else if (s.match(/^\d+$/)) {			// only a number
		if (s.length == 4) {
			// append current year
			var now = new Date();

			if (Date_Order == 'ymd') {
				ds = now.getFullYear() + "/" +
					s.substring(0,2) + "/" +
					s.substring(2,4);
			}
			else {
				ds = s.substring(0,2) + "/" +
					s.substring(2,4) + "/" +
					now.getFullYear();
			}
		}
		else {
			if (Date_Order == 'ymd') {
				y = parseInt(s.substring(0,4),10);
				if (y < 30) y += 2000;
				ds = y + "/" +
					s.substring(0,2) + "/" +
					s.substring(2,4);
			}
			else {
				y = parseInt(s.substring(4,s.length),10);
				if (y < 30) y += 2000;
				ds = s.substring(0,2) + "/" +
					s.substring(2,4) + "/" +
					y;
			}
		}
	}
	else {
		var re1 = /^(\d+)\W+(\d+)\W+(\d+)$/;	// 3 separated numbers
		if (s.match(re1)) {
			if (Date_Order == 'ymd') {
				y = parseInt(RegExp.$1,10);
				if (y < 30) y += 2000;
				ds = s.replace(re1, y+"/$1/$2");
			}
			else {
				y = parseInt(RegExp.$3,10);
				if (y < 30) y += 2000;
				ds = s.replace(re1, "$1/$2/")+y;
			}
		}
		else {
			var re2 = /^(\d+)\W+(\d+)$/;	// 2 separated numbers
			if (s.match(re2)) {
				var now = new Date();

				if (Date_Order == 'ymd') {
					ds = s.replace(re2, now.getFullYear() + "/$1/$2");
				}
				else {
					ds = s.replace(re2, "$1/$2/" + now.getFullYear());
				}
			}
		}
	}
	cd = new Date(ds);

	var validDate = true;
	if (cd == "Invalid Date") validDate = false;	// Netscape
	if (cd == "NaN")          validDate = false;	// MS IE4

	if (!validDate) return '';

	// Redisplay date 
	var y = cd.getFullYear();
	var m = cd.getMonth() + 1; 
	var d = cd.getDate();
	if (m < 10) m = "0" + m;
	if (d < 10) d = "0" + d;
	if (Date_Order == 'ymd') return y + DS + m + DS + d;
	if (Date_Order == 'dmy') return d + DS + m + DS + y;
	return m + DS + d + DS + y;
}

function timeOf(s) {
	// Validate and return a time string.

	// Alter time strings hhmm to hh:mm before parsing
	// Accept strings like 1a or 12pm.
	if (!s || s == 0) return "00:00";

	var ds;

	if (s.match(/^\s*(\d+)[^:]*$/)) {
		ds = RegExp.$1;
		var l = ds.length;
		if (l <= 2) {
			ds = ds + ':00';
		}
		else if (l == 3) {
			ds = ds.replace(/(\d)/,'$1:');
		}
		else {
			ds = ds.replace(/(\d\d)/,'$1:');
		}
		s = s.replace(/\d+/,ds);
	}

	var pm_adjust = 0;
	if (s.match(/\s*pm?/i)) pm_adjust = 12;

	s.match(/(\d+):([\d.]+)/);

	var h = parseInt(RegExp.$1,10);
	var m = parseFloat(RegExp.$2,10);

	var validTime = true;
	if (isNaN(h)) validTime = false;
	if (isNaN(m)) validTime = false;

	if (h >= 12) pm_adjust = 0;
	if (h > 24) validTime = false;
	if (h == 24) h = '00';
	if (m < 0 || m >= 60) validTime = false;
	
	if (!validTime) return '';

	if (pm_adjust > 0) h += pm_adjust;
	if (m < 10) m = '0' + m;
	return h + ":" + m;
}

/* -------------------------------------------------------------------------- */

/*
 * Set keyboard focus to first field that can receive focus.
 */

// HTML Field types which can receive keyboard focus.
var Can_Focus = {
	'text':            true,
	'textarea':        true,
	'checkbox':        true,
	'radio':           true,
	'select-one':      true,
	'select-multiple': true,
	'password':        true,
	'file':            true};

function focus_first_field(f) {
	if (f == null) { f = self.document.forms[Default_Form] }

	for (var i=0; i < f.elements.length; i++) { 
		if (Can_Focus[f.elements[i].type]) {
			f.elements[i].focus();
			if (f.elements[i].select) f.elements[i].select();
			return true;
		}
	}
}

function focus_first_blank_field(f) {
	if (f == null) { f = self.document.forms[Default_Form] }
	for (var i=0; i < f.elements.length; i++) { 
		if (Can_Focus[f.elements[i].type]) {
			if (is_empty(f.elements[i].value)) {
				f.elements[i].focus();
				if (f.elements[i].select) f.elements[i].select();
				return true;
			}
		}
	}
}

/* -------------------------------------------------------------------------- */

/*
 * Hide and show form fields and their label if one can be identified with
 * a name derived from the field name.
 */

var Field_Prefix = 'f_';
var Label_Prefix = 'l_';

function show_fields() {
	for (i = 0; i < arguments.length; i++) {
		if (typeof arguments[i] == 'object') {
			show_field(arguments[i]);
		}
	}
}

function show_field(input_field) {
	if (! typeof input_field == 'object') return;
	var e;
	if (input_field.type != 'radio') {
		input_field.style.display='inline';
	}
	else {	// Radio buttons
		for (var i = 0; i < input_field.length; i++) {
			input_field[i].style.display='block';
		}

		// Show table enclosing radio buttons
		e = input_field[0].parentNode.parentNode.parentNode;
		if (e && e.style) e.style.display='block';
	}

	// Show field container (and associated buttons)
	e = document.getElementById(Field_Prefix+input_field.name);
	if (e && e.style) e.style.display='inline';

	// Show field label
	e = document.getElementById(Label_Prefix+input_field.name);
	if (e && e.style) e.style.display='inline';
}

function hide_fields() {
	for (i = 0; i < arguments.length; i++) {
		if (typeof arguments[i] == 'object') {
			hide_field(arguments[i]);
		}
	}
}

function hide_field(input_field) {
	if (! typeof input_field == 'object') return;
	var e;
	if (input_field.type != 'radio') {
		input_field.style.display='none';
	}
	else {	// Radio buttons
		for (var i = 0; i < input_field.length; i++) {
			input_field[i].style.display='none';
		}

		// Hide table enclosing radio buttons
		e = input_field[0].parentNode.parentNode.parentNode;
		if (e && e.style) e.style.display='none';
	}

	// Hide field container (and associated buttons)
	e = document.getElementById(Field_Prefix+input_field.name);
	if (e && e.style) e.style.display='none';

	// Hide field label
	e = document.getElementById(Label_Prefix+input_field.name);
	if (e && e.style) e.style.display='none';
}

/* -------------------------------------------------------------------------- */

/* 
 * This revert() function supports the 'multiple' option of
 * Acme::HTML::ListMenus which produces a 'select-multiple' type of
 * list that the user can use to select multiple values.
 *
 * The revert() function is attached to a button the user can use to 
 * unset any changes and revert the settings back to the original values.
 */

function revert(select_field) {
	// Set to original settings.
	var i;
	for (i=0; i < select_field.options.length; i++) {
		select_field.options[i].selected = select_field.options[i].defaultSelected;
	}
	select_field.focus();
}

/* -------------------------------------------------------------------------- */

function formOf(e) {
	// Return the object of the form enclosing the given element.

	while (e && e.nodeName != "FORM") {
		e = e.parentNode;
	}
	return e;
}

/* -------------------------------------------------------------------------- */

/*
 * Text array functions
 *
 * Text arrays are text strings that begin and end with a delimiter character
 * that is also used to separate the fields of the text string (;1;2;3;).
 */

function split_array(text_array) {
	// Return an array of a text array's elements.

	if (text_array && text_array.match(/^(\W)(.*)\1$/)) { 
		return RegExp.$2.split(RegExp.$1);
	}
	return [ text_array ];	// return original as one element array
}

function is_array_filled(text_array) {
	// Return true if every element in text_array is non-empty

	var ta = split_array(text_array);
	if (ta.length == 0) return false;
	for (var i = 0; i < ta.length; i++) {
		if (ta[i].length == 0) return false;
	}
	return true;
}

function array_count(text_array) {
	// Return number of elements in the text array.

	return split_array(text_array).length;
}

function array_total(text_array) {
	// Return total of the elements in the text array.

	var val, total = 0;
	var ta = split_array(text_array);
	for (i = 0; i < ta.length; i++) {
		if (!isNaN(val = parseFloat(ta[i]))) total += val;
	}
	return total;
}

function array_total_remainder(total, text_array) {
	// Return total minus sum of the elements in the text array or 0
	// if the result is negative. The total argument can be a number,
	// a string, or an object with a 'value' component (like <input>
	// fields).

	if (typeof total == 'number') {
		return Math.max(0,total - array_total(text_array));
	}
	else if (typeof total == 'string') {
		var mytotal = parseFloat(total);
		if (!isNaN(mytotal)) {
			return Math.max(0,mytotal - array_total(text_array));
		}
		else {
			return 0;
		}
	}
	else {
		return 0;
	}
}

/* -------------------------------------------------------------------------- */

/*
 * The assign_options() function changes the options of a <select> field to be
 * those listed in the arguments. It saves the original settings in an object
 * so that descriptions of the options can be re-used for the new settings.
 * The new settings should be a sub-set of the original settings.
 */

function assign_options(select_field) {
	if (! (select_field
	    && select_field.type
	    && select_field.type.match(/^select/)) ) {
		return;	 		// not a select field
	}

	// Build options from arguments, which can be text arrays.
	var i, new_options = new Array;
	for (i = 1; i < arguments.length; i++) {
		if (typeof arguments[i] == 'string') {
			new_options = new_options.concat(split_array(arguments[i]));
		}
		else {
			new_options = new_options.concat(arguments[i]);
		}
	}

	// Save current options
	save_options(select_field);

	var saved_option = select_field.value;

	// Delete current options
	// Note that deleting an option automatically shifts other options up
	// so that we do not have to increment i to delete all options.
	for (var i = 0; select_field.options[i] != null; ) {
		// Keep empty (undefined) options in list
		if (select_field.options[i].value == '') { i++; continue; }

		// Delete option
		select_field.options[i] = null;
	}

	// Set new options
	var txt, opt;
	select_field.selectedIndex=0;
	for (var i = 0; i < new_options.length; i++) {
		opt = new_options[i];
		if (saved_options[select_field.name][opt]) {
			// Use display text from saved options.
			txt = saved_options[select_field.name][opt];
		}
		else {
			// Use option value as display text.
			txt = opt.replace(/\\s+/g, " ");
		}

		select_field.options[select_field.options.length] = new Option(txt,opt);
		if (opt == saved_option) select_field.selectedIndex = select_field.options.length - 1;
	}
}

var saved_options = new Object;

function save_options(select_field) {
	var field_name = select_field.name;

	// If saved object exists, the options have already been saved.
	if (saved_options[field_name]) return;

	var sv = saved_options[field_name] = new Object;

	for (var i = 0; i < select_field.options.length; i++) {
		sv[select_field.options[i].value] = select_field.options[i].text;
	}
}

/* -------------------------------------------------------------------------- */

/*
 * The assign_input() function changes the value of the specified field to be
 * input_field argument value, which is intended to be an input array.
 * The controlling JavaScript for the input array is executed to setup the 
 * input array to input only those fields listed in the text_array argument.
 * The new settings should be a sub-set of the original settings.
 */

function assign_input(input_field, text_array) {
	if (! (input_field && input_field.name) ) {
		return;	 		// not a input field (a dumb guess)
	}

	// Construct name of function to use to assign the text_array
	// from the name of the input_field. Each input array has their
	// own controlling functions.
	var func = "assign_"+input_field.name;
	if (eval('typeof '+func) == 'function') {
		func += '(text_array,input_field.value)';// function arguments
		eval(func);			// show/hide input fields
	}
	return;
}

/* -------------------------------------------------------------------------- */

/*
 * The assign_multiple() function selects all the options of a
 * <select ... multiple> tag to those listed in a text array.
 */

function assign_multiple(select_field, text_array) {
	if(! select_field.multiple) {
		// Input_field does not take multiple options.
		if (select_field.value) select_field.value = text_array;
		return;
	}

	var ta = split_array(text_array);

	// Build object for quick lookup of names and values.
	var i, lookup = new Object;
	for (i = 0; i < ta.length; i++) { lookup[ta[i]] = true; }

	// Scan options selecting or unselecting them.
	for (i = 0; i < select_field.options.length; i++) {
		if (lookup[select_field.options[i].value]) {
			select_field.options[i].selected = true;
		}
		else {
			select_field.options[i].selected = false;
		}
	}
}

/* -------------------------------------------------------------------------- */

/*
 * Aggregate functions 
 *
 * The following aggregate functions produce the sum, average, or count of their
 * arguments. The arguments can be values, form fields, or strings of form
 * field names. All the elements in the form are searched for fields with
 * the same name as the argument's form field or a name matching the string
 * arguments' name.
 *
 * These routines are intended to work with form detail records that are listed
 * as display field names. As detail records are added to the master form, their
 * sum, average, or count can be assigned to master form fields.
 */

function SUM() {

	var i, arg, frm, fld;
	var amt,sum = 0;
	for (var i = 0; i < arguments.length; i++) {
		arg = arguments[i];
		if (typeof arg == 'string' && arg.match(/[^\de.+-]/)) {
			// Match fields specified by argument text.
			if (! frm) { frm = self.document.forms[Default_Form] }
			sum += _sum_count(arg,frm)[0];
		}
		else if (typeof arg == 'object') {
			if (arg.form && (frm = arg.form)) {
				// Match fields with same name as argument.
				sum += _sum_count(arg.name,frm)[0];
			}
			else if (arg.length && arg.length > 0) {
				// Argument is an array of fields.
				for (var j = 1; j < arg.length; j++) {
					if (arg[j]) {
						amt = parseFloat(arg[j].value);
						if (!isNaN(amt)) sum += amt;
					}
				}
			}
		}
		else {
			// Add argument value.
			amt = parseFloat(arg);
			if (!isNaN(amt)) sum += amt;
		}
	}

	if (! isNaN(sum)) { return sum }
	else		  { return 0   }
}

function TOTAL() {

	var i, arg, frm, fld;
	var amt, sum = 0;
	for (var i = 0; i < arguments.length; i++) {
		arg = arguments[i];
		if (typeof arg == 'string' && arg.match(/[^\de.+-]/)) {
			// Match fields specified by argument text.
			if (! frm) { frm = self.document.forms[Default_Form] }
			sum += _sum_count(arg,frm)[0];
		}
		else if (typeof arg == 'object') {
			if (arg.form && (frm = arg.form)) {
				// Match fields with same name as argument.
				sum += _sum_count(arg.name,frm)[0];
			}
			else if (arg.length && arg.length > 0) {
				// Argument is an array of fields.
				for (var j = 1; j < arg.length; j++) {
					if (arg[j]) {
						amt = parseFloat(arg[j].value);
						if (!isNaN(amt)) sum += amt;
					}
				}
			}
		}
		else {
			// Add argument value.
			amt = parseFloat(arg);
			if (!isNaN(amt)) sum += amt;
		}
	}

	if (isNaN(sum)) { sum = 0 }

	return roundValue(sum,2);
}

function AVG() {

	var i, arg, frm, fld, cs;
	var count = 0;
	var amt, sum = 0;
	for (var i = 0; i < arguments.length; i++) {
		arg = arguments[i];
		if (typeof arg == 'string' && arg.match(/[^\de.+-]/)) {
			// Match fields specified by argument text.
			if (! frm) { frm = self.document.forms[Default_Form] }
			cs = _sum_count(arg,frm);
			sum += cs[0];
			count += cs[1];
		}
		else if (typeof arg == 'object') {
			if (arg.form && (frm = arg.form)) {
				// Match fields with same name as argument.
				cs = _sum_count(arg.name,frm);
				sum += cs[0];
				count += cs[1];
			}
			else if (arg.length && arg.length > 0) {
				// Argument is an array of fields.
				for (var j = 1; j < arg.length; j++) {
					if (arg[j]) {
						amt = parseFloat(arg[j].value);
						if (!isNaN(amt)) sum += amt;
					}
				}
				count += arg.length;
			}
		}
		else {
			// Add argument value.
			amt = parseFloat(arg);
			if (!isNaN(amt)) sum += amt;
			count++;
		}
	}

	if (isNaN(sum))   { sum   = 0 }
	if (isNaN(count)) { count = 0 }

	if (count != 0) { return (sum / count) }
	else            { return 0 }
}

function COUNT() {

	var i, arg, frm, fld;
	var count = 0;
	for (var i = 0; i < arguments.length; i++) {
		arg = arguments[i];
		if (typeof arg == 'string' && arg.match(/[^\de.+-]/)) {
			// Match fields specified by argument text.
			if (! frm) { frm = self.document.forms[Default_Form] }
			count += _sum_count(arg,frm)[1];
		}
		else if (typeof arg == 'object') {
			if (arg.form && (frm = arg.form)) {
				// Match fields with same name as argument.
				count += _sum_count(arg.name,frm)[1];
			}
			else if (arg.length && arg.length > 0) {
				// Argument is an array of fields.
				count += arg.length;
			}
		}
		else {
			// Add argument value.
			count++;
		}
	}

	if (! isNaN(count)) { return count }
	else		    { return 0     }
}

function _sum_count(fld_name,frm) {
	// Helper function for TOTAL(), SUM(), AVG(), and COUNT().
	// Return an array with first element the sum, and second is the count.

	var amt, sum = 0;
	var count = 0;;
	var el = frm.elements;
	for (var i = 0; i < el.length; i++) {
		if (el[i].name == fld_name) {
			amt = parseFloat(el[i].value);
			if (!isNaN(amt)) sum += amt;
			count++;
		}
	}
	return [ sum, count ];
}

/* -------------------------------------------------------------------------- */

/*
 * Radio button utilities
 *
 * The radio_value() function is a convenience function for finding the 
 * current value of a collection of radio buttons.
 * The radio_set() function is used to set the current value of a collection
 * of radio buttons.
 */

function radio_value(radio_field) {
	if (typeof radio_field != 'object') return '';
	if (typeof radio_field.length == 'undefined') return radio_field.value;
	for (var i = 0; i < radio_field.length; i++) {
		if (radio_field[i].checked) {
			return radio_field[i].value;
		}
	}
	return '';
}

function radio_set(radio_field,val) {
	if (typeof radio_field != 'object') return;
	if (typeof radio_field.length == 'undefined') return;
	for (var i = 0; i < radio_field.length; i++) {
		if (radio_field[i].value == val) {
			radio_field[i].checked = true;
		} else {
			radio_field[i].checked = false;
		}
	}
}

/* -------------------------------------------------------------------------- */

/*
 * Checkbox utilities
 *
 * The checkbox_value() function is a convenience function for finding the 
 * current value of a checkbox field.
 * The checkbox_set() function is another convenience function to check or
 * uncheck a checkbox field.
 */

function checkbox_value(checkbox_field) {
	if (typeof checkbox_field != 'object') return;
	if (checkbox_field.checked) {
		return checkbox_field.value;
	}
	return '';
}

function checkbox_set(checkbox_field,val) {
	if (typeof checkbox_field != 'object') return;
	if (checkbox_field.value == val) {
		checkbox_field.checked = true;
	} else {
		checkbox_field.checked = false;
	}
}

/* -------------------------------------------------------------------------- */

/*
 * Note: The following date routines are included here so that checkDate()
 * can accept relative dates. It would be nice if I could just 'include' the
 * library, but .... The daydiff() function is not used above, it is just
 * included here because I don't know where to put it yet. (mp 8/3/00)
 *
 * The today() and dbdate() functions were added to return today's date or
 * any date in a format for the database. As of 6/24/01, for PostgreSQL,
 * the format is m-d-yyyy. The separator can be set in the global variable DS.
 */

/*
 * Date utility routines
 *
 * Enclosed functions:
 *	 isLeapYear(y)
 *	 julian(m,d,y)
 *	 julianDate (j,y)
 *	 dayadd(days, startdate)
 *	 daydiff(startdate, enddate)
 *
 *	today()
 *	dbdate(date_string)
 */

var daymon = new Array(31,28,31,30,31,30,31,31,30,31,30,31);

function isLeapYear(y) {
	return ( ((y % 400) == 0) || ( ((y % 4) == 0) && (y % 100) != 0) );
}

function julian(m,d,y) {
	// return julian day of date
	daymon[1] = isLeapYear(y) ? 29 : 28;
	var jd = d;
	var i;

	for (i = 0; i < m; i++) 
		jd += daymon[i];

	return jd;
}

function dayadd(days, startdate) {
	// return a string representing the number of days added to startdate
	// If startdate is not given, the current date is used.
	var m, d, y;
	var jd;

	if (days == null || isNaN(days)) {
		return "";	// return empty string (a valid sql date)
	}

	// compute number of days (nights)
	var dd = startdate ? new Date(startdate) : new Date();
	m = dd.getMonth();
	d = dd.getDate();
	y = dd.getFullYear();
	jd = julian(m,d,y);

	jd += days;

	// date in previous year
	while (jd <= 0) {
		y -= 1;
		jd += (isLeapYear(y) ? 366 : 365);
	}

	// date in next year
	while (jd > (d = isLeapYear(y) ? 366 : 365)) {
		jd -= d;
		y += 1;
	}

	return julianDate(jd,y);
}

function daydiff(startdate, enddate) {
	// return the number of nights between two dates
	if (startdate == null || enddate == null
	|| startdate == "" || enddate == "") {
		return 0;
	}

	// Assume dates are valid dates

	var m1, d1, y1;
	var m2, d2, y2;
	var jd1, jd2;

	// compute number of days (nights)
	var dd1 = new Date(startdate);
	m1 = dd1.getMonth();
	d1 = dd1.getDate();
	y1 = dd1.getFullYear();
	jd1 = julian(m1,d1,y1);

	var dd2 = new Date(enddate);
	m2 = dd2.getMonth();
	d2 = dd2.getDate();
	y2 = dd2.getFullYear();
	jd2 = julian(m2,d2,y2);

	// adjust for years y2 > y1
	for (i=y1; i < y2; i++) {
		jd2 += 365;
		if (isLeapYear(i)) jd2++;
	}
	// adjust for years y2 < y1
	for (i=y2; i < y1; i++) {
		jd1 += 365;
		if (isLeapYear(i)) jd1++;
	}

	return (jd2 - jd1);
}

function julianDate (j,y) {
	// return date string represnting the julian date of the given year

	var m, d;

	if (y == null) {
		var today = new Date();
		y = today.getFullYear();
	}
	
	for (m = 0; j > 0; j -= daymon[m++]) {
		d = j;
	}

	// Note: The following date should be represented using local date
	// formatting conventions. It also needs to be in a format that is
	// correctly parsed by the database engine.

	if (m < 10) m = '0' + m;
	if (d < 10) d = '0' + d;

	return m + DS + d + DS + y;
}

function dbdate(ds) {
	// return date d as a string acceptable to the database
	var d = new Date(ds);

	var mm   = d.getMonth() + 1;
	var dd   = d.getDate();
	var yyyy = d.getFullYear();

	if (mm < 10) mm = '0' + mm;
	if (dd < 10) dd = '0' + dd;

	return mm + DS + dd + DS + yyyy;

}

function today() {
	// return today's date as a string acceptable to the database
	return dbdate(new Date);
}

/* -------------------------------------------------------------------------- */

/*
 * Round a value to a specified number of places, default 2
 */

function roundValue(val, precision) {
	// round input value and display with precision decimal points

	// This routine is suitable for monitary values as it always formats
	// the number with the exact number of decimal places specified in 
	// the precision argument.

	if (precision == null || isNaN(precision)) precision = 2;
	if (val == null || isNaN(val) || val == Infinity || val == -Infinity) {
		if (precision > 0) return "0." + zeros(precision)
		else return "0";
	}

	var p = Math.pow(10,precision);
	var v = parseFloat(val);	// remove non-digit characters
	v = Math.round(v * p) / p;	// round number

	var inputVal = ("" + v);	// convert back to string for processing
	var i = inputVal.indexOf(".");
	if (i == -1) {
		 if (precision == 0) return (inputVal);
		 else return (inputVal + "." + zeros(precision));
	}

	var decPart = inputVal.substring(0, i++);

	if (precision == 0) return decPart;

	// append 0s after decimal point for string processing
	inputVal += zeros(precision+1);

	var fracPart = inputVal.substring(i, i + precision);

	return ("" + decPart + "." + fracPart.substring(0,precision));
}

function zeros(n) {
	// return string of n zeros
	var s = "";
	for (i=1; i <= n; i++) { s += "0" }
	return s;
}
