/*
 * AcmeMenu v2.4
 *
 * Copyright Mackley F. Pexton, 2003.  All rights reserved.
 *
 * This is free software for individual, educational, and non-profit use.
 * Instructions and source code are available at www.acmebase.org/menu.
 * Optimized version of software is for sale at order.acmebase.com.
 * Commercial web sites are required to have an inexpensive license.
 * This software is part of the AcmeBase project at www.acmebase.org.
 * Send correspondence and feedback to: mack@acmebase.org.
 */

/******************************************************************************

AcmeMenu v2.4 -- Display pop-up menu and sub-menus.

The functions below display pop-up menus using CSS positioning and DOM.
They currently work on MS Internet Explorer 5+/PC, IE 5+/Mac, Netscape 6+,
and Opera 5+.

There are four primary functions:

	menu('menu_id','menu_header_id');	// display a menu below a header
	submenu('submenu_id');			// display submenu to the right

	make_menu('menu_id','menu_header_id');	// setup and activate menus
	hide_menus();				// hide all menus

A fifth function reset_menus() is used to reset the menu configuration
variables to their initial default values. It is useful when there are
several types of menus on a page, each with their own set of configuration
settings.

The make_menu() function returns a Menu object which has three methods and
two properties of use:

	m.show_menu()			// show a menu
	m.hide_menu()			// hide a menu
	m.hide_submenu()		// hide a menu and its submenus

	m.is_visible			// true if menu is shown
	m.is_submenu			// true if menu is a submenu

The menu object can also be used to set localized values of configuration
variables.

In order to place menu panels next to their headers on the page, a few utility
functions are used which might be of use elsewhere:

	getEventElementLeft(evt,e);	// get left coordinate of element e
	getEventElementTop(evt,e);	// get top coordinate of element e

	getElementLeft(e);		// only works with version 6 browsers
	getElementTop(e);		// only works with version 6 browsers

	getElementWidth(e);		// width of element including borders
	getElementHeight(e);		// height of element including borders

	getElementStyle(e,dom_property,css_attribute);	// get current style setting
	getBorderWidth(e,'right');	// top, bottom, left, right border width
	getPadding(e,'left');		// top, bottom, left, right padding
	getMargin(e,'top');		// top, bottom, left, right margin


A menu as defined below is a pop-up display of links or any other kind of form
input fields. Menu selections are contained by a <div> tag (or any block element)
that has an id attribute set to the name of the menu. All menus and submenus need
id attributes.

Example menu:

	<!-- Menu Header -->
	<a href="javascript:menu('mymenu')" id="mymenu_head" class="menu-head">Menu</a>

	...

	<!-- Main Menu -->
	<div id="mymenu" class="menu">
	<a href="http://www.somewhere.com">Go Somewhere</a>
	<a href="javascript:submenu('mysubmenu')" class="submenu-head">Submenu</a>
	<a href="javascript:submenu('myparam')" class="submenu-head">Parameters</a>
	<hr>
	<a href="javascript:self.print()">Print</a>
	<a href="javascript:self.close()">Exit</a>
	</div>

	<!-- Submenu Menu -->
	<div id="mysubmenu" class="submenu">
	<a href="http://www.nowhere.com">Go Nowhere</a>
	<a href="http://www.somewhere.com">Go Somewhere</a>
	...
	</div>

	<!-- Parameter Entry -->
	<div id="myparam" class="param">
	<form action="save.php">
	... form input tags ...
	<input type="submit" value="Save">
	</form>
	</div>

To set up a menu so that it automatically pops up the menus and their
cascading submenus as the user's mouse hovers over them, call the
make_menu() function like:

	<!-- Setup Up Menu -->
	<script language="JavaScript">make_menu('mymenu','mymenu_head');</script>

By calling make_menu(), the user is not required to click on the menu
buttons to activate them and the location of the submenus are automatically
positioned to be to the right of the menu button that activated them.

If the make_menu() function is not used to set up the menus to respond
to the mouse, then the menu locations cannot be automatically determined.
Therefore, the menu's x,y (left,top) location needs to be defined with
CSS style settings.

An <a> tag with the href attribute calling the submenu() function pops up a 
submenu. A submenu is just the same as a main menu in that it has a <div>
tag enclosing <a> tags, and it has an id attribute naming the submenu.

Its the function name "submenu()" in the href attribute of the <a> tag that
is sensed to "link" together the menus and their submenus.

The hide_menus() function hides all the displayed menus. It is useful when
performing an action that does not change the window display.

By default, clicking in a menu freezes it open until clicking outside its
borders.

The display of a menu and its <a> tag menu selections is controlled by 
CSS styles.

To allow the selected path to remain highlighted as the mouse hovers over
a submenu, the menu and submenu header elements have the string "-hilite"
appended to their class name (if they have one) when their submenu is
displayed. Those styles need to be defined in a stylesheet and they are
usually similar to the appearance defined for the A:hover style.

Example stylesheet:

	DIV.menu    { position:absolute; display:none; width:150px; }
	DIV.submenu { position:absolute; display:none; width:150px; }

	DIV.menu A,
	DIV.submenu A { display:block; width:100%; }

	DIV.menu A:hover,
	DIV.submenu A:hover,
	A.menu-head-hilite,
	A.submenu-head-hilite { background-color:#DDDDDD; }

	A.submenu-head        { background-image:url(/images/rtriangle.gif);
	                        background-position:right center;
	                        background-repeat:no-repeat; }

Menus also work well as as pop-up data entry forms because when the user
clicks inside a menu, its display is "frozen" on the screen until they
click away outside the menus. That behavior can be turned off by setting
the class variable Menu.AllowFreeze to "false".

Submenus can only appear in one location in a menu chain. They cannot be reused
with other menus.

This software tries to preserve any events (onMouseOver, onMouseOut, onClick)
defined by the web page author. Rollovers should work as always.

There is a debug function at the end that can be turned on or off by setting
the variable debug.on to true or false. It produces a flood of output
in a second window. Pausing the movement of the mouse over the menus causes
blank lines to be inserted into the debug window to separate tests.

The debug functions can also be turned on after a page is displayed by typing
the following into the browser address bar:

	javascript:debug.turn_on()



History:
	v2.0 - Rewrite AcmeBase menues, change event model, IE5 and 6.
	v2.1 - Get dimensions from CSS stylesheets, Netscape.
	v2.2 - Added Windows style menus, click start, click stop, Opera.
	v2.3 - Added <table> menus, image links for menu items, IE Macintosh.
	v2.4 - Triggers and fades, auto hide and fixed menus.

******************************************************************************/

// Uncomment following to turn on debug.
//debug.on = true;

// Uncomment following to turn on mouse coordinates.
//mouse_coord.on = true;

/*
 * Detect Browser Version
 */

bv.isNS     = (navigator.userAgent.indexOf("Netscape") >= 0) ? true : false;

bv.isOpr    = (navigator.userAgent.indexOf("Opera")    >= 0) ? true : false;
bv.isOpr5   = (navigator.userAgent.indexOf("Opera 5")  >= 0) ? true : false;
bv.isOpr6   = (navigator.userAgent.indexOf("Opera/6")  >= 0) ? true : false;
bv.isOpr7   = (navigator.userAgent.indexOf("Opera/7")  >= 0) ? true : false;

bv.isIE     = (document.all && ! bv.isOpr);
bv.isIE6    = (bv.isIE && document.compatMode && document.compatMode.indexOf("CSS1") >= 0) ? true : false;
bv.isIE5Mac = (bv.isIE && navigator.userAgent.indexOf('Mac') >= 0) ? true : false;
bv.isIE5    = (bv.isIE && ! bv.isIE5Mac && ! bv.isIE6) ? true : false;	// default version

bv.isMoz    = (! bv.isNS && ! bv.isOpr && ! bv.isIE && navigator.userAgent.indexOf("Mozilla") == 0) ? true : false;

function bv() {
	// Return all the browser version flags that match this browser.
	var s = "";
	for (var v in bv) if (v.substring(0,2) == "is" && bv[v]) s += (s ? " " : "") + v;
	return s;
}

// Uncomment the following if you have a non-menu page for old unrecognized browsers.
//if (! bv()) location.href = "http://www.your-web-site-address.com/nomenus.html";

/*
 * Main Functions
 */

function make_menu(id, header_id, keep_open) {

	// Make a menu out of an HTML object and its header object by
	// setting up event handlers to respond to mouse events.

	// The id of the menu is required.
	// The header_id is usually required except for static menus.
	// The seldom used keep_open flag keeps menus open after mouse clicks.

	debug("make_menu("+id+")");

	var m;

	if (Menu.menus[id]) {

		// This menu has been previously made, possibly by the menu()
		// function if the user clicked the menu header before the
		// menus were loaded. Delete the menu chain and remake it.

		debug("make_menu("+id+"): deleting previously made menu '"+id+"'.");

		for (var menu_id in Menu.menus) {
			m = Menu.menus[menu_id];
			if (m.top.id == id) {
				m.hide_menu();
				delete Menu.menus[menu_id];
			}
		}
	}

	var manual    = false;	// we want event handlers to auto open menus
	keep_open = (keep_open != null && keep_open) ? true : Menu.KeepOpen;

	m = new Menu(id, header_id, manual, keep_open);

	m.configure_menu();

	var header_e = m.header_e;

	if (header_e) {

		// Save current mouseover and mouseout event handlers.

		// Note: Original event definitions are normally saved in the
		// menu object, however, because of Netscape's thinking about
		// when to fire mouseout events, the onmouseover, onmouseout,
		// and onclick definitions need to be saved in the element's
		// object. See set_enclosed_elements_handler() function.

		if (header_e.onmouseover && header_e.onmouseover != Menu.onmouseover_handler)
			header_e.orig_onmouseover = header_e.onmouseover;
		if (header_e.onmouseout && header_e.onmouseout != Menu.onmouseout_handler)
			header_e.orig_onmouseout = header_e.onmouseout;

		// Assign event handlers to header element.
	
		header_e.onmouseover = Menu.onmouseover_handler;
		header_e.onmouseout  = Menu.onmouseout_handler;

		if (bv.isNS) Menu.set_enclosed_elements_handler(header_e);
	}
	else {
		// Setup static menus.

		// Static menus are those menus without a header element.
		// They must always be visible. (If there isn't a header element,
		// how can they be shown?) Static menus never do get "shown"
		// in show_menu(), so they can't get setup there. Finish
		// setting up the menu here.

		m.setup_menu();
	}

	// Setup handlers to hide all menus if mouse clicked outside of them.
	if (document.body.onclick && document.body.onclick != Menu.onclick_body_handler)
		Menu.orig_onclick_body_handler = document.body.onclick;
	document.body.onclick = Menu.onclick_body_handler;

	// Setup handlers to recompute menu positions if window resized.
	if (window.onresize && window.onresize != Menu.onresize_handler)
		Menu.orig_onresize_handler = window.onresize;
	window.onresize = Menu.onresize_handler;
	
	return m;	// return menu object
}

function menu(id, header_id, keep_open) {

	// Display a menu.
	// The header id is optional.
	// The optional keep_open flag prevents the menu from being hidden
	// when the user clicks away. It works with only manual menus here.

	debug("menu("+id+")");

	// Define Menu object if one is not already defined.
	var m = Menu.menus[id];
	if (!m) {

		// Check if menus have been loaded into the browser.
		if (! getElementRef(id)) {
			debug("menu(): menu '"+id+"' not found: page not loaded yet?");
			return;
		}

		var manual = true;
		keep_open = keep_open ? true : false;	// change to boolean

		m = new Menu(id, header_id, manual, keep_open);

		m.configure_menu();

		// Scan the menu node for any contained elements with a submenu()
		// function call in the href attribute. This is necessary in order
		// to find the header elements for the submenus.

		// Note: This is only performed on "manual" menus that have
		// not been setup with the make_menu() function. Menus setup
		// by make_menu() have mouseover and mouseout event handlers
		// defined that reveal the header elements of submenus so
		// that they can be built dynamically instead of calling
		// build_submenus() to set them all up at once.
		
		Menu.build_submenus(m, getElementRef(id));
	}

	if (! m.header_e) {
		debug('Menu: ERROR: cannot find the menu header element for the menu "'+id+'"');
		// Don't hide menu if there is not a header to re-display it.
		return;
	}


	// Note: the keep_open flag operates slightly different for manual
	// menus than with the auto menus. When set for auto menus, the menus
	// are never hidden, even if the menu header is clicked. Manual menus
	// toggle the display of the menu when the menu header is clicked. 
	if (m.keep_open && ! m.manual) return;

	// Toggle display/non-display of menu.
	if (m.is_visible) { 

		// Toggle ClickStop mode. This freezes the menu open, which
		// if there is a positioning error, allows the user to go
		// find the menu. This does not apply to manual menus.

		if (m.manual || m.ClickStop) {
			if (m.click_stop_auto_set) m.ClickStop = false;
			m.hide_submenu();
		}
		else {
			m.ClickStop = true;
			m.click_stop_auto_set = true;

			// Note: The body onclick handler has been triggered
			// by this time and has scheduled all menus -- including
			// this one -- to be hidden, so redisplay it again.
			m.show_menu();

			// Track the currently displayed top menu.
			Menu.click_stop_menu = m;
		}
	}
	else {
		m.show_menu();
	}
}

function submenu(id) {

	// Display a submenu.

	debug("submenu("+id+")");

	var m = Menu.menus[id];

	if (m) {

		// Ignore unless using manual menus.
		if (! m.manual) return;

		// Toggle display/non-display of submenu.

		if (m.is_visible) { m.hide_submenu(); }
		else {
			if (m.parent.child) m.parent.hide_submenu();
			m.parent.show_submenu(id);
		}

		return;
	}

	debug("submenu(): ERROR: The menu for submenu '"+id+
	  "' has not been setup with make_menu().");
}

function hide_menus(force_relocation) {

	// Hide all displayed menus.

	debug("hide_menus(): force_relocation="+
	  (force_relocation != null ? force_relocation : 'null'));

	var m;
	for (var id in Menu.menus) {
		m = Menu.menus[id];
		if (! m.keep_open) m.hide_menu();
		if (force_relocation) m.is_located = false;
	}

	Menu.click_start_enabled = false;	// reset
}

function reset_menus() {
	
	// Initialize all class constants.

/*
 * Menu Settings
 */

// Class Constants (change as desired)

// The following offset constants are used to position the menu or submenu
// relative to their header elements. These are the initial default settings
// that can be changed for each menu. New submenus receive their offsets from
// their parent menu. The measurments are for dropdown menus and submenus to
// the right, so positive x positions are to the right and positive y positions
// go down. For menus going up or to the left, the offset numbers are
// automatically reversed, thereby maintaining the "same offset" whether the
// menu goes right or left, up or down. This allows the direction of the 
// menu to be changed simply by changing the MenuPostion and SubmenuPosition
// constants below without having to change the offset constants.

// Menu panels are positioned from the lower left corner of the header.
Menu.MenuOffsetX = 0;		// Offset from left side of header object
Menu.MenuOffsetY = 0;		// Offset from bottom side of header object

// Submenu panels are positioned from the upper right corner of their header.
Menu.SubmenuOffsetX = 0;	// Offset from right side of header object
Menu.SubmenuOffsetY = 0;	// Offset from top of header object

// Menus and submenus can project out in different directions from their header.
// By default, menus drop "Down", and submenus pop "Right". The position codes
// can be set to one of the following constants:
//	U (up), D (down), L (left), or R (right).
Menu.MenuPosition    = "D";	// Down
Menu.SubmenuPosition = "R";	// Right

// Menus are usually repositioned if necessary to keep them within view.
// Set AllowReposition variable to "false" to turn off the feature.
Menu.AllowReposition = true;

// Menus can be classified as vertical menus (the default) or horizontal menus.
// The Y coordinate of vertical menus is adjusted slightly so that the first
// menu item is straight across from the header element. Horizontal menus have
// their placement adjusted slightly so the menu selections tend to line up
// vertically. The MenuOrientation and SubmenuOrientation constants can be set
// to one of the following constants:
//	V (vertical), H (horizontal), or N (neither).
Menu.MenuOrientation    = "V";
Menu.SubmenuOrientation = "V";

// Border dimensions and padding around menus and their headers need to be
// known for accurate placement of the menus. The dimensions are read from the
// element's style attribute or they are read from CSS stylesheet settings.
// The units of measure must be pixel units "px" in order to be correctly
// interpreted. If styles settings cannot be read by the browser (e.g. Opera),
// the following constants are used.

// Uncomment and define the following as needed.
//Menu.MenuHeaderBorderWidth = null;
//Menu.MenuHeaderPadding = null;
//Menu.MenuHeaderPaddingHorz = null;
//Menu.MenuHeaderPaddingVert = null;

//Menu.MenuBorderWidth = null;
//Menu.MenuPadding = null;
//Menu.MenuPaddingHorz = null;
//Menu.MenuPaddingVert = null;

//Menu.SubmenuHeaderBorderWidth = null;
//Menu.SubmenuHeaderPadding = null;
//Menu.SubmenuHeaderPaddingHorz = null;
//Menu.SubmenuHeaderPaddingVert = null;

//Menu.SubmenuBorderWidth = null;
//Menu.SubmenuPadding = null;
//Menu.SubmenuPaddingHorz = null;
//Menu.SubmenuPaddingVert = null;

// Opera 6 needs to know the top margin of the document body in order
// to correctly determine the Y coordinate of a menu header, but it cannot
// read the style settings. This is only used for Opera 6.
Menu.BodyMarginTop = 0;	

// Menus can be set to be dormant until the menu header is clicked.
// This activates a behavior similar to Windows system menus.
Menu.ClickStart = false;

// Menus can be set to stay open until a mouse click.
// If ClickStart is true, ClickStop is also forced to be true.
Menu.ClickStop = Menu.ClickStart || false;

// Menus can be set to stay open, even if a mouse click.
// If KeepOpen is true, ClickStop is also forced to be true.
Menu.KeepOpen = false;

// Menus can be set to stay open a set number of seconds after
// the mouse wanders outside them. It is like the ClickStop
// flag being set with an click event automatically scheduled
// to occur after the specified number of seconds.
Menu.AutoHideTime = 0;

// Menus can fade in and fade out if you set FadeDuration to the number
// of milliseconds over which to fade. To set only the fade in duration,
// set ShowFadeDuration, and to control just the fade out, set HideFadeDuration.
Menu.FadeDuration = 0;
Menu.ShowFadeDuration = 0;
Menu.HideFadeDuration = 0;

// The speed of a fade is determined by FadeSpeed, which is the number
// of milliseconds between each iteration of the fade. ShowFadeSpeed
// controls only the fade in speed and HideFadeSpeed controls the fade out.
Menu.FadeSpeed = 0;
Menu.ShowFadeSpeed = 0;
Menu.HideFadeSpeed = 0;

// Each iteration of fading reduces or increases the opacity by the
// percentage amount assigned to FadePercent. This value is used to
// compute FadeSpeed if it is not set to a non-zero number.
Menu.FadePercent = 10;
Menu.ShowFadePercent = 0;
Menu.HideFadePercent = 0;

// Triggers are functions that are executed when a menu is shown
// or hidden. They are executed asynchronously and they are passed
// the menu object as their sole argument.

// Any statements following the menu() or submenu() function calls
// in the menu item's href attribute are executed as a trigger.
// They are executed when showing a menu and when hiding it, just
// like they would be executed if the user clicked on the link.

Menu.Trigger = null;
Menu.ShowTrigger = null;
Menu.HideTrigger = null;

// Menus can be "frozen" open if the mouse is clicked inside them.
// Set AllowFreeze variable to "false" to turn off that feature.
Menu.AllowFreeze = true;

// Menu locations are cached once they are computed. However, if the inline
// position of the menus changes, the cached coordinates become invalid.
// Set CacheLocation to false to force the menu's coordinates to be
// computed each time they are shown. Alternately, you could call 
// hide_menus(true) with its argument set to true to force the relocation
// of the menus the next time they are shown.
Menu.CacheLocation = true;

// Menu locations can be fixed in a predetermined location by setting the CSS
// styles left and top, and by setting the MenuFixed configuration constant
// to "true". The same is true for submenus, their location is fixed on the
// page if SubmenuFixed is "true".
Menu.MenuFixed = false;
Menu.SubmenuFixed = false;

// The menu z-index is used to ensure the menu is displayed on top of the
// document. The submenu z-indexes are automatically incremented.
Menu.ZIndex = 1000;

// The highlight suffix is appended to a menu header's class name -- if it
// has one. This allows the selected path to stay highlighted as the mouse
// moves over the submenu.
Menu.HighlightSuffix = "-hilite";

// Timing delays are used before actually showing or hiding a menu.
// Increase these numbers if menus items have large padding. The mouse needs
// to go from a submenu header across the padding onto the submenu within
// the TimeToHide time to ensure smooth operation.
Menu.TimeToShow = 150;		// Delay before showing menu (milliseconds)
Menu.TimeToHide = 100;		// Delay before hiding menu (milliseconds)

}

reset_menus();	// initialize class constants

// Class Variables (do not change)
Menu.menus = {};		// All the defined menus and their Menu objects
Menu.click_start_enabled = false;// on/off flag
Menu.click_stop_menu = null;	// Current menu displayed in click stop mode.
Menu.keep_open_menu = null;	// Current menu displayed in keep open mode.
Menu.frozen_menu = null;	// Current menu that is frozen open.
Menu.auto_hide_timer = null;	// Timer for auto hiding menus.
Menu.max_z = 0;			// Maximum zIndex of top menu displayed.

// Regular expressions for matching menu() and submenu() functions in <a> tags.
Menu.menu_re    = /menu\(\s*(['"]?)([^'",)]+)/;
Menu.submenu_re = /submenu\(\s*(['"]?)([^'",)]+)/;

/*
 * Menu Object Definition
 */

function Menu(id, header, manual, keep_open) {

	// Menu object constructor.

	// Required arguments:
	//   id is set to the id attribute of the menu <div> tag.
	// Optional arguments:
	//   header can be a string or an object.
	//   manual is set to true for manual menus.
	//   keep_open is set to true for manual (non)menu in auto environment.
	
	debug("new Menu("+id+")");

	if (id) this.e = getElementRef(id);
	if (header) this.header_e = getElementRef(header);

	if (! this.e) {
		debug('Menu: ERROR: cannot find the menu "'+id+'"');
	}
	if (header && ! this.header_e) {
		debug('Menu: ERROR: cannot find the menu header "'+header+'"');
	}

	this.id			= id;

	this.is_submenu		= false;
	this.is_setup		= false;
	this.is_located		= false;
	this.is_finalized	= false;
	this.is_fixed		= false;
	this.is_static		= this.header_e ? false : true;
	this.is_visible		= this.is_static ? true : false;

	this.parent		= null;
	this.child		= null;
	this.top		= this;

	this.x			= null;
	this.y			= null;
	this.z			= Menu.ZIndex;

	this.evt		= null;

	this.frozen_e		= null;
	this.frozen_onmouseout	= null;
	this.saved_body_onclick	= null;

	this.show_timer		= 0;
	this.hide_timer		= 0;

	this.opacity		= 0;
	this.show_fade_timer	= 0;
	this.hide_fade_timer	= 0;

	this.manual		= manual;
	this.keep_open		= keep_open;

	// Register all defined menus.
	Menu.menus[id] = this;
}

/*
 * Event Handlers
 */

Menu.onmouseover_handler = function(evt) {
	evt = (evt) ? evt : ((window.event) ? window.event : null);
	var e = evt.target ? evt.target : evt.srcElement ? evt.srcElement : null;

	// Event handler attached to menu <div> tags.

	debug("Menu.onmouseover_handler(): e.id="+e.id, e);

	clearTimeout(Menu.auto_hide_timer);	// reset

	// Find menu object name that belongs to this event.
	var menu_id, submenu_id, submenu_target;	
	var m;
	for (var target = e; target != null; target = target.parentNode) {

		// Trigger original event if one was saved.
		// This applies to the menu header. In Netscape,
		// it also applies to all the elements processed by
		// set_enclosed_elements_handler().
		if (target.orig_onmouseover) target.orig_onmouseover(evt);

		if (target.id && (m = Menu.menus[target.id])) {

			// Found the menu container (<div>) element.

			if (Menu.frozen_menu
			&& Menu.frozen_menu.e != target) Menu.frozen_menu.thaw();

			m.save_event(evt);

			// Hide existing submenu if visible.
			if (m.child && (! submenu_id || m.child.id != submenu_id)) {
				m.child.hide_submenu();
			}

			// If this menu is an static menu (e.g. a vertical menu,
			// without a header), then we need to check and hide any
			// menus remaining open because of being in ClickStop mode.
			// Otherwise, we only need to check below when the mouse
			// is over the menu header element.
			if (m.is_static) m.hide_opened_menus();

			if (submenu_id) { m.show_submenu(submenu_id,submenu_target); }
			else            { m.show_menu(); }

			break;
		}

		if (target.href) {
			if (target.href.match(Menu.submenu_re)) {

				// Found submenu() function in <a> href attribute.

				// Submenu_id is quoted text or named variable.
				submenu_id = RegExp.$1 ? RegExp.$2 : eval(RegExp.$2);
				submenu_target = target;

				debug("Menu.onmouseover: found " +
				  "submenu(" + submenu_id + ") in target.");

				// Continue on to find menu container.
				continue;
			}

			if (target.href.match(Menu.menu_re)) {

				// Found menu() function in <a> href attribute.

				// Menu_id is quoted text or named variable.
				menu_id = RegExp.$1 ? RegExp.$2 : eval(RegExp.$2);

				debug("Menu.onmouseover: found " +
				  "menu(" + menu_id + ") in target.");

				if (Menu.frozen_menu) Menu.frozen_menu.thaw();

				if ((m = Menu.menus[menu_id])) {

					if (! m.is_located) m.save_event(evt);

					// Do not show menu if using ClickStart
					// and the user has not "clicked" yet.
					if (m.ClickStart && ! Menu.click_start_enabled) {
						debug("Menu.onmouseover:"+
						  " click start mode is not enabled."+
						  " Skip display of menu.");
						break;
					}

					// Hide any menus that are still open
					// because of ClickStop or keep_open
					// flags. 
					m.hide_opened_menus();

					m.show_menu();
				}

				break;
			}
		}
	}
}

Menu.onmouseout_handler = function(evt) {
	evt = (evt) ? evt : ((window.event) ? window.event : null);
	var e = evt.target ? evt.target : evt.srcElement ? evt.srcElement : null;

	// Event handler attached to menu <div> tags.

	// Note: Instead of descending into all levels below an element in 
	// set_enclosed_elements_handler() to ensure that all enclosed objects
	// that cause Netscape to trigger a mouseout event are covered with a
	// mouseover event handler, we simply filter some of the elements
	// causing spurious mouseout events here. (!!)

	if (bv.isNS) {
		if (e.nodeName == "TEXTAREA") return;
		if (e.nodeName == "OPTION") return;
	}

	debug("Menu.onmouseout_handler(): e.id="+e.id, e);

	// Find menu object name that belongs to this event.
	var m, top;
	var menu_id, submenu_id;
	for (var target = e; target != null; target = target.parentNode) {

		// Trigger original event if one was saved.
		// (This applies only to the menu header.)
		if (target.orig_onmouseout) target.orig_onmouseout(evt);

		if (target.id && (m = Menu.menus[target.id])) {

			// Found the menu container (<div>) element.

			// Leave menus displayed if keep_open specified.
			if (m.keep_open) break;

			// Leave menus displayed if ClickStop specified.
			top = m.top;
			if (top.ClickStop) {
				if (top.click_stop_auto_set) top.ClickStop = false;
				if (top.AutoHideTime) {
					debug("Menu.onmouseout_handler():"+
					  " scheduling auto hide: "+top.AutoHideTime);

					Menu.auto_hide_timer = setTimeout(
					  "hide_menus()", top.AutoHideTime);
				}
				break;
			}

			// Hide entire menu chain from bottom up.
			m.hide_submenu();
			while ((m = m.parent)) {
				m.hide_menu();
			}

			break;
		}

		if (target.href) {

			if (target.href.match(Menu.submenu_re)) {

				// Found submenu() function in <a> href attribute.

				// Submenu_id is quoted text or named variable.
				submenu_id = RegExp.$1 ? RegExp.$2 : eval(RegExp.$2);

				debug("Menu.onmouseout: found " +
				  "submenu(" + submenu_id + ") in target.");

				// Continue on to find enclosing menu container.
				continue;
			}

			if (target.href.match(Menu.menu_re)) {

				// Found menu() function in <a> href attribute.

				// Menu_id is quoted text or named variable.
				menu_id = RegExp.$1 ? RegExp.$2 : eval(RegExp.$2);

				debug("Menu.onmouseout: found " +
				  "menu(" + menu_id + ") in target.");

				if ((m = Menu.menus[menu_id])) {

					// Leave menus displayed if ClickStart,
					// ClickStop or keep_open is specified.
					if (m.ClickStop || m.keep_open) {
						if (m.AutoHideTime) {
							debug("Menu.onmouseout_handler():"+
							  " scheduling auto hide: "+
							  m.AutoHideTime);

							Menu.auto_hide_timer = setTimeout(
							  "hide_menus()", m.AutoHideTime);
						}

						debug("Menu.onmouseout_handler():"+
						  " Do not hide menu."+
						  " ClickStop="+m.ClickStop+
						  " keep_open="+m.keep_open);

						break;
					}

					m.hide_submenu();
				}

				break;
			}
		}
	}
}

Menu.onclick_handler = function(evt) {
	evt = (evt) ? evt : ((window.event) ? window.event : null);
	var e = evt.target ? evt.target : evt.srcElement ? evt.srcElement : null;

	// Event handler attached to menu <div> tags.

	debug("Menu.onclick_handler: e.id="+e.id, e);

	// Find menu object name that belongs to this event.
	var m;
	for (var target = e; target != null; target = target.parentNode) {

		if (target.href
		&& target.nodeName == "A"
		&& ! target.href.match(Menu.submenu_re)) {
			// A menu item was selected. Hide the menus.
			hide_menus();
			break;
		}

		if (! target.onclick) continue;	// find node with this event handler

		if (target.orig_onclick) target.orig_onclick(evt);

		if (target.id && (m = Menu.menus[target.id])) {

			// Found the menu container (<div>) element.
			// Keep menu displayed unless user clicks away.

			m.freeze();
			break;
		}
	}

	evt.cancelBubble = true;
	if (evt.stopPropagation) evt.stopPropagation();

	return true;
}

Menu.onclick_handler_ignore = function(evt) {
	evt = (evt) ? evt : ((window.event) ? window.event : null);
	var e = evt.target ? evt.target : evt.srcElement ? evt.srcElement : null;

	// Mouse click event handler attached to manual menu <div> tag.

	// This handler only gets attached to manual menus and only when
	// an onclick event handler is attached to the document body. It
	// captures and stops the click event bubbling. It allows "manual"
	// menus for entering form fields to exist on a page that also
	// uses "auto" menus -- menus that are to disappear when clicking
	// on the document body.

	debug("Menu.onclick_handler_ignore: e.id="+e.id, e);

	if (e.orig_onclick) e.orig_onclick(evt);

	evt.cancelBubble = true;
	if (evt.stopPropagation) evt.stopPropagation();
	return true;
}

Menu.onclick_body_handler = function(evt) {

	// Mouse click event handler attached to the <body> tag.

	debug("Menu.onclick_body_handler()");

	hide_menus();
	if (Menu.orig_onclick_body_handler) return Menu.orig_onclick_body_handler(evt);
}

Menu.onresize_handler = function(evt) {

	// Event handler called when the browser window is resized.
	// Resizing the window causes all the document inline element's
	// coordinates to change (e.g. the menu header elements) which
	// causes all the cached menu coordinates to be invalid.

	debug("Menu.onresize_handler()");

	hide_menus('relocate');
	if (Menu.orig_onresize_handler) return Menu.orig_onresize_handler(evt);
}

/*
 * Freezing methods
 */

// Note: instead of having a bunch of flags saving state when a menu is frozen
// or not, we simply remove the element's onMouseOut event handler temporarily
// and put it back when the user clicks on the document body.

// Class Method
Menu.onclick_body_handler_thaw = function(evt) {

	// Mouse click event handler attached to the <body> tag.
	// It interfaces with the object methods.

	debug("Menu.onclick_body_handler_thaw(): menu="+
	  (Menu.frozen_menu ? Menu.frozen_menu.id : "NULL"));

	if (Menu.frozen_menu) Menu.frozen_menu.thaw();
}

Menu.prototype.freeze = function() {

	debug("Menu.freeze("+this.id+")");

	if (Menu.frozen_menu) {
		if (Menu.frozen_menu == this) {
			// Ignore repeated clicks over frozen element.
			return;	
		}
		else {
			Menu.frozen_menu.thaw();
		}
	}

	this.frozen_e = this.e;
	this.frozen_onmouseout   = this.frozen_e.onmouseout;
	this.frozen_e.onmouseout = null;

	this.saved_body_onclick = document.body.onclick;
	document.body.onclick = Menu.onclick_body_handler_thaw;

	Menu.frozen_menu = this;

	// Mozilla fires a mouseout event just before processing a click event.
	if (bv.isMoz) this.show_menu();	// ensure menu is displayed
}

Menu.prototype.thaw = function() {

	debug("Menu.thaw("+this.frozen_e.id+")");

	if (this.frozen_e) {
		// Put back onmouseout and the body onclick event handlers.
		this.frozen_e.onmouseout = this.frozen_onmouseout;
		document.body.onclick    = this.saved_body_onclick;
	}

	this.frozen_e = null;		// clear for use as a flag

	Menu.frozen_menu = null;

	// Hide menu chain from bottom up.
	var m = this;
	do {
		debug("Menu.thaw(): hiding menu "+m.id);

		m.hide_menu();
	} while ((m = m.parent));
}

/*
 * Menu builder
 */

// Elements to ignore when trying to find submenus.
Menu.filter_object = {
	"#text":	true,
	"SELECT":	true,
	"TEXTAREA":	true};

Menu.build_submenus = function(m,e) {
	
	// Recursive function to traverse the nodes contained in the menu
	// element looking for submenus and building menu objects for each.

	debug("Menu.build_submenus: checking e.nodeName="+e.nodeName+", href="+e.href);

	if (e.href && e.href.match(Menu.submenu_re)) {
			
		// Found submenu() function in <a> href attribute.

		var id = RegExp.$1 ? RegExp.$2 : eval(RegExp.$2); // string or var

		var submenu = m.make_submenu(id,e);

		debug("Menu.build_submenus: found submenu("+id+") for "+m.id);

		// Look in submenu for more submenus.
		Menu.build_submenus(submenu, getElementRef(id));

		debug("Menu.build_submenus: pop return");

		return;
	}

	if (e.childNodes) {

		for (var n = 0; n < e.childNodes.length; n++) {

			if (! Menu.filter_object[e.childNodes[n].nodeName]) {

				debug("Menu.build_submenus: recursing to "+
				  e.childNodes[n].nodeName+", n="+n);

				Menu.build_submenus(m, e.childNodes[n]);
			}
		}
	}

	debug("Menu.build_submenus: pop");
}

Menu.set_enclosed_elements_handler = function(e) {
	
	// Set onmouseover handler to all the child elements of e
	// because NS6+ fires the mouseout event when the mouse
	// hovers over an ENCLOSED element. (!!)

	// Note: this routine is not recursive, it only looks at the
	// first level of child nodes directly under the element. See
	// onmouseout_handler() for the filtering of spurious events from
	// hovering over text within a TEXTAREA element or for hovering
	// over the scroll bar of a SELECT element.

	var child_e;

	if (e.childNodes) {

		for (var n = 0; n < e.childNodes.length; n++) {

			child_e = e.childNodes[n];

			if (child_e.nodeType != 1) continue;

			// Save original mouseover event.
			if (child_e.onmouseover
			&& child_e.onmouseover != Menu.onmouseover_handler)
				child_e.orig_onmouseover = child_e.onmouseover;

			child_e.onmouseover = Menu.onmouseover_handler;

			debug("Menu.set_enclosed_elements_handler: child_e=",child_e);
		}
	}
}

Menu.prototype.make_submenu = function(menu_id, header) {

	// Attach a new menu as a submenu to this menu.

	var submenu = new Menu(menu_id, header, this.manual, this.keep_open);

	// Link parent objects of this menu.
	submenu.parent = this;
	submenu.top    = this.top;

	// Set flag.
	submenu.is_submenu = true;

	// Set Z index so submenu is on top of parent.
	submenu.z = this.z + 10;
	
	return submenu;
}

Menu.prototype.configure_menu = function() {

	// Assign global class settings to object instance settings.
	// Configuration settings are only stored in the top menu object.

	// Note: Instead of having a bunch of assignment statements,
	// scan the components of the Menu class object and copy them
	// to this object. To filter out functions and class working
	// variables, we have cleverly named all the configuration
	// constants with capitalized names. All the other private
	// variables and function names in Menu begin with lowercase
	// characters.

	var v,c;
	for (v in Menu) {
		c = v.substr(0,1);
		if (c == c.toUpperCase()) {

			// Uncomment next if you want to check which constants
			// are actually being saved. Some browsers (Netscape 6)
			// add their own variables to a class object.
			//debug("Menu.configure_menu(): this."+v+" = "+Menu[v]);

			this[v] = Menu[v];
		}
	}
}


Menu.prototype.setup_menu = function() {

	// Attach event handlers to menu element.

	if (! this.e) return;

	if (this.manual) {

		// Manual menus do not use event handlers, so header elements are
		// not known, so autoplacement of submenus is not possible.

		this.is_located   = true;
		this.is_finalized = true;

		// Manual menus do not get "located", so their Z indexes don't
		// get set. As a convenience, set the Z index here.

		this.e.style.zIndex = 100;

		this.AllowReposition = false;	// force off

		// Attach a click event handler to the menu to ignore clicks
		// if there is already a click event handler on the document
		// body. The document body click handler is used by auto menus
		// to hide all shown menus. This allows a manual menu used for
		// entering form fields to coexist on the same page as auto
		// menus.

		if (document.body.onclick && ! this.e.onclick) {
			this.e.onclick = Menu.onclick_handler_ignore;
		}
	}
	else {
		if (! this.is_submenu) this.setup_top_menu();

		// Get any extra statements to execute when header is "clicked".
		// Set flag to execute triggers if found.
		if (this.header_e && this.header_e.href.match(/;\s*(.*)/)) {
			this.trigger_str = RegExp.$1;
		}

		if (this.trigger_str || this.top.has_show_trigger) this.has_show_trigger = true;
		if (this.trigger_str || this.top.has_hide_trigger) this.has_hide_trigger = true;

		// Simplify flags determining if this menu is a fixed menu
		// (a menu whose coordinates are predetermined).
		this.is_fixed = this.is_submenu
			      ? (this.top.SubmenuFixed || false)
			      : (this.top.MenuFixed || false);
		
		debug("Menu.setup_menu: menu is_fixed: "+this.is_fixed,this.e);

		// Setup auto menu event handlers.

		// Save current onmouseover and onmouseout event handlers.
		if (this.e.onmouseover && this.e.onmouseover != Menu.onmouseover_handler)
			this.e.orig_onmouseover = this.e.onmouseover;
		if (this.e.onmouseout && this.e.onmouseout != Menu.onmouseout_handler)
			this.e.orig_onmouseout  = this.e.onmouseout;
		if (this.e.onclick && this.e.onclick != Menu.onclick_handler)
			this.e.orig_onclick     = this.e.onclick;

		// Define event handlers.
		this.e.onmouseover = Menu.onmouseover_handler;
		this.e.onmouseout  = Menu.onmouseout_handler;
		if (this.top.AllowFreeze) this.e.onclick = Menu.onclick_handler;

		if (bv.isNS) Menu.set_enclosed_elements_handler(this.e);
	}

	this.is_setup = true;
}

Menu.prototype.setup_top_menu = function() {

	// Finish setting up configuration variables.
	// Adjust settings that are dependent on others.

	if (this.ClickStart) this.ClickStop = true;	// force on

	if (this.AutoHideTime) {
		// Convert time to milliseconds
		this.AutoHideTime = parseFloat(this.AutoHideTime) * 1000.0;
		this.ClickStop = true;			// force on
	}

	// Set flag if there are trigger functions to execute.
	if (this.ShowTrigger || this.Trigger) this.has_show_trigger = true;
	if (this.HideTrigger || this.Trigger) this.has_hide_trigger = true;

	// Setup fade parameters.
	if (bv.isIE5Mac || bv.isOpr) {
		// Fades don't work on these browsers.
		this.show_fade_duration = this.hide_fade_duration = 0;
		return;
	}

	this.show_fade_duration = this.ShowFadeDuration
				? this.ShowFadeDuration
				: this.FadeDuration
				? this.FadeDuration
				: 0;

	this.hide_fade_duration = this.HideFadeDuration
				? this.HideFadeDuration
				: this.FadeDuration
				? this.FadeDuration
				: 0;

	if (! this.show_fade_duration && ! this.hide_fade_duration) return;

	this.show_fade_speed	= this.ShowFadeSpeed
				? this.ShowFadeSpeed
				: this.FadeSpeed
				? this.FadeSpeed
				: this.ShowFadePercent
				? Math.ceil(this.show_fade_duration * this.ShowFadePercent/100)
				: this.FadePercent
				? Math.ceil(this.show_fade_duration * this.FadePercent/100)
				: Math.ceil(this.show_fade_duration * 10/100);	// default

	this.hide_fade_speed	= this.HideFadeSpeed
				? this.HideFadeSpeed
				: this.FadeSpeed
				? this.FadeSpeed
				: this.HideFadePercent
				? Math.ceil(this.hide_fade_duration * this.HideFadePercent/100)
				: this.FadePercent
				? Math.ceil(this.hide_fade_duration * this.FadePercent/100)
				: Math.ceil(this.hide_fade_duration * 10/100);	// default
	
	this.show_fade_percent	= this.show_fade_speed / this.show_fade_duration * 100;
	this.hide_fade_percent	= this.hide_fade_speed / this.hide_fade_duration * 100;
}

Menu.prototype.locate_menu = function() {

	// Locate menu on page.

	debug("Menu.locate_menu: this.evt="+this.evt);

	if (this.evt) {
		if (this.is_submenu)	{ this.compute_position_submenu(); }
		else			{ this.compute_position_menu();    }

		var fudge = bv.isIE5 ? -2 : 0;

		// Assign computed position to the menu element.
		this.e.style.left = (this.x + fudge) + "px";
		this.e.style.top  = (this.y + fudge) + "px";
		if (bv.isOpr5) this.e.style.pixelTop  = (this.y + fudge);

		debug("Menu.locate_menu: left="+this.e.style.left+", top="+this.e.style.top);
	}

	this.e.style.zIndex  = this.z;

	if (this.top.CacheLocation
	&& (! this.parent || this.parent.is_located))
		this.is_located = true;			// set
	this.is_finalized = false;			// reset
}

Menu.prototype.locate_static_menu = function(submenu) {

	// A menu that does not have a header is an static menu. It is always
	// visible, It does not pop-up. Locating the menu on the page
	// needs to be delayed until the mouse clicks on one of its menu items.

	// The menu's Y coordinate is not needed as submenus are always placed
	// relative to the Y coordinate of their header element.

	this.x = getEventElementLeft(submenu.evt, this.e, 
				this.get_border_width("left"),
				this.get_padding("left"));

	this.is_located = true;
}

Menu.prototype.finalize_menu = function() {

	// Compute final coordinates of menu.

	// Sometimes a menu has to be displayed before its width or
	// height can be determined so that it can be properly positioned.
	// This method computes those final placement coordinates.

	if (! this.e) return;

	this.is_finalized = true;

	if (this.manual) return;

	if (this.is_submenu) {
		// Submenus are initially positioned to the right (R).
		if (this.top.SubmenuPosition == "R") return;

		this.finalize_position_submenu();
	}
	else {
		// Menus are initially positioned down (D).
		if (this.top.MenuPosition == "D") return;

		this.finalize_position_menu();   
	}

	var fudge = bv.isIE5 ? -2 : 0;

	// Assign recomputed position to the menu element.
	this.e.style.left = (this.x + fudge) + "px";
	this.e.style.top  = (this.y + fudge) + "px";
	if (bv.isOpr5) this.e.style.pixelTop  = this.y;

	debug("Menu.finalize_menu: left="+this.e.style.left+", top="+this.e.style.top);
}

Menu.prototype.save_event = function(evt) {

	// Save event properties needed later.

	// Mouse event properties are used to dynamically position menus and
	// submenus. Because of timing, we wait until actually showing the
	// menus to compute the location in the document, at which time
	// the event has gone out of scope. This method saves only the
	// properties that are needed later in a pseudo-event object.

	this.evt = {
		"target"  : (evt.target ? evt.target : evt.srcElement ? evt.srcElement : null),

		"offsetX" : (evt.offsetX ? evt.offsetX : 0),
		"offsetY" : (evt.offsetY ? evt.offsetY : 0),
		"clientX" : (evt.clientX ? evt.clientX : 0),
		"clientY" : (evt.clientY ? evt.clientY : 0) };
}

/*
 * Showing methods
 */

Menu.prototype.show_menu = function() {

	debug("Menu.show_menu("+this.id+"): is_visible="+this.is_visible);

	clearTimeout(this.hide_timer);
	clearTimeout(this.show_timer);

	if (this.is_submenu) this.show_parentmenu();

	if (this.is_static) this.track_shown_open_menus();

	if (this.is_visible) return;

	// Schedule display of menu
	this.show_timer = setTimeout("Menu.now_show_menu('"+this.id+"')", this.top.TimeToShow);

	debug("Menu.show_menu("+this.id+"): schedule show: "+this.top.TimeToShow);
}

// Class Method
Menu.now_show_menu = function(menu_id) {

	// This function is called by setTimeout(). It is executed in the
	// window context and it interfaces with the object methods.

	debug("Menu.now_show_menu("+menu_id+")");

	var m = Menu.menus[menu_id];
	if (m) m.do_show_menu();
}

Menu.prototype.do_show_menu = function() {

	debug("Menu.do_show_menu("+this.id+"):"+
	  " is_visible="+this.is_visible+
	  ", is_static="+this.is_static);

	if (! this.e) {
		debug('Menu: ERROR: cannot find the menu "'+this.id+'"');
		return;
	}

	if (this.is_visible) return;

	// Last check for submenus rendered after parent hidden. (only in ClickStart mode)
	if (this.top.ClickStop && this.parent && ! this.parent.is_visible) return;

	if (! this.is_setup) this.setup_menu();

	if (! this.is_fixed
	&& (! this.is_located || (this.parent && ! this.parent.is_located))) this.locate_menu();

	// Show menu.
	if (this.top.show_fade_duration) {
		this.show_menu_fade();
	}
	else {
		this.e.style.display = "block";
		this.e.style.visibility = "visible";

		if (this.top.hide_fade_duration) {
			this.opacity = 100;
			this.show_menu_fade();
		}
	}

	if (! this.is_fixed) {

		if (! this.is_finalized) this.finalize_menu();

		if (this.top.AllowReposition) this.reposition_menu();
	}

	// Show select path by changing the menu header's class name.
	if (this.header_e.className) {
		this.header_e.className += this.top.HighlightSuffix;
	}

	this.is_visible = true;

	if (! this.is_submenu && ! this.manual) this.track_shown_open_menus();

	// Execute trigger functions asynchronously.
	if (this.has_show_trigger) setTimeout("Menu.now_show_trigger('"+this.id+"')", 1);
}

/*
 * Hiding methods
 */

Menu.prototype.hide_menu = function() {

	debug("Menu.hide_menu("+this.id+"): is_visible="+this.is_visible);

	clearTimeout(this.hide_timer);
	clearTimeout(this.show_timer);

	if (this.is_static) this.track_hidden_open_menus();

	if (! this.is_visible) return;

	// Schedule the menu to be hidden.
	this.hide_timer = setTimeout("Menu.now_hide_menu('"+this.id+"')", this.top.TimeToHide);

	debug("Menu.hide_menu("+this.id+"): schedule hide: "+this.top.TimeToHide);
}

// Class Method
Menu.now_hide_menu = function(menu_id) {

	// This function is called by setTimeout(). It is executed in the
	// window context and it interfaces with the object methods.

	debug("Menu.now_hide_menu("+menu_id+")");

	var m = Menu.menus[menu_id];
	if (m) m.do_hide_menu();
}

Menu.prototype.do_hide_menu = function() {

	debug("Menu.do_hide_menu("+this.id+"):"+
	  " is_visible="+this.is_visible+
	  ", is_static="+this.is_static);

	if (! this.is_visible || this.is_static) return;

	// Hide menu.
	if (this.top.hide_fade_duration) {
		this.hide_menu_fade();
	}
	else {
		this.e.style.visibility = "hidden";
		this.e.style.display = "none";

		if (this.top.show_fade_duration) {
			// Ensure menu is invisible, reset show fade settings.
			this.opacity = 0;
			this.hide_menu_fade();
		}
	}

	// Return menu header back to original class
	if (this.header_e.className) {
		var re = new RegExp(this.top.HighlightSuffix);
		this.header_e.className = this.header_e.className.replace(re,"");
	}

	this.is_visible = false;

	if (! this.is_submenu && ! this.manual) this.track_hidden_open_menus();

	// Execute trigger functions asynchronously.
	if (this.has_hide_trigger) setTimeout("Menu.now_hide_trigger('"+this.id+"')", 1);
}

/*
 * Fading methods
 */

Menu.prototype.show_menu_fade = function() {

	debug("Menu.show_menu_fade("+this.id+"): opacity="+this.opacity);

	clearTimeout(this.show_fade_timer);
	clearTimeout(this.hide_fade_timer);

	// Fading menus keep incrementing zIndex to keep new menus on top
	// to capture the mouse events first. This is especially useful
	// for fixed menus that overlap each other.
	if (this.z > Menu.max_z) { Menu.max_z = this.z; }
	else			 { this.e.style.zIndex = ++Menu.max_z; }
		
	this.e.style.display = "block";
	this.e.style.visibility = "visible";

	Menu.now_show_menu_fade(this.id);
}

// Class Method
Menu.now_show_menu_fade = function(menu_id) {

	// This function is called by setTimeout(). 

	debug("Menu.now_show_menu_fade("+menu_id+")");

	var m = Menu.menus[menu_id];
	if (m) {
		m.opacity += m.top.show_fade_percent;
		if (m.opacity > 100) m.opacity = 100;

		m.do_menu_fade();

		if (m.opacity < 100) {
			m.show_fade_timer = setTimeout(
			  "Menu.now_show_menu_fade('"+menu_id+"')",
			  m.top.show_fade_speed);
		}
	}
}

Menu.prototype.hide_menu_fade = function() {

	debug("Menu.hide_menu_fade("+this.id+"): opacity="+this.opacity);

	clearTimeout(this.show_fade_timer);
	clearTimeout(this.hide_fade_timer);

	Menu.now_hide_menu_fade(this.id);
}

// Class Method
Menu.now_hide_menu_fade = function(menu_id) {

	// This function is called by setTimeout().

	debug("Menu.now_hide_menu_fade("+menu_id+")");

	var m = Menu.menus[menu_id];
	if (m) {
		m.opacity -= m.top.hide_fade_percent;
		if (m.opacity < 0) m.opacity = 0;

		m.do_menu_fade();

		if (m.opacity > 0) {
			m.hide_fade_timer = setTimeout(
			  "Menu.now_hide_menu_fade('"+menu_id+"')",
			  m.top.hide_fade_speed);
		}
		else {
			// Faded out, hide the menu.
			m.e.style.visibility = "hidden";
			m.e.style.display = "none";
		}
	}
}

Menu.prototype.do_menu_fade = function() {

	// Set the alpha filter to the current menu opacity.
	// Used for both showing an hiding menus.

	if (bv.isIE5 || bv.isIE6) {
		this.e.style.filter =
		  "progid:DXImageTransform.Microsoft.Alpha(opacity="+this.opacity+")";
	}
	else if (bv.isNS || bv.isMoz) {
		this.e.style.MozOpacity = this.opacity + '%';
	}
}

/*
 * Sub-menu methods
 */

Menu.prototype.show_submenu = function(menu_id,header) {

	// Show a submenu.

	debug("Menu.show_submenu("+menu_id+")");

	var submenu = Menu.menus[menu_id];

	// Dynamically build a submenu if needed.
	if (! submenu) submenu = this.make_submenu(menu_id,header);

	// Link child object of menu that is displayed.
	this.child = submenu;

	// Pass along event object for positioning.
	submenu.evt = this.evt;

	submenu.show_menu();
}

Menu.prototype.hide_submenu = function() {

	// Recursive function to traverse the objects contained in the menu.

	debug("Menu.hide_submenu("+this.id+")");

	// Hide all descendant menus from bottom up.
	if (this.child) this.child.hide_submenu();

	this.hide_menu();
}

/*
 * Parent-menu methods
 */

Menu.prototype.show_parentmenu = function() {

	// Ensure parent menus are displayed.

	debug("Menu.show_parentmenu("+this.id+")");

	var m, child = this;

	while ((m = child.parent)) {

		debug("Menu.show_parentmenu("+this.id+"): showing parent "+m.id);

		m.show_menu();

		// If the parent menu already has a child menu displayed,
		// hide it first and reconnect this menu to be its child
		// menu.

		if (m.child && m.child != child) m.child.hide_submenu();

		m.child = child;	// connect links

		child = m;
	}
}

/*
 * Open menu methods
 */

Menu.prototype.hide_opened_menus = function() {

	// Hide any menus that have remained opened if they
	// do not match this menu's top menu.

	// Hide currently displayed menu if in ClickStop mode.
	if (Menu.click_stop_menu && Menu.click_stop_menu != this.top)
		Menu.click_stop_menu.hide_submenu();

	// Hide currently displayed menu if keep_open.
	if (this.keep_open
	&& Menu.keep_open_menu && Menu.keep_open_menu != this.top)
		Menu.keep_open_menu.hide_submenu();
}

Menu.prototype.track_shown_open_menus = function() {

	// Track the ClickStop and keep_open menus and their state flags.
	// This gets called when showing top level menus.

	debug("Menu.track_shown_open_menus:");

	if (this.ClickStop) {

		// Track the currently displayed top menu.

		// Because of timing delays showing menus, a top menu
		// might have just been displayed and assigned to the
		// click_stop_menu. Check one last time.
		if (Menu.click_stop_menu && Menu.click_stop_menu != this) {
			Menu.click_stop_menu.do_hide_menu();
		}

		Menu.click_stop_menu = this;
		Menu.click_start_enabled = true;

		debug("Menu.track_shown_open_menus: setting click_start_enabled",this.e);
	}

	if (this.keep_open) {

		// Track the currently displayed top menu.

		// As above, check one last time for keep_open_menu.
		if (Menu.keep_open_menu && Menu.keep_open_menu != this) {
			Menu.keep_open_menu.do_hide_menu();
		}

		Menu.keep_open_menu = this;
		Menu.click_start_enabled = false;	// reset

		debug("Menu.track_shown_open_menus: setting keep_open_menu",this.e);
	}
}

Menu.prototype.track_hidden_open_menus = function() {

	// Track the ClickStop and keep_open menus and their state flags.
	// This gets called when hiding top level menus.

	debug("Menu.track_hidden_open_menus:");

	if (Menu.click_stop_menu && Menu.click_stop_menu == this) {

		debug("Menu.track_hidden_open_menus:"+
		  " clear click_stop_menu:",Menu.click_stop_menu.e);

		if (this.click_stop_auto_set) this.ClickStop = false;

		Menu.click_stop_menu = null;
	}

	if (Menu.keep_open_menu && Menu.keep_open_menu == this) {

		debug("Menu.track_hidden_open_menus:"+
		  " clear keep_open_menu:",Menu.keep_open_menu.e);

		Menu.keep_open_menu = null;
	}
}

/*
 * Trigger methods
 */

// Class Method
Menu.now_show_trigger = function(menu_id) {

	// This function is called by setTimeout(). It executes 
	// trigger functions asynchronously when a menu is shown.

	debug("Menu.now_show_trigger("+menu_id+")");

	var m = Menu.menus[menu_id];
	if (!m) return;
	
	if (m.top.ShowTrigger) m.top.ShowTrigger(m);
	if (m.top.Trigger) m.top.Trigger(m);
	if (m.trigger_str) eval(m.trigger_str);
}

// Class Method
Menu.now_hide_trigger = function(menu_id) {

	// This function is called by setTimeout(). It executes 
	// trigger functions asynchronously when a menu is hidden.

	debug("Menu.now_hide_trigger("+menu_id+")");

	var m = Menu.menus[menu_id];
	if (!m) return;

	if (m.top.HideTrigger) m.top.HideTrigger(m);
	if (m.top.Trigger) m.top.Trigger(m);
	if (m.trigger_str) eval(m.trigger_str);
}


/*
 * Positioning methods
 */

Menu.prototype.compute_position_menu = function() {

	// Position menu relative to the lower left corner of the header
	// for a vertically oriented menu (V) positioned down (D).

	this.x	= getEventElementLeft(this.evt, this.header_e,
			this.get_header_border_width("left"),
			this.get_header_padding("left")) +
		  this.top.MenuOffsetX;

	this.y	= this.get_header_top() +
		  this.get_header_height() +
		  this.top.MenuOffsetY;

	debug("Menu.compute_position_menu: x="+this.x+", y="+this.y);
}

Menu.prototype.compute_position_submenu = function() {

	// Position submenu relative to the upper right corner of the header.
	// for a vertically oriented submenu (V) positioned to the right (R).
	// Align submenu so first option is horizonal from header element.

	var parent = this.parent;

	if (parent.is_static && ! parent.is_located) parent.locate_static_menu(this);

	// If submenu orientation is vertical (the default), place menu outside
	// this menu's borders, otherwise place the menu next to the header's
	// upper right corner.

	this.x	= (this.top.SubmenuOrientation == "V" && this.top.MenuOrientation == "V"
		? parent.x +
		  getElementWidth(parent.e, parent.getBorderWidth())
		: getEventElementLeft(this.evt, this.header_e,
			this.get_header_border_width("left"),
			this.get_header_padding("left")) +
		  getElementWidth(this.header_e, this.getHeaderBorderWidth())
		  ) +
		  this.top.SubmenuOffsetX;

	this.y	= (this.top.SubmenuOrientation != "N"
		? this.compute_vert_menu_y()
		: this.get_header_top()
		  ) +
		  this.top.SubmenuOffsetY;

	debug("Menu.compute_position_submenu: x="+this.x+", y="+this.y);
}

Menu.prototype.finalize_position_menu = function() {

        // Readjust coordinates for menus poping up or to the left.
	// The menu's height or width needs to be known first before adjusting
        // the coordinates, so the menu is rendered first and then queried to
	// find its size.

        // At this point, the default MenuPosition is "D" and the x,y coordinate
	// is currently the lower left corner of the header.

	debug("Menu.finalize_position_menu: MenuPosition="+this.top.MenuPosition);

	switch (this.top.MenuPosition) {
	case "U":                               // Up
		this.y -= getElementHeight(this.e) +
			  this.get_header_height() +
			  this.top.MenuOffsetY * 2;
		break;

	case "L":                               // Left
		this.x -= getElementWidth(this.e, this.getBorderWidth()) +
			  this.top.MenuOffsetX * 2;

		this.y	= this.get_header_top() +
			  this.top.MenuOffsetY;
		break;

	case "R":                               // Right
		this.x += getElementWidth(this.header_e, this.getHeaderBorderWidth());
		this.y	= this.get_header_top() +
			  this.top.MenuOffsetY;
		break;

	case "D":                               // Down
	default:
		// do nothing
	}
}

Menu.prototype.finalize_position_submenu = function() {

	// Readjust coordinates for submenus poping up or to the left.
	// This does the same function for submenus as
	// finalize_position_menu() does for menus above.

	// At this point, the default MenuPosition is "R" and the x,y coordinate
	// is currently the upper right corner of the header.

	debug("Menu.finalize_position_submenu: SubmenuPosition="+this.top.SubmenuPosition);

	switch (this.top.SubmenuPosition) {
	case "L":				// Left
		this.x	= (this.top.SubmenuOrientation == "V" && this.top.MenuOrientation == "V"
			? this.parent.x
			: getEventElementLeft(this.evt, this.header_e,
				this.get_header_border_width("left"),
				this.get_header_padding("left"))
			  ) -
			  getElementWidth(this.e, this.getBorderWidth()) -
			  this.top.SubmenuOffsetX;
		break;

	case "D":				// Down
		this.x	= (this.top.SubmenuOrientation != "N"
			? this.compute_horz_menu_x()
			: getEventElementLeft(this.evt, this.header_e,
				this.get_header_border_width("left"),
				this.get_header_padding("left"))
			  ) +
			  this.top.SubmenuOffsetX;

		this.y	= (this.top.SubmenuOrientation == "H" && this.top.MenuOrientation == "H"
			? this.parent.y +
			  getElementHeight(this.parent.e, this.parent.getBorderHeight())
			: this.get_header_top() +
			  this.get_header_height()
			  ) +
			  this.top.SubmenuOffsetY;
		break;

	case "U":				// Up
		this.x	= (this.top.SubmenuOrientation != "N"
			? this.compute_horz_menu_x()
			: getEventElementLeft(this.evt, this.header_e,
				this.get_header_border_width("left"),
				this.get_header_padding("left"))
			  ) +
			  this.top.SubmenuOffsetX;

		this.y	= (this.top.SubmenuOrientation == "H" && this.top.MenuOrientation == "H"
			? this.parent.y
			: this.get_header_top()
			  ) -
			  getElementHeight(this.e, this.getBorderHeight()) -
			  this.top.SubmenuOffsetY;
		break;

	case "R":				// Right
	default:
		// do nothing
	}
}

Menu.prototype.reposition_menu = function() {

	// Recompute coordinates if display is off the page.

	var e = this.e;
	var b = document.body;
	var wright  = b.scrollLeft + (b.clientWidth  ? b.clientWidth  : b.offsetWidth);
	var wbottom = b.scrollTop  + (b.clientHeight ? b.clientHeight : b.offsetHeight);
	var eleft   = e.offsetLeft;
	var etop    = e.offsetTop;
	var eright  = eleft + e.offsetWidth;
	var ebottom = etop  + e.offsetHeight;
	var reposition = false;

	if (eright > wright) {
		eleft = wright - e.offsetWidth;
		reposition = true;
	}

	if (ebottom > wbottom) {
		etop = wbottom - e.offsetHeight;
		reposition = true;
	}

	if (eleft < b.scrollLeft) {
		eleft = b.scrollLeft;
		reposition = true;
	}

	if (etop < b.scrollTop) {
		etop = b.scrollTop;
		reposition = true;
	}

	if (reposition) {
		// Reposition the menu to be within the window.
		this.e.style.left = eleft + "px";
		this.e.style.top  = etop  + "px";
		if (bv.isOpr5) this.e.style.pixelTop  = etop;
		this.is_located = false;	// reset for next time

		debug("Menu.reposition_menu: x="+eleft+", y="+etop);
	}
}

Menu.prototype.compute_horz_menu_x = function() {

	// Compute X coordinate for a horizontally oriented menu so that it
	// lines up vertically to the header element. Used when menu position
	// is "U" or "D".

	// The horizontal alignment is from the left of the header border
	// to just inside the submenu's border and padding.

	return getEventElementLeft(this.evt, this.header_e,
				this.get_header_border_width("left"),
				this.get_header_padding("left")) -
		 tableOffset(this.e) -
		 this.get_padding("left") -
		 this.get_border_width("left");
}

Menu.prototype.compute_vert_menu_y = function() {

	// Compute Y coordinate for a vertically oriented menu so that it
	// lines up horizontally to the header element. Used when menu position
	// is "L" or "R".

	// The vertical alignment is from the top of the header border
	// to just inside the submenu's border and padding.

	return this.get_header_top() -
		 tableOffset(this.e) -
		 this.get_padding("top") -
		 this.get_border_width("top");
}

Menu.prototype.get_header_top = function() {

	// This routine deals with the "bug" (a DOM deficiency) in
	// Netscape and Opera where an <a> tag encloses an <img> tag and
	// the the <a> tag doesn't have the CSS style setting display:block;,
	// which is common. The top coordinate returned is the top of
	// the <a> tag's text, not the top of the image.
	
	var header_e = this.header_e;

	if (bv.isIE) {

		return getEventElementTop(this.evt, header_e,
					this.get_header_border_width("top"),
					this.get_header_padding("top"));
	}
		
	// Look for an image surrounded by the header <a> tag that's not a
	// block element. Opera cannot retreive style settings, so it always
	// compares results.

	// Note: this does not work for Opera 7 as both the <img> and <a> tags
	// return the same offset, which are both wrong in table cells that have
	// their size automatically increased because of the content of the
	// surrounding cells. The work-around is to surround the <a> tag
	// with a <div> tag on the web page.

	if ( ((bv.isOpr5 || bv.isOpr6) && this.evt.target.toString().indexOf("HTMLImage") > 0)
	|| (this.evt.target.nodeName == "IMG"
	&& (getElementStyle(header_e,"display") == "inline")) ) {

		debug("Menu.get_header_top: Comparing top of IMG tag enclosed by header A tag.");

		var iy = getEventElementTop(this.evt);
		var hy = getEventElementTop(this.evt, header_e);
		return iy < hy ? iy : hy;
	}

	return getEventElementTop(this.evt, header_e);
}

Menu.prototype.get_header_height = function() {

	// Like get_header_top() above, this routine deals with the bug in
	// Netscape of an <a> tag enclosing an <img> tag. Unless the <a>
	// tag has a display:block CSS style setting, the height of the
	// element is the text height, not the image height.

	if (bv.isIE) return getElementHeight(this.header_e);

	if ( ((bv.isOpr5 || bv.isOpr6) && this.evt.target.toString().indexOf("HTMLImage") > 0)
	|| (this.evt.target.nodeName == "IMG"
	&& (bv.isOpr7 || getElementStyle(this.header_e,"display") == "inline")) ) {

		debug("Menu.get_header_height: Comparing height of IMG tag with header tag.");

		var ih = getElementHeight(this.evt.target);
		var hh = getElementHeight(this.header_e);
		return ih > hh ? ih : hh;
	}

	return getElementHeight(this.header_e);
}

/*
 * Style properties
 */

Menu.prototype.get_border_width = function(position) {

	// Get border width
	// Look at style settings before menu configuration variables.

	var width = getBorderWidth(this.e,position);

	if (getBorderWidth.found) return width;

	if (this.is_submenu && this.top.SubmenuBorderWidth != null)
		return this.top.SubmenuBorderWidth;

	if (this.top.MenuBorderWidth != null)
		return this.top.MenuBorderWidth;

	return 0;
}

Menu.prototype.get_header_border_width = function(position) {

	// Get header border width
	// Look at style settings before menu configuration variables.

	var width = getBorderWidth(this.header_e,position);
	if (getBorderWidth.found) return width;

	if (this.is_submenu) {
		if (this.top.SubmenuHeaderBorderWidth != null)
			return this.top.SubmenuHeaderBorderWidth;
	}
	else {
		if (this.top.MenuHeaderBorderWidth != null) 
			return this.top.MenuHeaderBorderWidth;
	}
	return 0;
}

Menu.prototype.get_padding = function(position) {

	// Get padding
	// Look at style settings before menu configuration variables.

	var padding = getPadding(this.e,position);

	if (getPadding.found) return padding;

	if (this.is_submenu) {
		if (this.top.SubmenuPaddingHorz != null
		&& (position == "left" || position == "right"))
			return this.top.SubmenuPaddingHorz;

		if (this.top.SubmenuPaddingVert != null
		&& (position == "top" || position == "bottom"))
			return this.top.SubmenuPaddingVert;

		if (this.top.SubmenuPadding != null)
			return this.top.SubmenuPadding;
	}

	if (this.top.MenuPaddingHorz != null
	&& (position == "left" || position == "right"))
		return this.top.MenuPaddingHorz;

	if (this.top.MenuPaddingVert != null
	&& (position == "top" || position == "bottom"))
		return this.top.MenuPaddingVert;

	if (this.top.MenuPadding != null)
		return this.top.MenuPadding;

	return 0;
}

Menu.prototype.get_header_padding = function(position) {

	// Get padding
	// Look at style settings before menu configuration variables.

	var padding = getPadding(this.header_e,position);

	if (getPadding.found) return padding;

	if (this.is_submenu) {
		if (this.top.SubmenuHeaderPaddingHorz != null
		&& (position == "left" || position == "right"))
			return this.top.SubmenuHeaderPaddingHorz;

		if (this.top.SubmenuHeaderPaddingVert != null
		&& (position == "top" || position == "bottom"))
			return this.top.SubmenuHeaderPaddingVert;

		if (this.top.SubmenuHeaderPadding != null)
			return this.top.SubmenuHeaderPadding;
	}

	return 0;
}

Menu.prototype.getBorderWidth = function() {
	if (this.BorderWidth == null) this.BorderWidth = Math.floor((this.get_border_width("left") + this.get_border_width("right"))/2);
	return this.BorderWidth;
}

Menu.prototype.getBorderHeight = function() {
	if (this.BorderHeight == null) this.BorderHeight = Math.floor((this.get_border_width("top") + this.get_border_width("bottom"))/2);
	return this.BorderHeight;
}

Menu.prototype.getHeaderBorderWidth = function() {
	if (this.HeaderBorderWidth == null) this.HeaderBorderWidth = Math.floor((this.get_header_border_width("left") + this.get_header_border_width("right"))/2);
	return this.HeaderBorderWidth;
}

/* ------------------------------------------------------------------------ */

/*
 * Utility functions
 */

function parseNum(s) {

	// Always return a parsed integer. No NaN!

	if (s) return parseInt("0"+s,10);
	return 0;
}

function getElementRef(id) {

	// Return a reference to the document element.
	// The id can be a string or an object.

	if (! id) return null;
	if (typeof id == "string") {
		if (document.getElementById) return document.getElementById(id);
		if (document.all) return document.all[id];
		return null;
	}
	if (typeof id == "object") return id;
	return null;
}

function getEventElementLeft(evt,target		,border_width,padding) {

	// Get the left (x) coordinate of target element.
	// The coordinate includes padding and border but not the margin.
	// The specified target element can be a container element to the
	// element actually receiving the event.

	// The border_width argument is used only for IE4 or IE5.
	// The padding argument is used only for IE5Mac.
	// They allow constants to be used for specifying target border size
	// and padding. They are computed if missing.

	if (! evt) evt = window.event;		// IE default value
	if (! target) target = evt.target     ? evt.target     : 
			       evt.srcElement ? evt.srcElement : null;
	target = getElementRef(target);		// can be string or object

	var x;					// returned coordinate

	var e, ox;
	var target_offset = 0, target_found = false;

	if (bv.isIE5 || bv.isOpr5) {

		if (border_width == null) border_width = getBorderWidth(target,"left");

		// Get element initially receiving the event.
		e = evt.target ? evt.target : evt.srcElement ? evt.srcElement : null;

		debug("getEventElementLeft(): IE5: begin evt.offsetX="+evt.offsetX+
		  ", border_width="+border_width,e);

		// Check for the case of: <a ...><img ...></a>
		// Unless a width has been specified for an <a> tag, it is not
		// in the chain of nodes traversed by the offsetParent link.

		if (! e.nodeName || e.nodeName != "IMG"
		|| ! e.parentNode || e.parentNode.nodeName != "A"
		|| ! e.offsetParent || e.offsetParent.nodeName == "A") {

			while (e) {

				// Add offset between event element and target element.
				if (e == target) break;

				debug("getEventElementLeft(): IE5: adding to"+
				  " target_offset "+e.offsetLeft+
				  ", target="+target.nodeName ,e);

				target_offset += e.offsetLeft;

				e = e.offsetParent;
			}
		}

		target_offset += border_width;

		x = evt.clientX - evt.offsetX - target_offset;

		if (bv.isIE5) x += document.body.scrollLeft;

		debug("getEventElementLeft(): IE5"+
		  " Computed x="+x+
		  ", (evt.offsetX="+evt.offsetX+
		  ", evt.clientX="+evt.clientX+
		  ", scrollLeft="+document.body.scrollLeft+
		  ", target_offset="+target_offset+
		  ", border_width="+border_width+
		  ", padding="+padding+
		  ")",target);

		return x;
	}
	else if (bv.isIE5Mac) {

		if (border_width == null) border_width = getBorderWidth(e,"left");
		if (padding == null)      padding      = getPadding(e,"left");

		// Get element initially receiving the event.
		e = evt.target ? evt.target : evt.srcElement ? evt.srcElement : null;

		debug("getEventElementLeft(): IE5Mac: begin "+
		  ", padding="+padding+", border="+border_width,e);

		// Check for the case of: <a ...><img ...></a>

		if (e.nodeName && e.nodeName == "IMG"
		&& e.parentNode && e.parentNode.nodeName == "A"
		&& e.offsetParent && e.offsetParent.nodeName != "A") {

			x = evt.clientX - evt.offsetX - border_width - padding;

			debug("getEventElementLeft(): IE5Mac:"+
			  " Computed x="+x+", (ox="+ox+
			  ", evt.clientX="+evt.clientX+
			  ", evt.offsetX="+evt.offsetX+
			  ", border_width="+border_width+
			  ", padding="+padding+
			  ")",target);
		}
		else {
			ox = 0;
			while (e) {
				debug("getEventElementLeft(): IE5Mac: adding "+e.offsetLeft,e);

				ox += e.offsetLeft;

				if (e == target) target_found = true;

				// Add offset between event element and target element.
				if (! target_found) { target_offset += e.offsetLeft; }

				e = e.offsetParent;
			}

			target_offset += border_width + padding;

			x = getMargin(document.body,"left") + ox - target_offset;

			debug("getEventElementLeft(): IE5Mac:"+
			  " Computed x="+x+", (ox="+ox+
			  ", border_width="+border_width+
			  ", padding="+padding+
			  ", body margin="+body_margin+
			  ", target_offset="+target_offset+
			  ")",target);
		}

		return x;
	}
	else if (bv.isOpr6) {

		x = getElementLeft(target);

		debug("getEventElementLeft(): Opr6: Computed x="+x);

		return x;
	}
	else {	// (bv.isIE6 || bv.isNS || bv.isMoz || bv.isOpr7)

		// Ignore TEXT nodes, go up to first ELEMENT node.
		e = target;
		while (e && e.nodeType != 1) { e = e.parentNode };

		x = getElementLeft(e);

		debug("getEventElementLeft(): DOM: Computed x="+x);

		return x;
	}

	return 0;	// cannot determine coordinate
}

function getEventElementTop(evt,target		,border_width,padding) {

	// Get the top (y) coordinate of target element.
	// The coordinate includes padding and border but not the margin.
	// The specified target element can be a container element to the
	// element actually receiving the event.

	// All arguments are optional except evt in Netscape (DOM).
	// The border_width argument is used only for IE4 or IE5.
	// The padding argument is used only for IE5Mac.
	// They allow constants to be used for specifying target border size
	// and padding. They are computed if missing.

	// Note: with the exception of Internet Explorer, the target element
	// must be a block type element (not inline) otherwise the Y 
	// coordinate is incorrect. It is based on the text size, and if
	// the element is an <a> tag enclosing an image, it is incorrect.

	if (! evt) evt = window.event;		// IE default value
	if (! target) target = evt.target     ? evt.target     : 
			       evt.srcElement ? evt.srcElement : null;
	target = getElementRef(target);		// can be string or object

	var y;					// returned coordinate

	var e, oy;
	var target_offset = 0, target_found = false;

	if (bv.isIE5 || bv.isIE5Mac || bv.isOpr5) {

		if (border_width == null) border_width = getBorderWidth(target,"top");
		if (bv.isIE5Mac && padding == null) padding = getPadding(target,"top");

		// Get element initially receiving the event.
		e = evt.target ? evt.target : evt.srcElement ? evt.srcElement : null;

		debug("getEventElementTop(): IE5: begin evt.offsetY="+evt.offsetY+
		  ", border_width="+border_width,e);

		// Get offset within element receiving the event.
		oy = evt.offsetY;

		// Check for the case of: <a ...><img ...></a>
		// Unless a width has been specified for an <a> tag, it is not
		// in the chain of nodes traversed by the offsetParent link.
		// This also applies to Opera 5 but e.nodeName is undefined
		// so we cannot check it or correct it.

		if (! e.nodeName || e.nodeName != "IMG"
		|| ! e.parentNode || e.parentNode.nodeName != "A"
		|| ! e.offsetParent || e.offsetParent.nodeName == "A") {

			while (e) {

				// Add offset between event element and target element.
				if (e == target) break;

				debug("getEventElementTop(): IE5: adding to"+
				  " target_offset "+e.offsetTop+
				  ", target="+target.nodeName ,e);

				target_offset += e.offsetTop;

				e = e.offsetParent;
			}
		}

		target_offset += border_width;
		if (bv.isIE5Mac) target_offset += padding;

		y = evt.clientY - oy - target_offset;

		if (bv.isIE5) y += document.body.scrollTop;

		debug("getEventElementTop(): IE5:"+
		  " Computed y="+y+
		  ", (oy="+oy+
		  ", scrollTop="+document.body.scrollTop+
		  ", evt.clientY="+evt.clientY+
		  ", target_offset="+target_offset+
		  ", border_width="+border_width+
		  ")",target);

		return y;
	}
	else if (bv.isOpr6) {

		y = getElementTop(target) + Menu.BodyMarginTop;

		debug("getEventElementTop(): Opr6: Computed y="+y);

		return y;
	}
	else {	// (bv.isIE6 || bv.isNS || bv.isMoz || bv.isOpr7)

		// Ignore TEXT nodes, go up to first ELEMENT node.
		e = target;
		while (e && e.nodeType != 1) { e = e.parentNode };

		y = getElementTop(e);

		debug("getEventElementTop(): DOM: Computed y="+y);

		return y;
	}

	return 0;	// cannot determine coordinate
}

function getElementLeft(e) {

	// Get the left (x) coordinate of an element.
	// The coordinate includes padding and border but not the margin.
	// This only works with version 6 browsers (IE6, NS6, Opr7).

	debug("getElementLeft(): begin e.offsetLeft="+e.offsetLeft,e);

	var x = e.offsetLeft;

	while ((e = e.offsetParent)) {
		debug("getElementLeft(): adding e.offsetLeft="+e.offsetLeft+
		  " and left border.",e);

		x += e.offsetLeft;
		x += getBorderWidth(e,"left");

		// Note: Netscape adds an extra 1px to the border between 
		// cells of tables that have a border defined. It gets
		// returned in the getBorderWidth() for the TD or TH
		// element when stepping through offsetParent nodes.
		// Internet Explorer 6 doesn't include its added pixel
		// in getBorderWidth(), so it needs to add that extra pixel.

		if (parseNum(e.border) > 0) {
			if (bv.isNS) x--;
			if (bv.isIE6 || bv.isOpr7) x++;
		}
	}

	debug("getElementLeft(): Computed x="+x,e);

	return x;
}

function getElementTop(e) {

	// Get the top (y) coordinate of an element.
	// The coordinate includes padding and border but not the margin.
	// This only works with version 6 browsers (IE6, NS6, Opr7).

	debug("getElementTop(): begin e.offsetTop="+e.offsetTop,e);

	var y = e.offsetTop;

	while ((e = e.offsetParent)) {
		debug("getElementTop(): adding e.offsetTop="+e.offsetTop+
		  " and top border.",e);

		y += e.offsetTop;
		y += getBorderWidth(e,"top");

		// Note: Netscape adds an extra 1px to the border between 
		// cells of tables that have a border defined, and one on top.

		if (parseNum(e.border) > 0) {
			if (bv.isNS) y--;
			if (bv.isIE6 || bv.isOpr7) y++;
		}
	}

	debug("getElementTop(): Computed y="+y,e);

	return y;
}

function getElementStyle(e,
			dom_prop,		css_attr,
			dom_gen_prop,		css_gen_attr,     position,
			dom_border_prop,	css_border_attr,  component,
			dom_border_gen_prop,	css_border_gen_attr) {

	// Get the element's style settings.

	// Both the DOM property names (e.g. borderBottomWidth) and the
	// corresponding CSS attribute names (border-bottom-width)
	// needs to be specified. If css_attr not set, it is set to dom_prop.
	// General properties and attributes for dom_prop and css_attr
	// are checked if specified. For example: a general property dom_gen_prop
	// for borderBottomWidth is borderWidth, and the css_gen_attr is
	// "border-width". The position argument can be one of:
	//   "top", "bottom", "left", or "right".
	// The dom_border_gen_prop and css_border_gen_attr are for the
	// special shortcut method of specifying borders like:
	//   "border:solid black 1px"
	// The last 5 arguments are only used for borders.
	// The multiple components of the "font" attribute are not parsed here
	// as they are for borders.

	if (! css_attr) css_attr = dom_prop;

	var style;

	if (e.style) {
		if ((style = e.style[dom_prop])) {
			debug("getElementStyle: e.style["+dom_prop+"]="+style,e);
		}
		else if (dom_gen_prop != null && (style = e.style[dom_gen_prop])) {
			debug("getElementStyle: e.style["+dom_gen_prop+"],"+position+"="+style,e);
			style = parseStyleSetting(style,position);
		}
		else if (dom_border_prop != null && (style = e.style[dom_border_prop])) {
			debug("getElementStyle: e.style["+dom_border_prop+"]="+style,e);
			style = parseBorderSetting(style,component);
		}
		else if (dom_border_gen_prop != null && (style = e.style[dom_border_gen_prop])) {
			debug("getElementStyle: e.style["+dom_border_gen_prop+"]="+style,e);
			style = parseBorderSetting(style,component);
		}

		if (style) {
			if (component == null || component != "width") {
				return style;
			}
			else {
				// IE5Mac returns "medium" for unset border widths.
				// Return style attribute setting only if it is a number,
				// otherwise look in style sheet settings below.

				if (style.match(/\d/)) return style;
			}
		}
	}

	var v = document.defaultView;
	if (v && v.getComputedStyle) {
		// W3C DOM (Netscape)
		style = v.getComputedStyle(e,"").getPropertyValue(css_attr);

		if (style != null) {
			debug("getElementStyle: getComputedStyle("+css_attr+")="+style,e);
			return style;
		}

		if (css_gen_attr != null) {
			style = parseStyleSetting(
			  v.getComputedStyle(e,"").getPropertyValue(css_gen_attr),
			  position);

			if (style != null) {
				debug("getElementStyle: getComputedStyle("+css_gen_attr+
				  ","+position+")="+style,e);
				return style;
			}
		}

		if (css_border_attr != null) {
			style = parseBorderSetting(
			  v.getComputedStyle(e,"").getPropertyValue(css_border_attr),
			  component);

			if (style != null) {
				debug("getElementStyle: getComputedStyle("+
				  css_border_attr+")="+style,e);
				return style;
			}
		}

		if (css_border_gen_attr != null) {
			style = parseBorderSetting(
			  v.getComputedStyle(e,"").getPropertyValue(css_border_gen_attr),
			  component);

			if (style != null) {
				debug("getElementStyle: getComputedStyle("+
				  css_border_gen_attr+")="+style,e);
				return style;
			}
		}
	}

	if (document.all && e.currentStyle) { // IE5 not IE4
		style = e.currentStyle[dom_prop];

		if (style != null) {
			debug("getElementStyle: e.currentStyle["+dom_prop+"]="+style,e);
			return style;
		}

		if (dom_gen_prop != null) {
			style = parseStyleSetting(
			  e.currentStyle[dom_gen_prop],
			  position);

			if (style != null) {
				debug("getElementStyle: e.currentStyle["+dom_gen_prop+
				  "],"+position+"="+style,e);
				return style;
			}
		}

		if (dom_border_prop != null) {
			style = parseBorderSetting(e.currentStyle[dom_border_prop],component);

			if (style != null) {
				debug("getElementStyle: e.currentStyle["+dom_border_prop+
				  "]="+style,e);
				return style;
			}
		}

		if (dom_border_gen_prop != null) {
			style = parseBorderSetting(e.currentStyle[dom_border_gen_prop],component);

			if (style != null) {
				debug("getElementStyle: e.currentStyle["+dom_border_gen_prop+
				  "]="+style,e);
				return style;
			}
		}
	}

	debug("getElementStyle: Cannot find style for "+dom_prop+"("+css_attr+")",e);

	return null;
}

function parseStyleSetting(setting,position) {

	// Parse a style setting for top, bottom, left, or right measurment settings.

	if (! setting) return null;
	if (! position) return setting;
	
	// Match one, two, three, or four space separated settings.
	setting.match(/(\S+)(\s+(\S+)(\s+(\S+)(\s+(\S+))?)?)?/);

	if (position == "top") {
		return RegExp.$1;
	}

	if (position == "right") {
		if (RegExp.$3) return RegExp.$3;
		return RegExp.$1;
	}

	if (position == "bottom") {
		if (RegExp.$5) return RegExp.$5;
		return RegExp.$1;
	}

	if (position == "left") {
		if (RegExp.$7) return RegExp.$7;
		if (RegExp.$3) return RegExp.$3;
		return RegExp.$1;
	}
}

function parseBorderSetting(setting,component) {

	// Parse number of pixels from a border setting like: "solid black 1px".
	// The component argument can be "style", "color", or "width".
	// Only "width" is parsed here for the menus.

	if (! component || component == "width") {
		if (setting.match(/(\d+px)/)) return RegExp.$1;
		return 0;
	}
}

function getBorderWidth(e,position) {

	// Get element's border width.
	// Optional position argument can be "top", "bottom", "left", or "right".
	// This routine always returns a number. A static flag getBorderWidth.found
	// is set to true if style setting was found.

	var width;
	
	var dom_gen_prop = "borderWidth";
	var css_gen_attr = "border-width";

	var dom_border_gen_prop = "border";
	var css_border_gen_attr = "border";

	if (! position) {
		width = getElementStyle(e,
			  dom_gen_prop,		css_gen_attr,
			  null,		 	null,			null,
			  dom_border_gen_prop,	css_border_gen_attr,	"width");
	}
	else {
		// Capitalize position to use to assemble DOM property names.
		var Position = position.substr(0,1).toUpperCase()+position.substr(1);

		var dom_prop = "border"  + Position + "Width";		//borderLeftWidth
		var css_attr = "border-" + position + "-width";		//border-left-width

		var dom_border_prop = "border"  + Position;		//borderLeft
		var css_border_attr = "border-" + position;		//border-left

		width = getElementStyle(e,
			  dom_prop,		css_attr,
			  dom_gen_prop,		css_gen_attr,		position,
			  dom_border_prop,	css_border_attr,	"width",
			  dom_border_gen_prop,	css_border_gen_attr);
	}

	// Ignore keywords "thin", "medium", and "thick".
	if (width == null || ! width.match(/\d/)) {
		getBorderWidth.found = false;
		debug("getBorderWidth(): position="+position+", border width="+
		  (width?width:"''")+" NOT FOUND",e);
		return 0;
	}
	else {
		getBorderWidth.found = true;
		debug("getBorderWidth(): position="+position+", width="+width,e);

		// Parse number of pixels. Assume units are "px".
		return parseNum(width);
	}
}

function getPadding(e,position) {
	
	// Get element's padding dimension.
	// Optional position argument can be "top", "bottom", "left", or "right".
	// This routine always returns a number. A static flag getPadding.found
	// is set to true if style setting was found.

	var padding;
	
	var s = "padding";

	if (! position) {
		padding = getElementStyle(e,s,s);
	}
	else {
		// Capitalize position to use to assemble DOM property names.
		var Position = position.substr(0,1).toUpperCase()+position.substr(1);
		padding = getElementStyle(e, (s+Position), (s+"-"+position), s, s, position);
	}

	if (padding == null) {
		getPadding.found = false;
		debug("getPadding(): position="+position+", padding NOT FOUND",e);
		return 0;
	}
	else {
		getPadding.found = true;
		debug("getPadding(): position="+position+", padding="+padding,e);

		// Parse number of pixels. Assume units are "px".
		return parseNum(padding);
	}
}

function getMargin(e,position) {
	
	// Get element's margin dimension.
	// Optional position argument can be "top", "bottom", "left", or "right".
	// This routine always returns a number. A static flag getMargin.found
	// is set to true if style setting was found.

	var margin;
	
	var s = "margin";

	if (! position) {
		margin = getElementStyle(e,s,s);
	}
	else {
		// Capitalize position to use to assemble DOM property names.
		var Position = position.substr(0,1).toUpperCase()+position.substr(1);
		margin = getElementStyle(e, (s+Position), (s+"-"+position), s, s, position);
	}

	if (margin == null) {
		getMargin.found = false;
		debug("getMargin(): position="+position+", margin NOT FOUND",e);
		return 0;
	}
	else {
		getMargin.found = true;
		debug("getMargin(): position="+position+", margin="+margin,e);

		// Parse number of pixels. Assume units are "px".
		return parseNum(margin);
	}
}

function tableOffset(e) {

	// Return the sum of the cellspacing, cellpadding, and border dimensions.

	// If cellSpacing or cellPadding is not set, their value is set to 1.

	var offset = 0;

	// Opera 5 and 6 do not have nodeName properties. Assume e is correct.

	if (e && (e.nodeName == "TABLE" || bv.isOpr5 || bv.isOpr6)) {

		debug("tableOffset: computing offset:"+
		  " cellPadding="+e.cellPadding+
		  ", cellSpacing="+e.cellSpacing+
		  ", border="+e.border,e);

		offset = (e.cellPadding == "" ? 1 : parseNum(e.cellPadding)) +
			 (e.cellSpacing == "" ? 1 : parseNum(e.cellSpacing)) +
			 parseNum(e.border);

		if (bv.isIE && e.cellPadding == "" && e.cellSpacing == "") offset++;
	}

	debug("tableOffset: offset="+offset,e);

	return offset;
}

/* Based on "Dynamic HTML The Definitive Reference" 2nd. Ed. by Danny Goodman p92. */

function getElementWidth(e,	border_width) {
	// Width including padding and borders, not margins.

	// The border_width argument is optional and only used for IE.
	// It allows constants to be used for specifying border sizes.

	var result = 0;
	if (e.offsetWidth) {	// (bv.isIE && bv.isIE5Mac && bv.isOpr)

		if (bv.isIE5Mac) {
			result = e.offsetWidth;
			debug("getElementWidth(): IE5Mac: offsetWidth="+result,e);
		}
		else {

			if (e.scrollWidth) {	result = e.scrollWidth; }
			else {			result = e.offsetWidth; }

			if (bv.isIE) {
				if (border_width == null) {
					result += getBorderWidth(e,"left") +
						  getBorderWidth(e,"right");
				}
				else {
					result += border_width * 2;
				}
			}
			debug("getElementWidth(): offsetWidth result="+result,e);
		}
	}
	else if (e.clip && e.clip.width) {
		result = e.clip.width;
		debug("getElementWidth(): clip.width="+result,e);
	}
	else if (e.style && e.style.pixelWidth) {
		result = e.style.pixelWidth;
		debug("getElementWidth(): pixelWidth="+result,e);
	}
	return parseInt(result);
}

function getElementHeight(e) {
	// Height including padding and borders, not margins.
	var result = 0;
	if (e.offsetHeight) {

		// Note: in Netscape && Opera, if element e is an <a> tag
		// enclosing an <img>, and the CSS style for the <a> tag
		// display attribute has not been set to "block", the height
		// returned is the <a> tag's text box height, not the <img>
		// tag's height.

		result = e.offsetHeight;
		debug("getElementHeight(): offsetHeight="+result,e);
	}
	else if (e.clip && e.clip.height) {
		result = e.clip.height;
		debug("getElementHeight(): clip.height="+result,e);
	}
	else if (e.style && e.style.pixelHeight) {
		result = e.style.pixelHeight;
		debug("getElementHeight(): pixelHeight="+result,e);
	}
	return parseInt(result);
}

/* ------------------------------------------------------------------------ */

/*
 * Debug
 */

function debug(msg,e) {

	if (! debug.on) return;

	clearTimeout(debug.break_time);

	// Turn off debug if debug message window has been closed.
	if (! debug.w || debug.w.closed) { debug.on = false; return; }

	// Note: This does not work well in Opera 7 as it auto-closes the debug
	// window document after a pause which causes the next debug message
	// to erase the previous message. 

	debug.w.document.write("<br>");
	debug.w.document.writeln(msg);
	if (e && e.nodeName) {
		debug.w.document.writeln(" &nbsp; &lt;"+e.nodeName);
		if (e.href) {
			debug.w.document.writeln(" href="+e.href);
		}
		debug.w.document.writeln("&gt;");
	}
	debug.w.scrollBy(0,100);	// an arbitrarily high number

	if (! bv.isOpr) debug.break_time = setTimeout(
	 'debug.w.document.writeln("<br><br>");debug.w.scrollBy(0,100)',
	 1000);
}

debug.turn_on = function() {
        debug.on = true;
        if (! debug.w || debug.w.closed) {
		debug.w = window.open();
		debug.w.blur();
        }

	debug.w.document.writeln("Browser version: "+bv());
}

debug.turn_off = function() {
        debug.on = false;
}

if (! debug.on) debug.on = false;	// set to true for debug output.

if (debug.on) debug.turn_on();		// open debug window

/*
 * Display mouse coordinates.
 */

function mouse_coord(evt) {
        // Display mouse coordinates in status bar.
	var x,y;
        if (window.event) {
		x = document.body.scrollLeft + window.event.x;
		y = document.body.scrollTop  + window.event.y;
        }
        else {
		x = evt.pageX;
		y = evt.pageY;
	}

	window.status = "X=" + x + " Y=" + y;

	// Uncomment following to output mouse coordinates in debug window too.
	debug("x="+x+", y="+y);
}
mouse_coord.turn_on = function() {
	if (! document.body) {
		// Need to wait for document to load before attaching event handler.
		setTimeout("mouse_coord.turn_on()",1000);
		return;
	}

	// Attach event handler.
        document.body.onmousemove = mouse_coord;
}
mouse_coord.turn_off = function() {
        document.body.onmousemove = null;
        window.status = "";
}

if (mouse_coord.on) mouse_coord.turn_on();

