package Acme::Base::Schema::Master;

=head1 NAME

Acme::Base::Schema::Master - Master object for Acme::Base::Schema

=head1 SYNOPSIS

    use Acme::Base::Schema::Master;

    my $join_clause = $t->master;

    my $mt = new Acme::Base::Schema::Master $join_clause;

    for my $tab ($mt->master_tables) {
	    push @master_fields, $mt->fields($tab);
    }

    my $field = $mt->master_field($table_name);

    my @join_hash = $mt->joins();

=head1 DESCRIPTION

Acme::Base::Schema::Master provides an object interface to the parsed tokens
of the master join clause specified in the schema.

The joins() method returns an array of pointers to hash structures that
has parsed components of the join clause used to join the master table. It
is the same structure as C<Acme::Base::Schema::Join->joins> method.

=head1 CHANGE LOG

 6/03/00  Initial version.
12/10/01  Subclass Acme::Base::Schema::Join.

=head1 AUTHOR

Mack Pexton <mack@acmebase.org>

=head1 COPYRIGHT

Copyright 2000, Mackley F. Pexton.  All rights reserved.

This is open software and is licensed under the terms of the GNU General Public License.

=cut

use Acme::Base::Schema::Join;

@Acme::Base::Schema::Master::ISA = qw( Acme::Base::Schema::Join );

sub master_tables { return Acme::Base::Schema::Join::join_tables(@_); }
sub master_field  { return Acme::Base::Schema::Join::join_field(@_);  }

1; 
