package Acme::Db::RecordSet;

=head1 NAME

Acme::Db::RecordSet - Database Record Sets for Form Processing

=head1 SYNOPSIS

    use Acme::Db::RecordSet;

    $rs = new Acme::Db::RecordSet;

    $val = $rs->get($table,$field,$record_number);

    $rs->set($table,$field,$record_number,$value);

=head1 DESCRIPTION

The I<Acme::Db::RecordSet> module holds database record sets for form processing.
The record set is hierarchical, a different number of records can be stored
for each table.

=head1 CHANGE LOG

 5/9/00  Initial version.

=head1 AUTHOR

Mack Pexton <mack@acmebase.org>

=head1 COPYRIGHT

Copyright 2000, Mackley F. Pexton.  All rights reserved.

This is open software and is licensed under the terms of the GNU General Public License.

=cut

sub new {
	my $this = shift;
	my $class = ref($this) || $this;

	my $self = {};

	return bless $self, $class;
}

sub set {
	my ($self, $table, $field, $recno, $value) = @_;

	$recno ||= 0;
	$self->{$table} ||= [];
	$self->{$table}->[$recno] ||= {};
	$self->{$table}->[$recno]->{$field} = $value;
}

sub get {
	my ($self, $table, $field, $recno) = @_;

	$recno ||= 0;
	return $self->{$table}->[$recno]->{$field};
}

sub tables {
	my ($self) = @_;

	# List tables in the record set.
	return keys %$self;
}

sub records {
	my ($self, $table) = @_;

	# Return a list of pointers to arguments for each record of table.
	return @{$self->{$table}};
}

1;
