package Acme::File;

=head1 NAME

Acme::File - File routines for Acme

=head1 SYNOPSIS

    use Acme::File qw(mktemp);

    $my_temp_filename = mktemp('temp','tmp');

=head1 DESCRIPTION

The I<Acme::File> module is for file routines specific to the Acme system.

The mktemp() function returns a file name that can be used as a unique temporary
file.

=head1 CHANGES

 6/97     Initial version
 4/16/01  Temporary file name uses a random number to make it unpredictable.

=head1 AUTHOR

Mack Pexton <mack@acmebase.org>

=head1 COPYRIGHT

Copyright 1997-2001, Mackley F. Pexton.  All rights reserved.

This is open software and is licensed under the terms of the GNU General Public License.

=cut

require 5.000;
require Exporter;

@ISA = qw(Exporter);
@EXPORT_OK = qw(mktemp);

use Acme::Config;

#-------------------------------------------------------------------------------

sub mktemp {		# make unique temporary file name
	my ($pattern,$type) = @_;
	$type ||= "tmp";
	my $unique = int(rand 9999) + 1;	# make file name non-predictable
	my $f;

	my $tmpdir = Acme::Config->Temp_Dir || '/var/tmp';

	# Iterate until a file name is found which does not exist.
	while ( ($f = "$tmpdir/$pattern$$-$unique.$type") && -e $f ) {
		$unique++
	}

	return $f;
}

1;
