package Acme::HTML::AdControl;

=head1 NAME

Acme::HTML::AdControl - return HTML code for search controls

=head1 SYNOPSIS

  use Acme::HTML::AdControl;

  print new Acme::HTML::AdControl;

=head1 DESCRIPTION

The new() method creates a control to search the different advertising tables.
This is a customized control, and does not have any options.

The form's Javascript also needs to include (hopefully to be automated...):

	* the lib/common.js library for functions hilite() and unhilite().

=head1 CHANGE LOG

 12/26/01   Initial version.

=head1 AUTHOR

Mack Pexton <mack@acmebase.org>

=head1 COPYRIGHT

Copyright 1998-2001, Mackley F. Pexton.  All rights reserved.

This is open software and is licensed under the terms of the GNU General Public License.

=cut

use Acme::HTML::SearchControl;
use Acme::HTML::ListMenus;
use Acme::Options;
use Acme::Config;

$Images = Acme::Config->Images;		# location of images

sub new {
	my $this = shift;
	my $class = ref($this) || $this;

	return new Acme::HTML::SearchControl
		table   => 'ad_sources',
		title   => 'Advertising',
		include => _advertising_control();
}

#------------------------------------------------------------------------------
#
#  Private routines
#
#------------------------------------------------------------------------------

sub _advertising_control {

	my $opt = new Acme::Options;

	my $sort_order_dropdown = Acme::HTML::ListMenus->Menu(
						[
							'Name',
							'Ad Source',
							'Inquiries',
							'Sales',
							'Volume',
							'Cost',
						],
						'sort_order',
						$opt->AdResponseSortOrder,
						);

	my $ad_source_options = Acme::HTML::ListMenus->ad_sources(
						'ad_sources',
						(','.$opt->AdResponseSources.','),
						40,
						'size="5"',
						undef,
						'multiple');

	my $r_checked = $opt->AdResponseRestrictSource ? 'checked' : '';

	return <<EOF;

<!-- Start Ad Response Report Control -->
<script language="JavaScript" type="text/javascript">
function ad_response(month_offset,year_offset) {
	var f = window.document.AdvertisingOptionsForm;

	var _restrict_source = f.restrict_source.checked ? "1" : "0";
	var _sort_order = f.sort_order.value;

	// Build comma separated list of selected ad sources.
	var a = new Array;
	var j = 0;
	var fld = f.ad_sources;
	for (var i=0; i < fld.length; i++) {
		if (fld[i].selected) a[j++] = fld[i].value;
	}
	var _ad_sources = a.join(',');

	var options = "";

	// check for custom input
	if (month_offset != null) options="month_offset=" + month_offset;
	if (year_offset != null)  options="year_offset="  + year_offset;

	// display ad response report
	var settings = "copyhistory,directories,location,menubar" +
	  ",resizable,scrollbars,status,toolbar" +
	  ",width=760,height=600";
	var w = window.open("","", settings);
	w.focus();
	w.location.href = 
	  "/cgi-bin/ad_response.cgi?" + options + (options ? "&" : "") +
	  "restrict_source=" + _restrict_source +
	  "&ad_sources="     + _ad_sources      +
	  "&sort_order="     + _sort_order;

}
</script>

<div class="control-ad">

<form method="get" action="javascript:void 0" name="AdvertisingOptionsForm" onsubmit="return(false)" style="margin:0px">

<table border="0" cellspacing="0" cellpadding="2" class="control-ad">
  <tr valign="top"> 
    <td width="16">
    </td>

    <td width="105" align="right" nowrap>
    </td>

    <td width="340" nowrap>
    <font class="plain">Advertising Response Report</font>
    <div class="group">

<!-- Start Button Bar -->
<div class="button-bar">
<table>
<tr>
 <td><a href="javascript:ad_response(0)">This Month</a></td>
 <td><a href="javascript:ad_response(-1)">Last Month</a></td>
 <td><a href="javascript:ad_response(0,0)">Year to Date</a></td>
 <td><a href="javascript:ad_response()">Specify Dates</a></td>
</tr>
</table>
</div>
<!-- End Button Bar -->

    <p class="control" style="margin-top:5px">
    Sort report by
    $sort_order_dropdown
    </p>
    
    <p class="control">
    <input type="checkbox" name="restrict_source" $r_checked>
    Restrict report to following ad sources
    </p>
    
    <p class="control">
    $ad_source_options
    </p>
    </div>
    </td>

    <td width="105" align="right" nowrap>
    </td>

    <td width="16"><img src="/images/blank.gif"></td>
  </tr>

  <tr>
  <td></td>
  <td></td>
  <td>
<table align="left"><tr><td>
<a
 href="/cgi-bin/calendar.cgi?calendar_id=11"
 target="advertising_calendar"
 title="Display scheduled advertising calendar."
 onClick="new_target(this)"
><img
   src="/images/calendar.gif"
   width=28
   height=22
   border=0
   onMouseOver="hilite(this)"
   onMouseOut="unhilite(this)"
></a>
</td><td valign="middle">
<a
 href="/cgi-bin/calendar.cgi?calendar_id=11"
 target="advertising_calendar"
 title="Display scheduled advertising calendar."
 onClick="new_target(this)"
>Scheduled Advertising</a>
</td></tr>
</table>
  </td>
  <td></td>
  <td></td>
  </tr>

</table>

</form>
</div>

EOF
}

1;
