package Acme::HTML::CalendarsControl;

=head1 NAME

Acme::HTML::CalendarsControl - return HTML code for search controls

=head1 SYNOPSIS

  use Acme::HTML::CalendarsControl;

  print new Acme::HTML::CalendarsControl;

=head1 DESCRIPTION

The new() method creates a control to search the calendars tables. This is
a customized control, and does not have any options.

The form's Javascript also needs to include (hopefully to be automated...):

	* the lib/common.js library for functions hilite() and unhilite().

=head1 CHANGE LOG

 8/1/01   Initial version.

=head1 AUTHOR

Mack Pexton <mack@acmebase.org>

=head1 COPYRIGHT

Copyright 1998-2001, Mackley F. Pexton.  All rights reserved.

This is open software and is licensed under the terms of the GNU General Public License.

=cut

use Acme::Options;
use Acme::HTML::ListMenus;
use Acme::Config;

$Images = Acme::Config->Images;		# location of images

$Select_Prog = '/cgi-bin/pick.cgi';		# program to select records
						# for View and Edit programs.
sub new {
	my $this = shift;
	my $class = ref($this) || $this;

	#
	# Note: this routine returns HTML code, not an object.
	#

	my $args = { @_ };

	# Get user options if not passed in arguments.
	my $opt = $args->{opt} || new Acme::Options;

	# Define simple variables to insert into the here document below.

	# Required parameters:
	#my $table = $args->{table};
	my $table = 'calendars';	# hardwire

	# Optional parameters:
	#my $searches = $args->{searches}
	#  ||  "/cgi-bin/searches.cgi?template=${table}_canned_searches.htm",
	#my $query = $args->{query}	|| "/cgi-bin/query.cgi?table=${table}",
	#my $help = $args->{help}	|| "${Help}/search_instructions.htm",
	#my $options = $args->{options}	|| "${table}_search_options",

	# Default parameters from database schema:
	my $s = new Acme::Base::Schema;
	my $t = $s->t($table);

	my $title = $args->{title} || $t->title;

	my $view = $args->{view}
	  || $t->view_prog
	  || Acme::Config->View_Prog;
	my $edit = $args->{edit}
	  || $t->edit_prog
	  || Acme::Config->Edit_Prog;
	# The following buttons are not used.
	#my $list = $args->{list}
	#  || $t->list_prog
	#  || Acme::Config->List_Prog;
	#my $count = $args->{count}
	#  || $t->count_prog
	#  || Acme::Config->Count_Prog;
	my $new	= $args->{new}
	  || $t->new_prog
	  || Acme::Config->New_Prog;

	unless ($title) {
		# Capitalize table name
		$title = $table;
		$title =~ s/_+/ /g;			# change _ to spaces
		$title =~ s/\b(\w+)\b/ucfirst(lc($1))/eg; # capitalize words
	}

	my $lastcal = $opt->LastCalendar || '';
	my $checked = $opt->LastCalendarSuperimpose || '0';
	$checked = $checked ne '0' ? ' checked' : '';

	my $val = ",$lastcal,";		# turn into a text array value of IDs
	my $attr = qq[size="5" onDblClick="${table}_dispatch('View')"];

	my $calendar_options = Acme::HTML::ListMenus->calendars("calendar_id",$val,50,$attr,'','multi');

	#########################################################
	## Note: Need to remember to include:
	##	<script src="/scripts/common.js"></script> 
	## in the header of the file.
	#########################################################

	return <<End_Of_HTML;

<!-- Start ${title} Control -->
<script language="JavaScript" type="text/javascript">
var ${table}_wn = Math.floor(Math.random() * 10000);
function show_${table}_control() {
	document.getElementById('${table}_control').style.display = "block";	
	document.getElementById('${table}_control_open').style.display = "none";
}
function hide_${table}_control() {
	document.getElementById('${table}_control').style.display = "none";
	document.getElementById('${table}_control_open').style.display = "block";
}
function ${table}_dispatch(request) {
	var f = window.document.${title}OptionsForm;

	f.request.value = request;

	f.target = "${table}_" + ${table}_wn++;

	switch (request) {
	  case "View":
		// Open new window, calendar resizes it depending upon
		// calendar settings that are not known here.

		var settings = "copyhistory,directories,location,menubar" +
		  ",resizable,scrollbars,status,toolbar" +
		  ",width=660,height=600";

		f.action = "${view}";

		var w = window.open("",f.target, settings);
		w.focus();
		f.submit();
		break;

	  case "Edit":
		var s = f.calendar_id;
		var i, count=0;
		for (i=0; i < s.options.length; i++) {
			if (s.options[i].selected) count++;
		}

		if (count == 0) {		// nothing selected
			break;
		} else if (count == 1) {
			f.action = "${edit}";
		} else {
			f.action = "${Select_Prog}";
		}

		open_form("",f.target);
		f.submit();	
		break;

	  case "New":
		f.action="${new}";

		open_form("",f.target);
		f.submit();
		break;
	}
}
</script>

<div id="${table}_control_open" class="control-closed" style="display:block">

<table border="0" cellspacing="0" cellpadding="2" class="control-closed">
  <tr valign="top"> 
    <td width="16"><a
         href="javascript:show_${table}_control()"><img
         src="$Images/control.gif"
         alt="Show ${title} Control"
	 name="${table}_control_image"
         border="0"
         onMouseOver="hilite(this)" onMouseOut="unhilite(this)"
        ></a></td>

    <td width="105" align="right" nowrap><a
         href="javascript:show_${table}_control()"
         title="Show ${title} Control"
         onMouseOver="hilite(self.document.images['${table}_control_image'])"
	 onMouseOut="unhilite(self.document.images['${table}_control_image'])"
	 class="plain"
        ><nobr>&nbsp;&nbsp; &nbsp; ${title}</nobr></a></td>

    <td width="340" class="search-closed"><a
         href="javascript:show_${table}_control()"><img
         src="$Images/transparent.gif"
         alt="Show ${title} Control"
	 width="340"
	 height="18"
         border="0"
         onMouseOver="hilite(self.document.images['${table}_control_image'])"
	 onMouseOut="unhilite(self.document.images['${table}_control_image'])"
        ></a></td>

    <td width="105">&nbsp;</td>

    <td width="16"><img src="$Images/blank.gif"></td>
  </tr>
</table>
</div>

<div id="${table}_control" class="control-opened" style="display:none">

<table border="0" cellspacing="0" cellpadding="2" class="control-opened">
  <tr valign="top"> 
    <td width="16"><a
         href="javascript:hide_${table}_control()"><img
         src="$Images/control_open.gif"
         alt="Hide ${title} Control"
         border="0"
	 name="${table}_control_open_image"
         onMouseOver="hilite(this)" onMouseOut="unhilite(this)"
        ></a></td>

    <td width="105" align="right" nowrap><a
         href="javascript:hide_${table}_control()"
         title="Hide ${title} Control"
         onMouseOver="hilite(self.document.images['${table}_control_open_image'])"
	 onMouseOut="unhilite(self.document.images['${table}_control_open_image'])"
	 class="plain"
        ><nobr>&nbsp;&nbsp; &nbsp; ${title}</nobr></a><br>
    </td>

    <td width="340" class="group" nowrap align="left" valign="middle"> 
      <form name="${title}OptionsForm"
	action="javascript:${table}_dispatch('View')"
      	method="get"
	style="margin:0px"
      >
      <p class="control">
      <table align="center"><tr><td>
      $calendar_options
      </td></tr><tr><td>
      <input type="checkbox" name="superimpose"$checked> Superimpose
      </td></tr></table>
      </p>
      <input type="hidden" name="table" value="${table}">
      <input type="hidden" name="request" value="List">
      </div>
      </form>
    </td>

    <td width="105"><form><table border="0" cellspacing="0" cellpadding="1">
        <tr> 
          <td><input type="button" value="View" class="control" accesskey="V" onClick="${table}_dispatch('View')"></td>
        </tr>
        <tr> 
          <td><input type="button" value="Edit" class="control" accesskey="E" onClick="${table}_dispatch('Edit')"></td>
        </tr>
        <tr>
          <td><input type="button" value="New" class="control" accesskey="N" onClick="${table}_dispatch('New')"></td>
        </tr>
	</table></form>
     </td>

    <td width="16"><img src="$Images/blank.gif"></td>
  </tr>
</table>

</div>
<!-- End ${title} Control -->

End_Of_HTML
}

1;
