package Acme::HTML::CallbacksControl;

=head1 NAME

Acme::HTML::CallbacksControl - return HTML code for search controls

=head1 SYNOPSIS

  use Acme::HTML::CallbacksControl;

  print new Acme::HTML::CallbacksControl;

=head1 DESCRIPTION

The new() method creates a control to search the callbacks tables. This is
a customized control, and does not have any options.

The form's Javascript also needs to include (hopefully to be automated...):

	* a section in the keypress() handler to execute dispatch routine

	* the lib/common.js library for functions hilite() and unhilite().

=head1 CHANGE LOG

 8/10/00   Initial version.

=head1 AUTHOR

Mack Pexton <mack@acmebase.org>

=head1 COPYRIGHT

Copyright 1998-2000, Mackley F. Pexton.  All rights reserved.

This is open software and is licensed under the terms of the GNU General Public License.

=cut

use Local::Acme::User		qw(user);
use Acme::HTML::ListMenus;
use Acme::Config;

$Images = Acme::Config->Images;		# location of image files


sub new {
	my $this = shift;
	my $class = ref($this) || $this;

	#
	# Note: this routine returns HTML, not an object.
	#

	my $user_options_dropdown = Acme::HTML::ListMenus->users("uid",user());

	#########################################################
	## Note: Need to remember to include:
	##	<script src="/scripts/common.js"></script>
	## in the header of the file.
	#########################################################

	return <<End_Of_HTML;

<!-- Start Callbacks Control -->
<script language="JavaScript" type="text/javascript">
function show_callbacks_control() {
	document.getElementById('callbacks_control').style.display = "block";	
	document.getElementById('callbacks_control_open').style.display = "none";
}
function hide_callbacks_control() {
	document.getElementById('callbacks_control').style.display = "none";
	document.getElementById('callbacks_control_open').style.display = "block";
}

function callbacks(week_offset) {
	var f = document.CallbacksOptionsForm;

	var restrict_uid = f.restrict_uid.checked ? "1" : "0";
	var uid_sort = f.uid_sort.checked ? "1" : "0";
	var uid = f.uid.options[f.uid.selectedIndex].value;
	var options = '';

	// check for custom input
	if (week_offset != null) options="week_offset=" + week_offset;

	// display callbacks list
	var settings = "copyhistory,directories,location,menubar" +
	  ",resizable,scrollbars,status,toolbar" +
	  ",width=660,height=600";

	var w = window.open("","", settings);
	w.focus();
	w.location.href = 
	  "/cgi-bin/callbacks.cgi?" + options + (options ? "&" : "") +
	  "restrict_uid=" + restrict_uid +
	  "&uid=" + uid +
	  "&uid_sort=" + uid_sort;
}
function callbacks_dispatch(request) {
	var f = window.document.CallbacksOptionsForm;

	f.request.value = request;

	switch (request) {
	  case "New":
		f.action="/cgi-bin/form.cgi";

		f.target = 'callback_new' + Math.floor(Math.random() * 100);

		open_form("",f.target);
		f.submit();
		break;
	}
}
</script>

<div id="callbacks_control_open" class="control-closed" style="display:block">

<table border="0" cellspacing="0" cellpadding="2" class="control-closed">
  <tr valign="top"> 
    <td width="16"><a
         href="javascript:show_callbacks_control()"><img
         src="$Images/control.gif"
         alt="Show Callbacks Control"
	 name="callbacks_control_image"
         border="0"
         onMouseOver="hilite(this)" onMouseOut="unhilite(this)"
        ></a></td>

    <td width="105" align="right" nowrap><a
         href="javascript:show_callbacks_control()"
         title="Show Callbacks Control"
         onMouseOver="hilite(self.document.images['callbacks_control_image'])"
	 onMouseOut="unhilite(self.document.images['callbacks_control_image'])"
	 class="plain"
        ><nobr>&nbsp;&nbsp; &nbsp; Callbacks</nobr></a></td>

    <td width="340" class="search-closed"><a
         href="javascript:show_callbacks_control()"><img
         src="$Images/transparent.gif"
         alt="Show Callbacks Control"
	 width="340"
	 height="18"
         border="0"
         onMouseOver="hilite(self.document.images['callbacks_control_image'])"
	 onMouseOut="unhilite(self.document.images['callbacks_control_image'])"
        ></a></td>

    <td width="105"> 
      &nbsp;
    </td>

    <td width="16"><img src="$Images/blank.gif"></td>
  </tr>
</table>
</div>

<div id="callbacks_control" class="control-opened" style="display:none">

<table border="0" cellspacing="0" cellpadding="2" class="control-opened">
  <tr valign="top"> 
    <td width="16"><a
         href="javascript:hide_callbacks_control()"><img
         src="$Images/control_open.gif"
         alt="Hide Callbacks Control"
	 name="callbacks_control_open_image"
         border="0"
         onMouseOver="hilite(this)"
	 onMouseOut="unhilite(this)"
        ></a></td>

    <td width="105" align="right" nowrap><a
         href="javascript:hide_callbacks_control()"
         title="Hide Callbacks Control"
         onMouseOver="hilite(self.document.images['callbacks_control_open_image'])"
	 onMouseOut="unhilite(self.document.images['callbacks_control_open_image'])"
	 class="plain"
        ><nobr>&nbsp;&nbsp; &nbsp; Callbacks</nobr></a><br>

      <a href="/cgi-bin/options.cgi#callback_options" onClick="new_target(this,'options')" class="link">Options</a><br>
    </td>

    <td width="340" class="group" align="left" nowrap> 
      <form name="CallbacksOptionsForm"
	action="javascript:callbacks_dispatch('New')"
      	method="get"
	style="margin:0px"
      >

<!-- Start Button Bar -->
<div class="button-bar">
<table>
<tr>
 <td><a href="javascript:callbacks(0)">This Week</a></td>
 <td><a href="javascript:callbacks(-1)">Last Week</a></td>
 <td><a href="javascript:callbacks(1)">Next Week</a></td>
 <td><a href="javascript:callbacks()">Specify Dates</a></td>
</tr>
</table>
</div>
<!-- End Button Bar -->

      <p class="control">
      <input type="checkbox" name="uid_sort"> Sort by user ID
      <p class="control">
      <input type="checkbox" name="restrict_uid"> Restrict to following user
      <p class="control">
      $user_options_dropdown
      </p>
      <input type="hidden" name="table" value="customer_log">
      <input type="hidden" name="request" value="New">
      </form>
    </td>

    <td width="105"><form><table border="0" cellspacing="0" cellpadding="1">
        <tr>
          <td><input type="button" value="New" class="control" accesskey="N" onClick="callbacks_dispatch('New')"></td>
        </tr>
	</table></form>
     </td>

    <td width="16"><img src="$Images/blank.gif"></td>
  </tr>
</table>
</div>
<!-- End Callbacks Control -->

End_Of_HTML
}

1;
