package Acme::HTML::HideShow;

=head1 NAME

HideShow - Hide document sections with buttons to display/hide it

=head1 SYNOPSIS

    use Acme::HTML::HideShow;

    my $hs = new Acme::HTML::HideShow	show_button => "image_file_name",
    					hide_button => "image_file_name";

    print $hs->hideshow("text to be initially hidden.",$show_label,$hide_label)

    print "other stuff ..."

    print $hs->javascript;

=head1 DESCRIPTION

These routines return the HTML code to display a button and
start a span that version 4 browsers will hide from view. When the user
activates the button, the button is hidden and the text with a new button
is displayed.

The I<Acme::HTML::HideMoreLess> package have similar functions to this module.
This package provides a module interface, whereas the Acme::HTML::HideMoreLess
package are utility routines.

=head1 CHANGE LOG

 04/23/00 Initial version.
 09/06/00 Changed JavaScript reverences to 'all' collection to getElementById().

=head1 AUTHOR

Mack Pexton <mack@acmebase.org>

=head1 COPYRIGHT

Copyright 2000, Mackley F. Pexton.  All rights reserved.

This is open software and is licensed under the terms of the GNU General Public License.

=cut

use Acme::Config;

#------------------------------------------------------------------------------

sub new {
	my $this = shift;
	my $class = ref($this) || $this;

	my $self = {
		button_id => 'hs',
		button_n => 0,
		javascript => '',
		show_button => Acme::Config->Show_Button_Image,
		hide_button => Acme::Config->Hide_Button_Image,
		show_label => 'Show hidden text',
		hide_label => 'Hide text',
		@_
		};

	return bless $self, $class;
}

#------------------------------------------------------------------------------

sub hideshow {
	my $self = shift;

	my $text = shift;
	my $show_label = shift || $self->{show_label};
	my $hide_label = shift || $self->{hide_label};

	my $show_button = $self->{show_button};
	my $hide_button = $self->{hide_button};

	my $id = $self->{button_id} . $self->{button_n}++;

	$self->{javascript} .= <<End_Of_Javascript;
function show_$id() {
	var c;
	c = document.getElementById('${id}');
	c.style.display = "inline";	
	c = document.getElementById('${id}_show_button');
	c.style.display = "none";
}
function hide_$id(goto_begin) {
	var c;
	c = document.getElementById('${id}');
	c.style.display = "none";       
	c = document.getElementById('${id}_show_button');
	c.style.display = "inline";
}
End_Of_Javascript

	return	qq(<!-- Start HideShow -->) . 
		qq(<span id="${id}_show_button" style="display:inline">) .
		qq(<a href="javascript:void show_$id()">) .
		qq(<img src="$show_button" width=13 height=13) .
		qq( alt="$show_label" border=0) .
		qq( onMouseOver="hilite(this)" onMouseOut="unhilite(this)">) .
		qq(</a>) .
		qq(</span>) .
		qq(<span id="$id" style="display:none">) .
		qq(<a href="javascript:void hide_$id()">) .
		qq(<img src="$hide_button" width=13 height=13) .
		qq( alt="$hide_label" border=0) .
		qq( onMouseOver="hilite(this)" onMouseOut="unhilite(this)">) .
		qq(</a>) .
		$text .
		qq(</span>) .
		qq(<!-- End HideShow -->);
}

#------------------------------------------------------------------------------

sub javascript {
	my $self = shift;

	my $prefix = <<End_Of_HTML;
<!-- Begin HideShow Scripts -->
<script language="javascript" type="text/javascript">
End_Of_HTML

	my $postfix = <<End_Of_HTML;
</script>
<!-- End HideShow Scripts -->
End_Of_HTML

	return	$prefix .  $self->{javascript} . $postfix;
}

1;
