package Acme::HTML::KeepAlive;

=head1 NAME

Acme::Base - AcmeBase top level object

=head1 SYNOPSIS

	use Acme::HTML::KeepAlive;

	my $time_before_starting = 60;
	Acme::HTML::KeepAlive::keep_alive($time_before_starting);

=head1 DESCRIPTION

The keep_alive() function tricks the server into keeping a possibly very long
process alive by trickling characters to the browser periodically.
The argument is the initial time to wait before trickling.

=head1 CHANGE LOG

 3/6/03   Initial version.

=head1 AUTHOR

Mack Pexton <mack@acmebase.org>

=head1 COPYRIGHT

Copyright 2003, Mackley F. Pexton.  All rights reserved.

This is open software and is licensed under the terms of the GNU General Public License.

=cut

require 5.6.0;
use strict;

sub keep_alive {

	my $TrickleSec = 5;		# repeat interval in seconds
	my $handler = sub {
		print ".\n";
		alarm $TrickleSec;
	};

	$SIG{ALRM} = $handler;
	alarm (shift || $TrickleSec);		# set timer
}

1;
