package Acme::HTML::SimpleCalendar;

=head1 NAME

Acme::HTML::SimpleCalendar - HTML calendar

=head1 SYNOPSIS

    use Acme::HTML::SimpleCalendar;

    print Acme::HTML::SimpleCalendar->month(1,2000);
     or
    my $javascript_function = "open_diary";
    my $start_day = (localtime)[3];	# today's day
    my $compare = '>=';		# dates greater or equal to today get links
    print Acme::HTML::SimpleCalendar->month(1,2000,$javascript_function,$start_day,$compare);

=head1 DESCRIPTION

The month() method prints a calander. The days are links to a specified
JavaScript function. The arguments are month (1 - 12) and the year (yyyy).

A JavaScript callback function name can be specified. If it is, the dates
on the calendar become links to the JavaScript function.

Two other optional arguments are the day of month and a comparison operator
( <, <=, > >=, ==, != ). The default comparison operator is >=, so that only
those days greater than or equal to the specified day of month will be actual
links to the JavaScript callback function. Those days less than the specified
day are just text.

The JavaScript function is passed the date formatted like "mm-dd-yyyy".

=head1 NOTE

This module has been deemphasized in favor of building calendars on the client
with the JavaScript calendars.js.

=head1 CHANGE LOG

 11/14/98 Initial version
  9/29/00 Changed prcal() routine to Acme::HTML::SimpleCalendar->month() method.

=head1 AUTHOR

Mack Pexton <mack@acmebase.org>

=head1 COPYRIGHT

Copyright 1997-2000, Mackley F. Pexton.  All rights reserved.

This is open software and is licensed under the terms of the GNU General Public License.

=cut

require 5.000;
require Exporter;

@ISA = qw(Exporter);
@EXPORT_OK = qw(month);

sub month {
	my $self = shift;

	# Print html for calendar
	# Each date is a link to a JavaScript function calendar(date_string).

	local $m = shift;	# valid values are 1-12
	local $y = shift;	# 4 digit year like 1998
	local $callback = shift;# JavaScript callback function name
	local $mday = shift;	# optional day which those less are not links
	local $compare = shift;	# optional compare operator <, <=, > >=, ==, !=
				# to compare to $today, default is >=

	local $checkdate = "";	# routine optimization flag (see _daylink)
				# if not set, all days are links to $callback

	local $tabindex = $ENV{HTTP_USER_AGENT} =~ /MSIE/ ? -1 : 32767;

	if (defined $mday) {
		$compare = ">=" unless ($compare);	# default comparison
		$checkdate++;				# set flag
	}

	open CAL, "cal $m $y |";	# use unix utility

	my $s = "<pre style=\"margin-bottom:0pt\">";
	$s .= <CAL>;
	$s .= <CAL>;
	while (<CAL>) {
		if (length > 1) {
			if ($callback) {
				if ($checkdate) {
					s{(\d+)}
					 {_daylink($1)}ge;
				}
				else {
					s{(\d+)}
					 {<a href='javascript:void $callback("$m-$1-$y");' tabindex=$tabindex>$1</a>}go;
				}
			$s .= $_;
			}
		}
	}
	close CAL;
	chomp $s;	# remove trailing newlines
	return $s . "</pre>";
}

sub _daylink {
	# helper routine for month()
	# Returns argument or link text depending if the given day
	# compares with $mday (see month()).

	my $d = shift;

	return $d unless eval "$d $compare $mday";

	# HACK: set today's date to big and bold
	return qq(<a href='javascript:void $callback("$m-$d-$y");' tabindex=$tabindex><big><b>$d</b></big></a>) if eval "$d == $mday";

	return qq(<a href='javascript:void $callback("$m-$d-$y");' tabindex=$tabindex>$d</a>);
}

1;
