package Acme::HTTP;

=head1 NAME

Acme::HTTP - HTTP utility routines for Acme

=head1 SYNOPSIS

    use Acme::HTTP qw(escape	unescape);

    my $url = "http::/www.somewhere.com/cgi-bin/program.cgi?" .
      escape("program's arguments with weird @#% characters.");

    $url = unescape($url);

=head1 DESCRIPTION

Escape() and unescape() encode and decode urls. Usually,
CGI escape()/unescape() are used. This is included for those Acme programs
not using CGI or have not been converted yet.

Escape() returns the given string encoded as a safe HTTP string, suitable
for urls.

Unescape() returns the decoded string as encoded by escape();

Unlike other routines I have seen to encode/decode urls, these routines
handle multi-line strings.

=head1 CHANGE LOG

 3/20/1998 Initial version.

=head1 AUTHOR

Mack Pexton <mack@acmebase.org>

=head1 COPYRIGHT

Copyright 1998, Mackley F. Pexton.  All rights reserved.

This is open software and is licensed under the terms of the GNU General Public License.

=cut

require 5.000;
require Exporter;

@ISA = qw(Exporter);
@EXPORT_OK = qw(escape unescape);


#use CGI qw(escape unescape);
#
#sub escape   { return defined shift ? CGI::escape($_) : '' }
#sub unescape { return defined shift ? CGI::unescape($_) : '' }

#------------------------------------------------------------------------------

sub escape {
	my $s = shift;
	return '' unless defined($s);
## Cannot use pluses as spaces as Javascript unescape() does not remove them.
##	$s =~ tr/ /+/;		# change spaces to pluses 
	$s =~ s/([^\w.-])/uc sprintf("%%%02x",ord($1))/eg;
	return $s;
}

#------------------------------------------------------------------------------

sub unescape {
	my $s = shift;
	return '' unless defined($s);
	$s =~ tr/+/ /;		# change pluses to spaces
	$s =~ s/%([0-9A-Fa-f]{2})/pack("c",hex($1))/eg;
	return $s;
}

1;
