package Acme::Money;

=head1 NAME

Money - Format money

=head1 SYNOPSIS

    use Acme::Money qw(dollars dollars_us dollars_can commafy);

    print dollars($amount);
    print dollars($amount, $precision);

    print dollars_us($amount, $precision);
    print dollars_can($amount, $precision);

    print commafy($amount);
    print commafy($amount, $precision);

=head1 DESCRIPTION

Dollars() formats the amount to have a dollar sign and commas. The
number of digits after the decimal point can optionally be specified.

Dollars_us() formats amounts like dollars() with a "US" appended to the
amount. Dollars_can() formats amounts like dollars() with a "CAN" appended
to the amount.

Commafy() formats the amount to have commas (no dollar sign). The
number of digits after the decimal point can optionally be specified.


=head1 CHANGE LOG

 11/23/1998 Initial version.
 10/23/2002 Changed default precision to be 2 if it is not defined.

=head1 AUTHOR

Mack Pexton <mack@acmebase.org>

=head1 COPYRIGHT

Copyright 1998, Mackley F. Pexton.  All rights reserved.

This is open software and is licensed under the terms of the GNU General Public License.

=cut

require 5.6.0;
use strict;

require Exporter;

our @ISA = qw(Exporter);
our @EXPORT_OK = qw(dollars dollars_us dollars_can commafy);

sub dollars {	  return '$' . commafy(@_); }

sub dollars_us {  return '$' . commafy(@_) . 'US'; }

sub dollars_can { return '$' . commafy(@_) . 'CAN'; }

sub commafy {
	my ($dollars, $precision) = @_;
	$dollars ||= 0;
	$precision = 2 if ! defined $precision;

	return $dollars unless $dollars =~ /\d/;	# need to have a digit

        my $remainder = '';
        if ($dollars =~ s/\.(.*)$//) {  # remove cents and decimal
                $remainder = $1;        # save removed part for later
        }

	1 while $dollars =~ s/(\d)(\d\d\d)(?!\d)/$1,$2/;	# comma-fy

	if ($precision > 0) {
		# append 0's to $remainder
		$remainder .= "0" x ($precision - length($remainder));
		$dollars .= '.' . substr($remainder,0,$precision);
	}

	return $dollars;
}

1;
